angular.module('resources.fixturesResource', []);
angular.module('resources.fixturesResource').factory('FixtureDataModel', [
  'clubsService',
  'Urls',
  'ImageCacheFactory',
  function (clubsService, Urls, ImageCacheFactory) {
    
    var attachClubsDetails = function () {
      if (this.homeClubId) {
        this.homeClubDetails = clubsService.get(this.homeClubId);
      }
      if (this.awayClubId) {
        this.awayClubDetails = clubsService.get(this.awayClubId);
      }
    };
    var ninetyMinutes = 60000 * 90;
    // +60 minutes  
    var extraTime = 60000 * 120;
    // +120 minutes
    var initialize = function () {
      var startTime = new Date(this.startTime);
      this.startTime = startTime.getTime();
      this.endTime = startTime.getTime() + ninetyMinutes;
      this.endTimeExtraTime = startTime.getTime() + extraTime;
      this.st = new Date(this.startTime);
      
      this.refresh();
    };

    function refresh () {
      var imgUrlEnd = '';
      if (this.state == 'fulltime') {
        imgUrlEnd = 'Done';
      }
      var endUrl = 'sports/football/competitions/' + this.competition.optaId + '/styledCompetitionImage' + imgUrlEnd + '.png';
      this.imgUrl = Urls.urlBuilder(endUrl);      
      if (this.state === 'firsthalf' || this.state === 'halftime' || this.state === 'extrafirsthalf' || this.state === 'extrasecondhalf') {
        this.fixtureState = 'inProgress';
        this.inProgress = true;
      }
    }

    function getCurrentTeamId(club) {
      if(!club.primaryTeamId) {
        return false;
      }
      if(club.primaryTeamId == this.homeTeamId){
        return this.homeTeamId;
      }
      if(club.primaryTeamId == this.awayTeamId){
        return this.awayTeamId;
      }
      return false;
    }

    var fixtureDataModel = {
      attachClubsDetails: attachClubsDetails,
      initialize: initialize,
      $getCurrentTeamId: getCurrentTeamId,
      refresh: refresh
    };
    return {
      bind: function bindFunction(fixture, refresh) {
        fixture = angular.extend(fixture, fixtureDataModel);
        fixture.initialize();
      },
      unbind: function unbindFunction(fixture) {
        for (var method in fixtureDataModel) {
          delete fixture[method];
        }
        return fixture;
      }
    };
  }
]);
/**
* @name fixturesService
* @description Service to fetch fixtures from API
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.fixturesResource').service('fixturesService', [
  'DS',
  '$http',
  '$q',
  '$localForage',
  '$filter',
  'teamsService',
  'FixtureDataModel',
  'club',
  '$interval',
  'DataToolService',
  function fixturesService(DS, $http, $q, $localForage, $filter, teamsService, FixtureDataModel, club, $interval, DataToolService) {
    /**
    * Define resource for DS
    **/
    var fixtures = DS.defineResource({
      name: 'fixtures',
      deserialize: function (resourceName, response) {
        response = response.data._data;
        DataToolService.ifIsArray(response, [setupFixture]);

        // filter fixture results to get all 'prematch' fixturesList.state
        var preMatchFixtures = $filter('filter')(response, { state: 'prematch' });
        var orderByDatePreMatchFixtures = $filter('orderBy')(preMatchFixtures, '+startTime');
        if (orderByDatePreMatchFixtures && orderByDatePreMatchFixtures[0]) {
          orderByDatePreMatchFixtures[0].fixtureState = 'nextFixture';
          orderByDatePreMatchFixtures[0].isNextFixture = true;
          if (!fixtures.currentIndex) {
            fixtures.currentFixture = orderByDatePreMatchFixtures[0]; 
            fixtures.currentIndex = response.indexOf(orderByDatePreMatchFixtures[0]);
            startRefresh(orderByDatePreMatchFixtures[0]);

            // var now = new Date().getTime();
            // orderByDatePreMatchFixtures[0].state = 'firsthalf';
            // orderByDatePreMatchFixtures[0].startTime = now + 7000;
            // orderByDatePreMatchFixtures[0].refresh();
          }          
        }
        return response;
      }
    });
    var setupFixture = function(fixture) {
      FixtureDataModel.bind(fixture, this.refresh);
      //Add PlayerShirts   
      addHomeAwayGoalImages(fixture);
      var now = new Date().getTime();
      if (now < this.startTime) {
        $timeout(function() {
          this.startRefresh(fixture);
        }, fixture.startTime - now);
      }
    };

    function addHomeAwayGoalImages(fixture) {
      var teamIds = {
        home: fixture.homeTeamId,
        away: fixture.awayTeamId
      };
    
      if (club.clubId == fixture.homeClubId) {
        teamsService.addPlayerHeadShots(fixture.homeGoals, teamIds);
        teamsService.addPlayerShirts(fixture.awayGoals, fixture.awayTeamId);
      } else {
        teamsService.addPlayerHeadShots(fixture.awayGoals, teamIds);
        teamsService.addPlayerShirts(fixture.homeGoals, fixture.homeTeamId);
      }
    }

    fixtures.refresh = function(fixture) {
      fixtures.find(fixture.id, {bypassCache: true})
        .then(function(updatedFixture) {
          if (updatedFixture.fixtureState != 'inProgress') {
            $interval.cancel(stopRefersh);
        }
      });
    };

    var tenMinutes = 10 * 60 * 1000;
    var stopRefersh;
    var startRefresh = function(fixture) {
      if (stopRefersh) {
        $interval.cancel(stopRefersh);        
      }
      stopRefersh = $interval(function() {
        fixtures.refresh(fixture);        
      }, tenMinutes);
    };


    /**
    * Hack to stop adding daylight saving which increments time by hour if uk by clinet javascript
    **/
    fixtures.fixTimeForDaylight = function (timeStamp) {
      var dateTimeMatch = timeStamp.replace('T', ' ');
      dateTimeMatch = dateTimeMatch.replace('Z', ' ');
      return dateTimeMatch;
    };

    fixtures.getLocalForage = function () {
      return $http.get('app/resources/fixtures/data/fixtures.json');
    };
    fixtures.getLocal = function () {
      var localDataPromise = $q.defer();
      return $localForage.getItem('fixtures').then(function (localFixturesData) {
        if (localFixturesData && localFixturesData !== '') {
          localDataPromise.resolve(localFixturesData);
          return localDataPromise.promise;
        } else {
          localDataPromise.reject();
          return localDataPromise.promise;
        }
      }, function () {
        localDataPromise.reject();
        return localDataPromise.promise;
      });
    };
    function isLocalDataExpired(dataObject) {
      var notExpired = new Date(dataObject._meta.validUntil) < new Date();
      return !notExpired;
    }
    fixtures.searchAll = function (params) {
      // First get fixtures from local Store
      var localFixturesPromise = fixtures.getLocal();
      var localParams = params;
      // Now inject the fixtures into the DS data store
      // ** Remember this may be old data!
      // *** return this promise so that the application can 
      // continue to bootstrap
      localFixturesPromise.then(function (fixturesLocal) {
        var fixturesLocalData = fixturesLocal._data;
        for (var i = fixturesLocalData.length - 1; i >= 0; i--) {
          fixtures.inject(fixturesLocalData[i]);
        }
      });
      // end of local data concerns ...
      // Start the process of getting the remote data
      var remoteFixturesPromise = $q.defer();
      // on receipt of the local data we check if this data is
      // valid
      localFixturesPromise.then(function (localFixturesData) {
        if (isLocalDataExpired(localFixturesData.data)) {
          // if data is valid we simply return and resolve
          // the promise
          remoteFixturesPromise.resolve(localFixturesData.data._data);
        } else {
          // else we go off to the server and find fresh data
          fixtures.findAll(localParams).then(function (fixturesData) {
            remoteFixturesPromise.resolve(fixturesData);
          });
        }
      }, function () {
        fixtures.findAll(localParams).then(function (fixturesData) {
          remoteFixturesPromise.resolve(fixturesData);
        });
      });
      return {
        localPromise: localFixturesPromise,
        remotePromise: remoteFixturesPromise.promise
      };
    };
    return fixtures;
  }

]);

angular.module('resources.friendsResource', []);

angular.module('resources.friendsResource')
  .service('friendsService', FriendsService)

function FriendsService(DS, $rootScope, ASSET_PATH, Urls, DataToolService) {

  function setupFriend(friend) {
    var imageUrl  =  ASSET_PATH.USERS + friend.userId + '/100x100/profilepic.jpg';
    friend.imageUrl = Urls.urlRemoteBuilder(imageUrl);
    friend.avatarUrl = friend.imageUrl;
  }

  var friends = DS.defineResource({
    name: 'user/friends',
    endpoint: 'friends',
    relations: {
      belongsTo: {
        users: {
          parent: true,
          localField: 'user',
          localKey: 'userId'
        }
      }
    },
    // serialize: function(resourceName, data) {
    //   return {
    //     "token": $rootScope.token,
    //     "userId": $rootScope.user.id,
    //     "_request": data
    //   };
    // },
    deserialize: function(resourceName, response) {
      if (response.data && response.data._data) {
        var result = response.data._data;
        DataToolService.ifIsArray(result, [setupFriend]);
        return result;
      }
    }
  });

  friends.bindStats = function(stats) {
    var friend = this.filter({'userId': stats.userId});
    console.log(friend);
    for (var i = friend.length - 1; i >= 0; i--) {
      friend[i].stats = stats;
      friend[i].predictionAccuracy = stats.statistics.predictionAccuracy;
    };
  };

  return friends;
}

FriendsService.$inject = ['DS', '$rootScope', 'ASSET_PATH', 'Urls','DataToolService'];

angular.module('resources.fixtureStatisticsResource', []);
(function () {
  'use strict';
  /**
    * @name fixtureStatisticsService
    * @description Service to fetch fixture statistics from api
    * {link} http://angular-data.pseudobry.com/documentation/guide/angular-data-resource/relations understand relationships 
    * for how relations work in relations in DS.defineResource
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
  function fixtureStatisticsService(DS, $q) {
    var fixtureStatistics = DS.defineResource({
      name: 'fixtures/statistics',
      idAttribute: 'id',
      endpoint: 'statistics',
      deserialize: function (resourceName, response) {
        console.log(response);
        return response.data._data;
      },
      relations: {
        belongsTo: {
          fixtures: {
            parent: true,
            localField: 'fixture',
            localKey: 'fixtureId'
          }
        }
      }
    });
    return fixtureStatistics;
  }
  /**
    * Add service to resources.fixtureStatisticsResource Module
    **/
  angular.module('resources.fixtureStatisticsResource').service('fixtureStatisticsService', [
    'DS',
    '$q',
    fixtureStatisticsService
  ]);
}());
angular.module('resources.clubsResource', []);
  /**
    * @name clubsService
    * @description Service to fetch club resources from api
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
angular.module('resources.clubsResource').service('clubsService', [
  'DS',
  '$q',
  'club',
  'Urls',
  'ImageCacheFactory',
  'ExceptionService',
  function clubsServiceFunction(DS, $q, club, Urls, ImageCacheFactory, ExceptionService) {
    
    var shadeColor = function(color, percent) {
      var f=parseInt(color.slice(1),16),t=percent<0?0:255,p=percent<0?percent*-1:percent,R=f>>16,G=f>>8&0x00FF,B=f&0x0000FF;
      return "#"+(0x1000000+(Math.round((t-R)*p)+R)*0x10000+(Math.round((t-G)*p)+G)*0x100+(Math.round((t-B)*p)+B)).toString(16).slice(1);
    };
    
    var initClub = function(club) {
      club.imgUrl = buildClubLogoUrl(club.id);
    };
    /**
    * Define resource for DS
    **/
    var clubs = DS.defineResource({
      name: 'clubs',
      deserialize: function (resourceName, response) {
        var clubs = response.data._data;
        if (!_.isArray(clubs) && typeof clubs === 'object') {
          initClub(clubs);
        } else {
          for (var i = clubs.length - 1; i >= 0; i--) {
            initClub(clubs[i]);
          }
        }
        return clubs;
      }
    });
    function buildClubLogoUrl(clubId) {
      var pathEnd = 'sports/football/clubs/' + clubId + '/' + 'clubLogo.svg';
      return Urls.urlBuilder(pathEnd);
    }
    function getCurrentAppClub() {
      return clubs.get(club.id);
    }
    clubs.getStyle = function(club, homeAway) {
      var style = {};
      try {
        if (club && club.properties && club.properties.primaryColourHex) {
          var properties = club.properties[homeAway];
          style.modalBackground = {
            'background-color': properties && properties.predictModalBgColour ? properties.predictModalBgColour : club.properties.primaryColourHex            
          };
          style.lineupBackground = {
            'background-color': properties && properties.predictLineupBg ? properties.predictLineupBg : shadeColor(style.modalBackground['background-color'], -0.2)            
          };
          style.playerText = {
            'color': properties && properties.predictPlayerTextColour ? properties.predictPlayerTextColour : club.properties.contrastColourHex            
          };
          style.shirtNumberColor = {
            'color': properties && properties.shirtNumberColour ? properties.shirtNumberColour : club.properties.contrastColourHex            
          };
          style.statsBarColor = {
            'background-color': properties && properties.defaultStatsBarColour ? properties.defaultStatsBarColour : club.properties.primaryColourHex  
          };
          style.predictButtonBg = {
            'background-color': properties && properties.predictButtonBgColour ? properties.predictButtonBgColour : club.properties.primaryColourHex  
          };
          style.predictBorderRadius = {
            'border-color': properties && properties.predictBorderRadiusColour ? properties.predictBorderRadiusColour : club.properties.contrastColourHex  
          };
          style.fixtureText = {
            'color': properties && properties.predictFixtureTitleColour ? properties.predictFixtureTitleColour : club.properties.contrastColourHex 
          };
          return style;
        } else {
          return null;
        }
      } catch (err) {
        ExceptionService.notify({error: err});
        return {};
      }
    }
    return clubs;
  }
]);

angular.module('resources.teamsStatisticsResource', []);
(function () {
  'use strict';
  /**
    * @name teamsStatisticsService
    * @description Service to fetch team statistics from api
    * {link} http://angular-data.pseudobry.com/documentation/guide/angular-data-resource/relations understand relationships 
    * for how relations work in relations in DS.defineResource
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
  function teamsStatisticsService(DS, teamsService, teamsStatisticsDataModel) {
    var teamsStatistics = DS.defineResource({
      name: 'teams/statistics',
      idAttribute: 'id',
      endpoint: 'statistics',
      relations: {
        belongsTo: {
          teams: {
            // Will be assigned to url like /teams/t8/statistics/1
            // Or url like /teams/t8/statistics                        
            localField: 'teams',
            localKey: 'teamId',
            parent: true
          }
        }
      },
      deserialize: function (resourceName, response) {
        
        var teamsStats = response.data._data;
        if ( Object.prototype.toString.call( teamsStats ) === '[object Array]' ) {
          for (var i = teamsStats.length - 1; i >= 0; i--) {
            teamsStatisticsDataModel.bind(teamsStats[i]);
          };
        } else if (typeof teamsStats == 'object'){
          teamsStatisticsDataModel.bind(teamsStats);
        }

        return teamsStats;
      }
    });
    
    return teamsStatistics;
  }
  /**
    * Add service to resources.teamsStatisticsResource Module
    **/
  angular.module('resources.teamsStatisticsResource').service('teamsStatisticsService', [
    'DS',
    'teamsService',
    'teamsStatisticsDataModel',
    teamsStatisticsService
  ]);
}());

/**
* @name teamsStatisticsDataModelService
* @description Service to fetch predictions resources from api and POST, PUT, GET requests
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
* @requires {Object} $q Angularjs $q service
**/
angular
.module('resources.teamsStatisticsResource')
.factory('teamsStatisticsDataModel', [
  function() {
    function getTopGoalScorer() {
      var topScorer = this.players[this.players.length - 1];

      for (var i = this.players.length - 2; i >= 0; i--) {          
          if (this.players[i].totalGoals > topScorer.totalGoals) {
            topScorer = this.players[i].statistics[0];
          }
      }
      return topScorer;
    };

    function getAverageGoalsPerGame () {
      if ( this.statistics[0] && !_.isNumber(this.statistics[0].averageGoalsPerGame)) {
        return 0;
      }
      return Math.round(this.statistics[0].averageGoalsPerGame * 100) / 100;
    };

    function getTotalGoalsThisSeason () {
      if( this.statistics[0] && !_.isNumber(this.statistics[0].totalGoals)){
        return 0;
      }
      return this.statistics[0].totalGoals;
    };

    function getAverageGoalsPerGameBar (awayTeamGoals) {
      var homeTeamGoals = this.$getAverageGoalsPerGame();

      if (homeTeamGoals === 0 && awayTeamGoals === 0) {
        return '50%';
      }
      if (homeTeamGoals > 0 && awayTeamGoals === 0) {
        return '100%';
      }
      if (homeTeamGoals === 0 && awayTeamGoals > 0) {
        return '0%';
      }
      return (homeTeamGoals /(homeTeamGoals+awayTeamGoals))*100 + '%';
    };

    function getTotalGoalsBar (awayTeamGoals) {
      var homeTeamGoals = this.$getTotalGoalsThisSeason();
      if (homeTeamGoals === 0) {
        return '0%';
      }
      return (homeTeamGoals /(homeTeamGoals+awayTeamGoals))*100 + '%';
    };

    var teamsStatisticsDataModel = {
        $getTopGoalScorer:       getTopGoalScorer,
        $getTotalGoalsThisSeason: getTotalGoalsThisSeason, 
        $getAverageGoalsPerGame: getAverageGoalsPerGame,
        $getAverageGoalsPerGameBar: getAverageGoalsPerGameBar,
        $getTotalGoalsBar: getTotalGoalsBar
    };

    return {
      bind: function bindFunction(teamsStatistics) {
        teamsStatistics = angular.extend(
          teamsStatistics,
          teamsStatisticsDataModel
        );
      },
      unbind: function unbindFunction(teamsStatistics){
        for (var method in teamsStatisticsDataModel) {
            delete teamsStatistics[method];
        };
        return teamsStatistics;
      }
    };
  }
]);
  

angular.module('resources.teamsResource', []);
(function () {
  'use strict';
  /**
    * @name clubsService
    * @description Service to fetch club resources from api
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
  var teamsService = [
    'DS',
    '$q',
    'club',
    'Urls',
    'teamsDataModel',
    function teamsServiceFunction(DS, $q, club, Urls, teamsDataModel) {
        /**
        * Define resource for DS
        **/
        var positionsMap = {
            'Forward': 'F',
            'Midfielder': 'M',
            'Defender': 'D',
            'Goalkeeper': 'G'
        };

      var teams = DS.defineResource({
        name: 'teams',
        maxAge: 900000,
        deleteOnExpire: 'aggressive',
        deserialize: function (resourceName, response) {
          // data could be object or array
          var data = response.data._data;

          if (typeof data === 'object') {
            data.players = teams.sortPlayersByPosition(data.players);
            teamsDataModel.bind(data);
          }else if (typeof data === 'Array') {
            for (var i = data.length - 1; i >= 0; i--) {
              data[i].players = teams.sortPlayersByPosition(data[i].players);
              teamsDataModel.bind(data[i]);
            }
          }

          teams.addPlayerImages(data);

          return data;
        }
      });

      teams.sortPlayersByPosition = function(players){
        var playerTypes = ["Forward","Midfielder","Defender","Goalkeeper"];

        players = _.sortBy(players, function(player){return player.position == playerTypes[0] && player.realPosition == "Striker";});
        players = _.sortBy(players, function(player){return player.position == playerTypes[0] && player.realPosition != "Striker";});
        players = _.sortBy(players, function(player){return player.position == playerTypes[1] && player.realPosition == "Attacking Midfielder";});
        players = _.sortBy(players, function(player){return player.position == playerTypes[1] && player.realPosition != "Attacking Midfielder";});
        players = _.sortBy(players, function(player){return player.position == playerTypes[2];});
        players = _.sortBy(players, function(player){return player.position == playerTypes[3];});
        players = _.sortBy(players, function(player){
          return playerTypes.indexOf(player.position) === -1;
        });
        return players;
      };

      teams.addPlayerImages = function(data){
        var isCurrentClubTeam = false;
          if (typeof data === 'object') {

            if (data.clubId === club.clubId) {
              isCurrentClubTeam = true;
            }
            // Check if is team which app this is?
            // loop though players and add images
            if (isCurrentClubTeam) {
              teams.addPlayerHeadShots(data.players, data.id);
            } else {
              teams.addPlayerShirts(data.players, data.id);
            }

          } else if (typeof data === 'Array') {
            data.map(function(team) {
              var isCurrentClubTeam = false;

              if(team.clubId === club.clubId) {
                isCurrentClubTeam = true;
              }

              if(isCurrentClubTeam){
                teams.addPlayerHeadShots(team.players);
              } else {
                teams.addPlayerShirts(team.players, team.id);
              }
            });
          }
      };

      teams.availablePlayers = function availablePlayers(players, fixture) {
          var fixtureDate = new Date(fixture.startTime);
          fixtureDate = fixtureDate.getTime();
          var playerList = angular.copy(players);

          return playerList.filter(function(player) {
              //remove player if they leave before the fixture
              if(player.leaveDate) {
                  var playerLeaveDate = new Date(player.leaveDate);
                  playerLeaveDate = playerLeaveDate.getTime();

                  if(playerLeaveDate < fixtureDate) {
                      return false;
                  }
              }
              //filter out those not in this competition
              if(player.competitionIds) {
                  var competitions = player.competitionIds;
                  competitions.filter(function(competition, index) {
                      return competitions.predictablId === fixture.competitions.predictablId;
                  });
                  return competitions.length > 0;
              } else if (player.competitions) {
                  return player.competitions.indexOf(fixture.competition.optaId) !== -1;
              }
          });
      };

      teams.playerPositionShortCode = function playerPositionShortCode(position) {
          return positionsMap[position];
      }

      teams.addPlayerHeadShots = function(players, teamIds){
        var OWN_GOAL_TEXT = 'own';

        players.map(function(player) {
          var playerId = player.id || player.playerId;
          if (player.goalType === OWN_GOAL_TEXT) {
            player.isHeadShot = false;
            player.imgUrl = buildTeamShirtImgUrl(teamIds.away);
          } else {
            player.isHeadShot = true;

            Urls.setUrl(player, 'imgUrl', buildPlayerImgUrl(playerId), [
              buildPlayerImgUrl(teamIds.home),
              buildPlayerImgUrl('default-player')
            ]);
          }
        });
      };

      teams.addPlayerShirts = function(players, teamId) {
        var teamShirtUrl = buildTeamShirtImgUrl(teamId);

        Urls.asyncImageExist(teamShirtUrl)
        .then(function() {
          players.map(function(player) {
            player.isHeadShot = false;
            player.imgUrl = teamShirtUrl;
          });
        }, function() {
          var defaultTeamShirtUrl = buildTeamShirtImgUrl('default-team');
          players.map(function(player) {
            player.isHeadShot = false;
            player.imgUrl = defaultTeamShirtUrl;
          });
        });
      };

      function buildPlayerImgUrl(playerId) {
        var customUrl = 'sports/football/players/' + playerId + '/headShot.png';
        return Urls.urlBuilder(customUrl);
      };

      function buildTeamShirtImgUrl(teamId) {
        var customUrl = 'sports/football/teams/' + teamId + '/headShot.svg';
        return Urls.urlBuilder(customUrl);
      };
      function buildPlayerActionImgUrl(playerId) {
        var customUrl = 'sports/football/players/' + playerId + '/actionShot.jpg';
        return Urls.urlBuilder(customUrl);
      };
      teams.getPlayerById = function(id){
        var allTeams = this.getAll();
        for (var i = allTeams.length - 1; i >= 0; i--) {
          for (var j = allTeams[i].players.length - 1; j >= 0; j--) {
            if(allTeams[i].players[j].id == id){
              return allTeams[i].players[j];
            }
          };
        };
      }

      return teams;
    }
  ];
  /**
    * Add service to teamsResource Module
    **/
  angular.module('resources.teamsResource').service('teamsService', teamsService);
}());

/**
* @name teamsDataModelService
* @description Service to fetch predictions resources from api and POST, PUT, GET requests
**/
angular.module('resources.teamsResource')
.factory('teamsDataModel', [
  function teamsDataModelFunction() {   
      var teamsDataModel = {
          $getPlayerById: function(id) {
            for (var i = this.players.length - 1; i >= 0; i--) {
                if (this.players[i].id === id) {
                  return this.players[i];
                }
            }
            return false;
          }
      };

      return {
        bind: function bindFunction(teams) {
          teams = angular.extend(
            teams,
            teamsDataModel
          );
        },
        unbind: function unbindFunction(teams) {
          for (var method in teamsDataModel) {
              delete teams[method];
          }
          return teams;
        }
      };
    }
]);

(function () {
  'use strict';
  
  // resource definition
  angular
    .module('resources.userChallengesResource', []);

}());
/**
* @name usersChallengeService
* @description Service to fetch challenges for specific users from api
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.userChallengesResource')
.service('userChallengesService', [
  'DS',
  'Urls',
  function(DS, Urls) {

    var userChallenges = DS.defineResource({

      name: 'user/challenges',
      idAttribute: 'id',
      endpoint: 'challenges',
      
      relations: {
        belongsTo: {
          users: {
            parent: true,
            localField: 'user',
            localKey: 'userId'
          }
        }
      },
      deserialize: function (resourceName, response) {

        var userChallengesResponse = response.data._data;
        
        if( _.isObject(userChallengesResponse) && userChallengesResponse.aroundMe && userChallengesResponse.aroundMe.length){

          for (var i = userChallengesResponse.aroundMe.length - 1; i >= 0; i--) {
            var imgUrl = Urls.urlRemoteBuilder('users/'+ userChallengesResponse.aroundMe[i].userId + '/100x100/profilepic.jpg');
            var fallback = Urls.urlRemoteBuilder('users/default-user/100x100/profilepic.jpg');
            Urls.setUrl(userChallengesResponse.aroundMe[i], 'imgUrl', imgUrl, fallback);
          }

        }

        if(_.isArray(userChallengesResponse)){
          for (var i = userChallengesResponse.length - 1; i >= 0; i--) {
            if(userChallengesResponse[i].aroundMe && userChallengesResponse[i].aroundMe.length){
              for (var j = userChallengesResponse[i].aroundMe.length - 1; j >= 0; j--) {

                var imgUrl = Urls.urlRemoteBuilder('users/'+ userChallengesResponse[i].aroundMe[j].userId + '/100x100/profilepic.jpg');
                var fallback = Urls.urlRemoteBuilder('users/default-user/100x100/profilepic.jpg');
                Urls.setUrl(userChallengesResponse[i].aroundMe[j], 'imgUrl', imgUrl, fallback);

              }
            }
          }
        }
        return userChallengesResponse;
      }
    });

    return userChallenges;

  }
]);

(function () {
  'use strict';
  /**
	* productsResource module init
	**/
  angular.module('resources.productsResource', []);
}());
/**
* @name productsService
* @description Service to fetch products from API
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.productsResource')
.service('productsService', [
  'DS',
  'club',
  'Urls',
  '$filter',
  'BackgroundService',
  'PurchasePluginService',
  function productsService(DS, club, Urls, $filter, BackgroundService, PurchasePluginService) {
    /**
    * Define resource for DS
    **/
    var products = DS.defineResource({
      name: 'products',
      deserialize: function (resourceName, response) {
        var products = _.sortBy(response.data._data, 'totalPrice');

		products.map(function(product, index){
			switch(product.type) {
				case 'backgrounds':
					return backgroundProductDeserialiser(product);
				case 'avatars':
					return avatarProductDeserialiser(product);
				default:
					return defaultProductDeserialiser(product);
			}
		});

        var balls = _.where(products, {type: 'balls'});
        PurchasePluginService.init(balls);
        return products;
      }
    });
    return products;

	//////////////////////////////////////////
	Urls.urlRemoteBuilder('coins/freeBalls.png');
	function backgroundProductDeserialiser(product) {
		var endUrl = 'products/' + product.id + '/android/' + product.productImage;
		var RESIZER_URL = 'https://engine.playabl.com/api/resizer?path=';
		product.productImage = Urls.urlRemoteBuilder(endUrl);
		product.thumbnail = Urls.cache(RESIZER_URL + endUrl + '&width=200');
		if (product.productImageIsLocal) {
		  product.thumbnail = BackgroundService.defaultBackground;
		  product.productImage = BackgroundService.defaultBackground;
		  product.numberOfItems = 0;
		}
		return product;
	}

	function defaultProductDeserialiser(product) {
		product.totalPrice = product.totalPrice / 100;
		product.productImage = Urls.urlRemoteBuilder(product.productImage);
		return product;
	}

	function avatarProductDeserialiser(product) {
		var clubUrl = 'sports/football/clubs/' + club.clubId + '/avatar/';
		product.productImage =
			Urls.urlRemoteBuilder(clubUrl + product.productImage);
		return product;
	}
  }
]);

/**
* productsResource module init
**/
angular.module('resources.endorsementsResource', []);

/**
* @name endorsementService
* @description Service to fetch endorsement from API
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.endorsementsResource').service('endorsementsService', [
  'DS',
  'Urls',
  'ImageCacheFactory',
  function (DS, Urls, ImageCacheFactory) {
    /***
    * Define resource for DS
    **/
    var endorsements = DS.defineResource({
      name: 'promotions',
      deserialize: function (resourceName, response) {
        var promotions = response.data._data;
        var urls = [];
        for (var i = promotions.length - 1; i >= 0; i--) {
          if (typeof promotions[i] === 'object') {
            promotions[i].mainImageUrl = Urls.urlRemoteBuilder(promotions[i].mainImageUrl);
            urls.push(promotions[i].mainImageUrl);
          }
        }
        return promotions;
      }
    });
    return endorsements;
  }
]);
(function () {
  'use strict';
  /**
	* usersServiceResource module init
	**/
  angular.module('resources.usersResource', []);
}());
angular.module('resources.usersResource').factory('UsersDataModel', [
  'Urls',
  'club',
  'ASSET_PATH',
  function (Urls, club, ASSET_PATH) {

    var initialize = function() {
      if (this.facebook && this.facebook.id) {
        this.facebookId = this.facebook.id;
      }
      if (this.country) {
        this.flagImageUrl = Urls.urlRemoteBuilder('countries/' + this.country.toLowerCase() + '/flags/28x14/main.png');        
      }

        for (var i = this.clubLoyalties.length - 1; i >= 0; i--) {
          if (this.clubLoyalties[i].clubId === club.clubId) {
            this.currentClubLoyalty = this.clubLoyalties[i];
            if (this.currentClubLoyalty.nextLevelXP > this.currentClubLoyalty.previousLevelXP) {
              this.currentClubLoyalty.xpPercent = (this.currentClubLoyalty.xp - this.currentClubLoyalty.previousLevelXP) * 100 / (this.currentClubLoyalty.nextLevelXP - this.currentClubLoyalty.previousLevelXP);
            }
          }
        }

        if (!this.avatarUrl) {
            var avatarUrl  = !this.facebook.id ?
                ASSET_PATH.CLUBS + club.clubId + '/defaultUser.svg' :
                ASSET_PATH.USERS + this.id + '/100x100/profilepic.jpg';

            this.avatarUrl = Urls.urlRemoteBuilder(avatarUrl);
        }

        this.level = this.currentClubLoyalty.level;
        this.levelString = this.getLevelString();
    };
    var getLevelString = function() {
        var lvl = Math.floor(this.currentClubLoyalty.level / 5) * 5;
        if (lvl === 0) {
          lvl = 1;
        }
        return 'level' + lvl;
    };

    var userDataModel = {
      getLevelString: getLevelString,
      initialize: initialize
    };
    return {
      bind: function bindFunction(user) {
        user = angular.extend(user, userDataModel);
        user.initialize();
        return user;
      },
      unbind: function unbindFunction(fixture) {
        for (var method in userDataModel) {
          delete fixture[method];
        }
        return fixture;
      }
    };
  }
]);


/**
* @name usersService
* @description Service to fetch users resources from api
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.usersResource').service('usersService', [
  'DS',
  'UsersDataModel',
  '$rootScope',
  function (DS, UsersDataModel, $rootScope) {
    /**
    * Define resource for DS
    **/
    var users = DS.defineResource({
      name: 'users',
      relations: {
        hasMany: {
          products: {
            localField: 'products',
            foreignKey: 'userId'
          },
          friends: {
            localField: 'friends',
            foreignKey: 'userId'
          }
        }
      },
      deserialize: function (resourceName, res) {
        var user = UsersDataModel.bind(res.data._data);
        return user;
      }
    });

    users.updateCurrentUser = function(){
      if($rootScope.user && $rootScope.user.id){
        users.find($rootScope.user.id, {bypassCache:true});
      }
    };

    return users;
  }
]);

(function () {
  'use strict';
  
  // resource definition
  angular
    .module('resources.userStatisticsResource', []);

}());
/**
* @name usersChallengeService
* @description Service to fetch challenges for specific users from api
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resources.userStatisticsResource')
.service('userStatisticsService', [
  'DS',
  'Urls',
  'club',
  'DataToolService',
  function(DS, Urls, club, DataToolService) {

    var getLevelString = function(level) {
        var lvl = Math.floor(level / 5) * 5;
        if (lvl === 0) {
          lvl = 1;
        }
        return 'level' + lvl;
    };

    var setupUserStats = function(userStat) {
      console.log(userStat);
	    var DEFAULT_COUNTRY = 'GB';

      // for (var i = userStat.clubLoyaltyStatistics.length - 1; i >= 0; i--) {
        // if (userStat.clubLoyaltyStatistics[i].clubId === club.clubId) {
      userStat.clubStats = userStat.statistics;
      if (!userStat.avatarUrl) {
        userStat.avatarUrl = userStat.statistics.avatarUrl;
      }
      userStat.displayName = userStat.statistics.displayName;
      userStat.country = userStat.statistics.country || DEFAULT_COUNTRY;
      userStat.level = userStat.clubStats.maxLevel;
      userStat.levelString = getLevelString(userStat.level);
      userStat.imageUrl = Urls.urlRemoteBuilder('users/'+userStat.userId+'/100x100/profilepic.jpg');
      userStat.flagImageUrl = Urls.urlRemoteBuilder('countries/' + userStat.country.toLowerCase() + '/flags/28x14/main.png');
        // }
      // };
    }

    var userStatistics = DS.defineResource({

      name: 'user/statistics',
      idAttribute: 'userId',
      endpoint: 'statistics',

      relations: {
        belongsTo: {
          users: {
            parent: true,
            localField: 'user',
            localKey: 'userId'
          }
        }
      },
      deserialize: function (resourceName, response) {
        var result = response.data._data;
        DataToolService.ifIsArray(result, [setupUserStats]);
        console.log('deserialize');
        if (result instanceof Array) {
          return result
        } else {
          return [ result ];
        }
      }
    });

    return userStatistics;

  }
]);


angular.module('resources.challengesResource', []);
(function () {
  'use strict';
  /**
    * @name challengesService
    * @description Service to fetch challenges resources from api
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
  function challengesService(DS, $q, DataToolService, Urls) {
    /**
        * Define resource for DS
        **/
    var challenges = DS.defineResource({
      name: 'challenges',
      deserialize: function (resourceName, response) {
        var challenges = response.data._data;
        DataToolService.ifIsArray(challenges, [setupChallenge]);
        for (var i = 0; i < challenges.length; i++) {
            if (challenges[i] == null) {
                challenges.splice(i, 1);
                i--;
            }
        };
        return challenges;
      }
    });

    var getImageForView = function (challenge, url, bg, promo, promoter, source) {
        var challengePrizeImage;
        if (promo) {
          url = 'challenges/' + challenge.id + '/' + url;
        } else {
          url = 'sports/football/clubs/' + challenge.clubId + '/challenges/' + url;
        }
        if (source === 'remote') {
          url = Urls.urlRemoteBuilder(url);
        } else {
          url = Urls.urlBuilder(url);
        }
        if (bg) {
          return 'url(' + url + ')';
        }
        return url;
    };


    var setupChallenge = function(challenge) {
        if (challenge.promoter && (challenge.promoter == 'inReview' || challenge.promoter.indexOf('inReview') != -1) ) {
            challenge.isPublished = false;
            return;
        }
        if(challenge.membersCurrent == 1 || challenge.membersCurrent == 2) {
            challenge.isCreateButton = false;
        }

        if (challenge.isRegional) {
            challenge.isNational = true;
        }

        if (challenge.membersMax == 2) {
            challenge.isFanVFan = true;
            challenge.isGlobal = false;
        } else {
            challenge.isFanVFan = false;
        }

        if (challenge.isPromo == true) {
            if(challenge.prizeImageUrl) {
                challenge.prizeImageUrl = Urls.urlRemoteBuilder(challenge.prizeImageUrl);
            }
            if(challenge.backgroundImageUrl) {
                challenge.backgroundImageUrl = Urls.urlRemoteBuilder(challenge.backgroundImageUrl);
            }
            if(challenge.logoImageUrl) {
                challenge.logoImageUrl = Urls.urlRemoteBuilder(challenge.logoImageUrl);
            }
        }

        if (challenge.challengeType == 'fb-fvf') {
            challenge.backgroundImageUrl = "fvf/" + challenge.stake + "/challenge.jpg";
            challenge.logoImageUrl = "fbChallengeLogo.png";
        } else if (challenge.isFanVFan) {
            
            if(challenge.stake == 100){
                challenge.backgroundImageUrl = 'fanChallengeBronze.jpg';
                challenge.logoImageUrl = 'versusChallengeLogo.png';
            }

            if(challenge.stake == 500){
                challenge.backgroundImageUrl = 'fanChallengeSilver.jpg';
                challenge.logoImageUrl = 'versusChallengeLogo.png';
            }

            if(challenge.stake == 1000){
                challenge.backgroundImageUrl = 'fanChallengeGold.jpg';
                challenge.logoImageUrl = 'versusChallengeLogo.png';
            }

        };

        if(challenge.winners && challenge.winners.length){
            for (var cw = challenge.winners.length - 1; cw >= 0; cw--) {
              challenge.winners[cw].imgUrl = Urls.urlRemoteBuilder('users/'+ challenge.winners[cw].userId + '/100x100/profilepic.jpg');

              Urls.setUrl(challenge.winners[cw], 'imgUrl', challenge.winners[cw].imgUrl, Urls.urlRemoteBuilder('users/default-user/100x100/profilepic.jpg'));
            };
        }


        ///////////
        var challengeStyle = {};
        var challengeLogo = '';
        if (challenge.isFanVFan) {
            challengeStyle['background-image'] = 'url(' + 'images/' + challenge.backgroundImageUrl + ')';
            challengeLogo = 'images/' + challenge.logoImageUrl;
        } else if (challenge.isGlobal && !challenge.isPromo) {
            challengeStyle['background-image'] = getImageForView(challenge, 'global/challenge.png', true);
            challengeLogo = getImageForView(challenge, 'global/logo.png');
        } else if (challenge.isNational && !challenge.isPromo) {
            challengeStyle['background-image'] = getImageForView(challenge, 'national/challenge.png', true);
            challengeLogo = getImageForView(challenge, 'national/logo.png');
        } else if (challenge.isPromo) {
            var idx = challenge.backgroundImageUrl.indexOf("/images/");
            idx += 8;
            var img = challenge.backgroundImageUrl.substring(idx);
            challenge.resizedImage = Urls.cache('https://engine.playabl.com/api/resizer?path=' + img + '&width=500');
            challengeStyle['background-image'] = 'url(' + challenge.resizedImage + ')';
            challengeLogo = challenge.logoImageUrl;
        } else {
            challengeStyle['background-image'] = getImageForView(challenge, 'user/challenge.png', true);
            challengeLogo = '';
        }

        if (challenge.prizeType === 'balls') {
            challenge.prizeImageUrl = 'images/balls.png';
        } else if (challenge.prizeType === 'ticket') {
            challenge.prizeImageUrl = 'images/ticket.png';
        }

        challenge.style = challengeStyle;
        challenge.logo = challengeLogo;

    }

    challenges.fanVFan = [{
        "backgroundImageUrl": "fanChallengeGold.jpg",
        "challengeType": "fanVfan",
        "id": "1000",
        "isCreateButton":true,
        "isFanVFan":true,
        "isAuto": false,
        "isGlobal": false,
        "isNational": false,
        "isPromo": false,
        "isPublic": true,
        "isPublished": true,
        "isRegional": false,
        "isSettled": false,
        "logoImageUrl": "versusChallengeLogo.png",
        "membersCurrent": 0,
        "membersMax": 2,
        "name": "Fan v Fan - 1000",
        "prizeName": "balls",
        "shareText": "I am set to win 1000 balls in a fan v fan challenge",
        "stake": 1000,
        "losingPosition": "",
        "winningPosition": "Play one vs one against a real fan and win double your winnings! The more you put down, the more you can win back!",
        "style": {
            "background-image": 'url(images/fanChallengeGold.jpg)'
        },
        "logo": 'images/versusChallengeLogo.png',
        "prizeImageUrl": 'images/balls.png'
      },{
        "backgroundImageUrl": "fanChallengeSilver.jpg",
        "challengeType": "fanVfan",
        "id": "500",
        "isCreateButton":true,
        "isFanVFan":true,
        "isAuto": false,
        "isGlobal": false,
        "isNational": false,
        "isPromo": false,
        "isPublic": true,
        "isPublished": true,
        "isRegional": false,
        "isSettled": false,
        "logoImageUrl": "versusChallengeLogo.png",
        "membersCurrent": 0,
        "membersMax": 2,
        "name": "Fan v Fan - 500",
        "prizeName": "balls",
        "shareText": "I am set to win 500 balls in a fan v fan challenge",
        "stake": 500,
        "losingPosition": "",
        "winningPosition": "Play one vs one against a real fan and win double your winnings! The more you put down, the more you can win back!",
        "style": {
            "background-image": 'url(images/fanChallengeSilver.jpg)'
        },
        "logo": 'images/versusChallengeLogo.png',
        "prizeImageUrl": 'images/balls.png'
      },{
        "backgroundImageUrl": "fanChallengeBronze.jpg",
        "challengeType": "fanVfan",
        "id": "100",
        "isCreateButton":true,
        "isFanVFan":true,
        "isAuto": false,
        "isGlobal": false,
        "isNational": false,
        "isPromo": false,
        "isPublic": true,
        "isPublished": true,
        "isRegional": false,
        "isSettled": false,
        "logoImageUrl": "versusChallengeLogo.png",
        "membersCurrent": 0,
        "membersMax": 2,
        "name": "Fan v Fan - 100",
        "prizeName": "balls",
        "shareText": "I am set to win 100 balls in a fan v fan challenge",
        "stake": 100,
        "losingPosition": "",
        "winningPosition": "Play one vs one against a real fan and win double your winnings! The more you put down, the more you can win back!",
        "style": {
            "background-image": 'url(images/fanChallengeBronze.jpg)'
        },
        "logo": 'images/versusChallengeLogo.png',
        "prizeImageUrl": 'images/balls.png'
    }];

    challenges.addFanVFan = function(fixtures) {
        // We don't want FanVFan anymore, as we have Facebook challenges
        return;
        
        for (var i = fixtures.length - 1; i >= 0; i--) {
            var fixtureChallenges = challenges.filter({
                where: {
                    fixtureId:fixtures[i].id
                }
            });

            var addGold = true, addSilver = true, addBronze = true;
            for (var j = fixtureChallenges.length - 1; j >= 0; j--) {
                if(fixtureChallenges[j].isFanVFan && fixtureChallenges[j].challengeType == 'fvf' && !fixtureChallenges[j].isCreateButton){
                    if(fixtureChallenges[j].stake == 1000){
                        addGold = false;
                    }
                    if(fixtureChallenges[j].stake == 500) {
                        addSilver = false;
                    }
                    if(fixtureChallenges[j].stake == 100) {
                        addBronze = false;
                    }
                }
            };

            if (fixtures[i].state == 'prematch') {
                if (addBronze) {
                    challenges.fanVFan[2]["fixtureId"] = fixtures[i].id;
                    challenges.fanVFan[2]["id"] = 'bronze' + '-' + fixtures[i].id;
                    challenges.inject(challenges.fanVFan[2]);
                } else {
                    // remove fanvfan
                    var bronzeChallenges = challenges.filter({
                        where:{
                            fixtureId: fixtures[i].id,
                            isFanVFan: true,
                            stake: 100,
                            isCreateButton:true
                        }
                    });
                    for (var j = bronzeChallenges.length - 1; j >= 0; j--) {
                        challenges.eject(bronzeChallenges[j].id);
                    };
                }
                if (addSilver) {
                    challenges.fanVFan[1]["fixtureId"] = fixtures[i].id;
                    challenges.fanVFan[1]["id"] = 'sliver' + '-' + fixtures[i].id;
                    challenges.inject(challenges.fanVFan[1]);
                } else {
                    // remove fanvfan
                    var silverChallenges = challenges.filter({
                        where:{
                            fixtureId: fixtures[i].id,
                            isFanVFan: true,
                            stake: 500,
                            isCreateButton:true
                        }
                    });
                    for (var j = silverChallenges.length - 1; j >= 0; j--) {
                        challenges.eject(silverChallenges[j].id);
                    };
                }
                if (addGold) {
                    challenges.fanVFan[0]["fixtureId"] = fixtures[i].id;
                    challenges.fanVFan[0]["id"] = 'gold' + '-' + fixtures[i].id;
                    challenges.inject(challenges.fanVFan[0]);
                } else {
                    var goldChallenges = challenges.filter({
                        where:{
                            fixtureId: fixtures[i].id,
                            isFanVFan: true,
                            stake: 1000,
                            isCreateButton:true
                        }
                    });
                    for (var j = goldChallenges.length - 1; j >= 0; j--) {
                        challenges.eject(goldChallenges[j].id);
                    };
                }                
            }

        };
    }
    
    return challenges;
  }
  /**
    * Add service to challengesResource Module
    **/
  angular.module('resources.challengesResource').service('challengesService', [
    'DS',
    '$q',
    'DataToolService',
    'Urls',
    challengesService
  ]);
}());

angular.module('resources.challengeMembersResource', [])
.constant('challengeMemberResourceConstants', {
    PRIORITY:           2,
    MAX_BULK_REQUESTS:  30,
    MEMBER: {
        challangeId:    null,
        userId:         null
    }
});


angular.module('resources.challengeMembersResource')
.service('challengeMembersService', [
  'Urls',
  'DataToolService',
  'DS',
  function(Urls, DataToolService, DS) {
      
    var members = DS.defineResource({
      name: 'challenge/members',
      idAttribute: 'id',
      endpoint: 'members',
      relations: {
        belongsTo: {
          challenges: {
            parent: true,
            localField: 'challenge',
            localKey: 'challengeId'
          }
        }
      },
      deserialize: function (resourceName, response) {
        var challengeMembers = response.data._data;
        DataToolService.ifIsArray(challengeMembers, [addUserProfileImages, shortenName]);
        return challengeMembers;
      }
    });

    function addUserProfileImages(member) {
      if (member.avatarUrl) {
        member.imgUrl = member.avatarUrl;
      } else {
        member.imgUrl = Urls.urlRemoteBuilder('users/'+member.userId+'/100x100/profilepic.jpg');        
      }
      //Urls.setUrl(member, 'imgUrl', Urls.urlRemoteBuilder('users/'+member.userId+'/100x100/profilepic.jpg'), Urls.urlRemoteBuilder('users/default-user/100x100/profilepic.jpg'));
    };

    function shortenName(member, nameLength) {
      if(!nameLength){
        nameLength = 8;
      }
      member.name = member.displayName;
      member.displayName = member.displayName.split(' ')[0]; 
      if (member.displayName.length > nameLength) {
        member.displayName = member.displayName.substr(0,nameLength) + '...';
      }
    };
    
    return members;
  }
]);
angular.module('resource.documentsResource', []);

/**
* @name documentsService
* @description Service to fetch documents from API
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resource.documentsResource').service('documentsService', [
  'DS',
  function(DS) {
    /**
    * Define resource for DS
    **/
    var documents = DS.defineResource({
      name: 'documents',
      deserialize: function (resourceName, response) {
        return response.data._data;
      }
    });
    return documents;
  }
]);

angular.module('resources.enquiriesResource', []);
  angular.module('resources.enquiriesResource').service('enquiriesService', [
    'DS',
    function(DS) {
      /**
      * Define resource for DS
      **/
      var enquiries = DS.defineResource({
        name: 'enquiries',
        deserialize: function (resourceName, response) {
          return response.data._data;
        }
      });
      return enquiries;
    }
]);

angular.module('resource.countriesResource', []);
/**
* @name countries
* @description Service to fetch countries resources from api
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('resource.countriesResource').service('countriesService', [
  'DS',
  function (DS) {
    /**
    * Define resource for DS
    **/
    var countries = DS.defineResource({
      name: 'countries',
      endpoint: '/locations/countries',
      idAttribute: 'code',
      deserialize: function (resourceName, response) {
        return response.data._data;
      }
    });
    return countries;
  }
]);

(function () {
  'use strict';
  /**
	* predictionsResource module init
	**/
  angular
  	.module('resources.predictionsResource', [])
  	.constant('predictionResourceConstants', {

        // making few requests does not work so we
        // have needed to make sure we request EVERYthing
        // that we need at once... do not set this to less than 15
        //MAX_BULK_PREDICTIONS: 30,

        PREDICTION: {
            challengeId:    "",
            committed:      false,
            fixtureId:      "",
            homeTeamScore:  "+",
            awayTeamScore:  "+",
            homeGoals:      [],
            awayGoals:      [],
            clubId:         ""
        },

        SCORER: {
            // set in the service
            order:          1,      // goal order which goal was scored first
            // set in the service
            overallOrder:   1,
            // set in the service
            team:           "home",
            // set in the service
            teamId:         "",
            // set in the service
            playerId:       "",
            firstName:      "",
            lastName:       "",
            displayName:    "",
            goalType:       "goal", // (goal/own/penalty)
            period:         ""      //  (firsthalf/secondhalf/extrafirsthalf/extrasecondhalf/shootout)
        }

    });
}());
(function () {
  'use strict';
  /**
    * @name predictionsService
    * @description Service to fetch predictions resources from api and POST, PUT, GET requests
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/
  var predictionsService = [
    'DS',
    '$q',
    '$filter',
    'predictionResourceConstants',
    'predictionsDataModel',
    'teamsService',
    'fixturesService',
    function predictionsServiceFunction(DS, $q, $filter, P, predictionsDataModel, teamsService, fixturesService) {
      
      /**
      * Define resource for DS
      **/

      function createTotalGoalResults(prediction){
            if(!prediction.userResults || !_.isArray(prediction.userResults.goalResults)){
                  return;
            }
            prediction.userResults["totalGoalResults"] = {
                  "desc": {
                    "name": "totalGoalResults",
                    "shortDesc": "The total of all of your goal results"
                  },
                  "xp": 0,
                  "maxXP": 0,
                  "points": 0,
                  "maxPoints": 0
            };

            var goalResults = prediction.userResults.goalResults;

            for (var i = goalResults.length - 1; i >= 0; i--) {
                  prediction.userResults.totalGoalResults.xp += goalResults[i].total.xp;
                  prediction.userResults.totalGoalResults.maxXP += goalResults[i].total.maxXP;
                  prediction.userResults.totalGoalResults.points += goalResults[i].total.points;
                  //ServerError: prediction.userResults.totalGoalResults.maxPoints += goalResults[i].total.maxPoints;
            };
      };

      var setupPrediction = function(prediction) {
        if (!prediction.homeTeamId || !prediction.awayTeamId) {
          var fixture = fixturesService.get(prediction.fixtureId);
          prediction.homeTeamId = fixture.homeTeamId;
          prediction.awayTeamId = fixture.awayTeamId;
        }
          createTotalGoalResults(prediction);
          predictionsDataModel.bind(prediction);
      };

      var predictions = DS.defineResource({
        
        name: 'predictions',
        idAttribute: 'id',

        deserialize: function (resourceName, response) {
          var predictions = response.data._data;
          if (_.isArray(predictions)){
            for (var i = predictions.length - 1; i >= 0; i--) {
              setupPrediction(predictions[i]);
            };
          } else if(_.isObject(predictions)){
            setupPrediction(predictions);
          }

          return predictions;
        },
        beforeCreate: function(resourceName, prediction, cb) {
          predictionsDataModel.nillAllGoalsIfNeeded(prediction);
          predictionsDataModel.unbind(prediction);
          cb(null, prediction);
        },
        beforeSave: function(resourceName, prediction, cb) {
          predictionsDataModel.nillAllGoalsIfNeeded(prediction);
          predictionsDataModel.unbind(prediction);
          cb(null, prediction);
        }
      });
      predictions.setScorers = function(prediction) {
        var homePlayers = teamsService.get(prediction.homeTeamId).players;
        var awayPlayers = teamsService.get(prediction.awayTeamId).players;
        predictions.setScorersData(prediction.homeGoals, homePlayers);
        predictions.setScorersData(prediction.awayGoals, awayPlayers);
      };
      predictions.getPredictionByIds = function(config) {
        
        var prediction = predictions.filter({
          'fixtureId':config.fixtureId,
          'challengeId':config.challengeId
        });

        if(!prediction.length){
          prediction = angular.copy(P.PREDICTION);
          prediction.committed = true;
          predictionsDataModel.bind(prediction);
          
          prediction.challengeId = config.challengeId;
          prediction.clubId = config.clubId;
          prediction.fixtureId = config.fixtureId;
          prediction.homeTeamId = config.homeTeamId;
          prediction.awayTeamId = config.awayTeamId;

          return prediction;
          // inject into 
          // localPredictions.inject(prediction);
        }

        // need to fix asap ... 
        return prediction[0];
      }


      predictions.setScorersData = function(predictionScorers, teamPlayers) {
            
            for (var s = predictionScorers.length - 1; s >= 0; s--) {
                  for (var p = teamPlayers.length - 1; p >= 0; p--) {

                        if (predictionScorers[s].playerId == teamPlayers[p].id) {
                              predictionScorers[s] = angular.extend(
                                    angular.copy(teamPlayers[p]),
                                    predictionScorers[s]
                              );
                        }
                  }
            }
      };

      /**
      * @name predictions.findPredictionByFixtureId
      * @params {String} fixtureId (systemId of fixture)
      * @return {Promise|array} 
      **/
      predictions.findPredictionByFixtureId = function(fixtureId){
        return predictions.find(
          {'fixtureId':fixtureId}
        );
      }


      return predictions;
    }
  ];
  
  /**
  * Add service to resources.predictionsResource Module
  **/
  angular
    .module('resources.predictionsResource')
    .service('predictionsService', predictionsService);

}());
(function () {
  'use strict';
  /**
    * @name predictionsService
    * @description Service to fetch predictions resources from api and POST, PUT, GET requests
    * @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
    * @requires {Object} $q Angularjs $q service
    **/

  var predictionsDataModel = [
    
    'DS',
    '$q',
    '$filter',
    'predictionResourceConstants',
    
    function predictionsDataModelFunction(DS, $q, $filter, P) {
      
      function addHomeGoal(scorerObject){
        // get an empty scorer and create a new one
        var newScorer = angular.copy(P.SCORER);

        // set some defaults which may be overloaded by the controller
        newScorer.goalType = "goal";
        newScorer.overallOrder = getMaxOveralOrder(this) + 1;
        newScorer.playerId = scorerObject.id;

        // add the controller properties
        angular.extend(newScorer, scorerObject);
        newScorer.team = "home";

        // set order and Overall order 
        newScorer.order = this.homeGoals.length + 1;

        this.homeGoals.push(newScorer);
        this.homeTeamScore = _addGoal(this.homeTeamScore);
      }

      function addAwayGoal(scorerObject){
        // get an empty scorer and create a new one
        var newScorer = angular.copy(P.SCORER);

        // set some defaults which may be overloaded by the controller
        newScorer.goalType = "goal";
        newScorer.overallOrder = getMaxOveralOrder(this) + 1;
        newScorer.playerId = scorerObject.id;

        // add the controller properties
        angular.extend(newScorer, scorerObject);
        newScorer.team = "away";

        // set order and Overall order 
        newScorer.order = this.awayGoals.length + 1;

        this.awayGoals.push(newScorer);
        this.awayTeamScore = _addGoal(this.awayTeamScore);
      }
      function addGoal(team, scorerObject) {
        if (team == 'home') {
          this.$addHomeGoal(scorerObject);
        } else {
          this.$addAwayGoal(scorerObject);
        }
      }

      function _addGoal(goalCount){
        if(goalCount == '+'){
          return 1;
        }
        console.log(goalCount);
        return goalCount+1;
      }
      
      function _removeGoal(goalCount){
        if(goalCount == 0 || goalCount == '+'){
          return 0;
        }
        console.log(goalCount);
        return goalCount-1;
      }
      
      function getMaxOveralOrder(prediction) {
        return prediction.awayGoals.length + prediction.homeGoals.length;
      }

      
      function resetOverallOrder(prediction){
        var overallOrder = 0;
        for (var i = prediction.awayGoals.length - 1; i >= 0; i--) {
          overallOrder++;
          prediction.awayGoals[i].overallOrder = overallOrder;
        };
        for (var j = prediction.homeGoals.length - 1; j >= 0; j--) {
          overallOrder++;
          prediction.homeGoals[j].overallOrder = overallOrder;
        };
      }

      function removeAwayGoal(goalOrder){
        
        // we can't remove goals that dont exist
        if(!this.awayGoals.length){
          return;
        }

        // setup indexes for finding goal to remove
        var index = null;
        var newOrder = 1;

        // look for the right item and reset the orders (one loop)
        for (var i = 0;  i < this.awayGoals.length; i++) {
          if(this.awayGoals[i].order == goalOrder) {
            index = i;
          } else {
            this.awayGoals[i].order = newOrder;
            newOrder++;
          }
        };

        //remove it from the array
        this.awayGoals.splice(index,1);

        // reset overall order
        resetOverallOrder(this);

        // set the score
        this.awayTeamScore = _removeGoal(this.awayTeamScore);
      }

      function removeHomeGoal(goalOrder){

        // we can't remove goals that dont exist
        if(!this.homeGoals.length){
          return;
        }

        // setup indexes for finding goal to remove
        var index = null;
        var newOrder = 1;

        // look for the right item and reset the orders (one loop)
        for (var i = 0;  i < this.homeGoals.length; i++) {
          if(this.homeGoals[i].order == goalOrder){
            index = i;
          }else{
            this.homeGoals[i].order = newOrder;
            newOrder++;
          }
        };

        //remove it from the array
        this.homeGoals.splice(index,1);

        // reset overall order
        resetOverallOrder(this);

        // remove the goal score
        this.homeTeamScore = _removeGoal(this.homeTeamScore);
      }

      function removeGoal(team, goalOrder) {
        if (team == 'home') {
          this.$removeHomeGoal(goalOrder);
        } else {
          this.$removeAwayGoal(goalOrder);
        }
      };

      function nilGoals(team) {
        if (team == 'home') {
          this.homeGoals = [];
          this.homeTeamScore = 0;
        } else {
          this.awayGoals = [];
          this.awayTeamScore = 0;
        }
      }

      function nillAllGoalsIfNeeded(prediction){
        if(prediction.homeTeamScore == '+'){
          prediction.$nilGoals('home');
        }
        if(prediction.awayTeamScore == '+'){
          prediction.$nilGoals('away');
        }
      }

      function getTeamScore(team) {
        if (team == 'home') {
          return this.homeTeamScore == '+' ? 0 : this.homeTeamScore;
        } else {
          return this.awayTeamScore == '+' ? 0 : this.awayTeamScore;
        }
      };

      function revert(){
        this.homeTeamScore = this.$originalPrediction.homeTeamScore;
        this.awayTeamScore = this.$originalPrediction.awayTeamScore;
        this.homeGoals = angular.copy(this.$originalPrediction.homeGoals);
        this.awayGoals = angular.copy(this.$originalPrediction.awayGoals);
      }

      var predictionsDataModel = {

        $addGoal:     addGoal,
        $addAwayGoal: addAwayGoal,
        $addHomeGoal: addHomeGoal,

        $removeGoal:  removeGoal,
        $removeHomeGoal: removeHomeGoal,
        $removeAwayGoal: removeAwayGoal,

        $nilGoals:    nilGoals,

        $getTeamScore: getTeamScore,

        $revert:            revert,

      };

      return {
        nillAllGoalsIfNeeded: nillAllGoalsIfNeeded,

        bind: function bindFunction(prediction){

          prediction.$originalPrediction = angular.copy(
            prediction
          );

          prediction = angular.extend(
            prediction,
            predictionsDataModel
          );

        },
        unbind: function unbindFunction(prediction){
          if(prediction["$originalPrediction"]){
            delete prediction["$originalPrediction"];
          }
          for(var method in predictionsDataModel) {
              delete prediction[method];
          };
          return prediction;
        }
      };
    }

  ];
  
  /**
  * Add service to resources.predictionsResource Module
  **/
  angular
    .module('resources.predictionsResource')
    .factory('predictionsDataModel', predictionsDataModel);

}());

angular.module('resources.activities', []);

angular.module('resources.activities').service('activitiesService', [
  'DS',
  function (DS) {
    /**
    * Define resource for DS
    **/
    var activities = DS.defineResource({
      name: 'activities',
      endpoint: '/activities',
      deserialize: function (resourceName, response) {
        return response.data._data;
      }
    });
    return activities;
  }
]);


angular.module('resources.userActivities', []);

angular.module('resources.userActivities').service('userActivitiesService', [
  'DS',
  'DataToolService',
  function (DS, DataToolService) {
    var setupUserActivity = function(data) {
      data.totalNumberToday = data.numberRemainingToday + data.numberCompletedToday;
    };
    /**
    * Define resource for DS
    **/
    var userActivities = DS.defineResource({
      name: 'users/Activities',
      idAttribute: 'id',
      endpoint: 'activities',
      relations: {
        belongsTo: {
          users: {
            parent: true,
            localKey: 'userId'
          }
        }
      },    
      serialize: function(resourceName, data) {
        return {
          userId: data.userId,
          _request: data
        };
      },
      deserialize: function (resourceName, response) {
        DataToolService.ifIsArray(response.data._data, [setupUserActivity]);
        return response.data._data;
      }
    });
    return userActivities;
  }
]);
