'use strict';
angular.module('shell', [
  'ionic',
  'ngCordova',
  'angular-data.DS',
  // DATA RESOURCES
  'resources.fixturesResource',
  'resources.fixtureStatisticsResource',
  'resources.clubsResource',
  'resources.teamsResource',
  'resources.teamsStatisticsResource',
  'resources.userChallengesResource',
  'resources.challengesResource',
  'resources.challengeMembersResource',
  'resources.productsResource',
  'resources.endorsementsResource',
  'resources.friendsResource',
  'resources.usersResource',
  'resources.userStatisticsResource',
  'resources.predictionsResource',
  'resources.enquiriesResource',
  'resources.activities',
  'resources.userActivities',
  // GENERATED CONFIG MODULE
  'shell.gen_constants',
  'shell.gen_languages',
  'shell.gen_faq',
  // SHELL MODULES
  'shell.constants',
  'shell.services',
  'shell.controllers',
  'shell.directives',
  'shell.filters',
  // BLADES MODULES
  'blades.fixture',
  'blades.predict',
  'blades.shop',
  'blades.user',
  'blades.challenge',
  'blades.endorsement',
  'blades.settings',
  'blades.authentification',
  'blades.share',
  'blades.help',
  'blades.matchReport',
  'blades.newsFeed',
  // OTHER MODULES
  'ngTouchend',
  'ngTouchstart',
  'paching',
  'angular-cache',
  'oc.lazyLoad',
  'pascalprecht.translate',
  'ngStorage',
  'ngResource',
  'iso-3166-country-codes',
  'facebook',
  'LocalForageModule',
  'timer',
  'akoenig.deckgrid',
    // Fix iOS9 infdig https://github.com/angular/angular.js/issues/12241
  'ngIOS9UIWebViewPatch'
]).run([
  '$rootScope',
  '$timeout',
  '$q',
  'club',
  'Urls',
  'AppBuilder',
  'AuthService',
  '$state',
  '$ionicHistory',
  '$ionicPlatform',
  'PushNotificationPluginService',
  'NetworkStateService',
  'SupersonicAdsPluginService',
  'DeepLinkService',
  'AppsflyerPluginService',
  function ($rootScope, $timeout, $q, club, Urls, AppBuilder, AuthService, $state, $ionicHistory, $ionicPlatform, PushNotificationPluginService, NetworkStateService, SupersonicAdsPluginService, DeepLinkService, AppsflyerPluginService) {
    console.log('App running: ' + club.clubId);
    $rootScope.isWebView = ionic.Platform.isWebView();
    $rootScope.hideLayout = true;
    $rootScope.canGoBack = false;
    $rootScope.hideFooter = false;
    $rootScope.hideHeader = true;
    $rootScope.showMainLogo = true;
    $rootScope.showMatchReport = false;
    $rootScope.appContainer = {};

    document.addEventListener("deviceready", function() {
      $rootScope.appContainer['min-height'] = window.innerHeight + 'px';
      PushNotificationPluginService.initialize();
      SupersonicAdsPluginService.preload();      
      DeepLinkService.initialize();
      AppsflyerPluginService.initialize();
      if (window.analytics) {        
        window.analytics.startTrackerWithId(club.gAnalyticsTrackerId);
      }
    }, false);
    
    $rootScope.scorePredictorLogo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/scorePredictorLogo.png');
    
    // Try to autoLogin, using the stored token 
    AuthService.autoLogin();
    
    /***
    * Build the app, get main settings and load blades. On success, 
    * 'build.succeeded' event is emitted on the $rootScope,
    * on fail, 'build.failed' event is emmited on the $rootScope.
    */
    AppBuilder.build();

    $rootScope.registeredMenu = AppBuilder.registeredMenu;
    /***
    * StateChange interceptor, called before any state changes
    */
    $rootScope.$on('$stateChangeStart', function (event, toState, toStateParam, fromState) {
      console.log('stateChangeStart: ' + toState.name + ' (' + new Date().getTime()  + ')' + $rootScope.stateChange);
      if ( (toState.name == 'shell.login' || toState.name == 'shell.register') && $rootScope.user) {
        event.preventDefault();
        return;
      }
      $rootScope.stateChange = false;
      if (AppBuilder.isBuilt === false && toState.name !== 'splash') {
        event.preventDefault();
        $state.go('splash');
      } else if (toState.requiresLogin === true && AuthService.isAuthenticated() === false) {
        // prevent from changing state
        event.preventDefault();
        $state.go('shell.login');
      }
    });
    $rootScope.$on('$stateChangeSuccess', function (evt, toState) {
      console.log('stateChangeSuccess: ' + toState.name + ' (' + new Date().getTime()  + ')' + $rootScope.stateChange);
      // this is used to hide the menus on spefic state, defined in the stateProvider
      $rootScope.stateChange = true;
      $timeout(function() {
        $rootScope.hideLayout = !!toState.hideLayout;
        $rootScope.canGoBack = !toState.isRootView;
        $rootScope.showMainLogo = !!toState.showMainLogo;
        $rootScope.showMatchReport = !!toState.showMatchReport;
        $rootScope.forceGoBack = !!toState.forceGoBack;
        if ($rootScope.hideLayout) {
          $rootScope.mainContentStyle = {
            'z-index': 10
          };
        } else {
          $rootScope.mainContentStyle = {
            'z-index': 1
          };
        }
      });
    });
    // Hardware back button
    $ionicPlatform.ready(function() { 
      $ionicPlatform.registerBackButtonAction(function(e){
        if ($state.is('shell.fixtures')) {
          if (confirm('Are you sure you want to Exit?')) {
              ionic.Platform.exitApp();
              return false;
          } else {
              e.preventDefault();
              return false;
          }
        } else {
          $rootScope.goBack();          
        }
      }, 101);
    });
    // set our custom back button function
    $rootScope.goBack = function (backCount) {
      $ionicHistory.goBack(backCount);
    };
  }
]);
/***
* SHELL MODULES
*/
angular.module('shell.services', []);
angular.module('shell.constants', ['pascalprecht.translate']);
angular.module('shell.controllers', []);
angular.module('shell.directives', []);
angular.module('shell.filters', []);

'use strict';
angular.module('shell').config([
  'DSProvider',
  '$ionicConfigProvider',
  '$urlRouterProvider',
  '$stateProvider',
  'UrlsProvider',
  '$httpProvider',
  '$translateProvider',
  'pachingConfigProvider',
  'clubProvider',
  function (DSProvider, $ionicConfigProvider, $urlRouterProvider, $stateProvider, UrlsProvider, $httpProvider, $translateProvider, pachingConfigProvider, clubProvider) {
    // main app state
    $stateProvider.state('shell', {
      url: '/',
      abstract: true,
      templateUrl:'app/shell/templates/layout.html'
    }).state('splash', {
      url: 'splash',
      requiresLogin: false,
      canGoBack: false,
      isRootView: true,
      hideLayout: true,
      hideHeader: true,
      templateUrl: 'app/shell/templates/splash.html',
      controller: 'SplashController'
    });
    // if no state, go main. (default route)
    $urlRouterProvider.otherwise('shell');
    
    // confugre the default Base URL for angular data
    //UrlsProvider.setApiUrl('https://stagingengine.playabl.com/api/');
    UrlsProvider.setApiUrl('https://engine.playabl.com/api');
    var cdnImageURL = 'https://playabllive.blob.core.windows.net/images/';
    UrlsProvider.setRemoteImageUrl(cdnImageURL);
    if(window.cordova){
      UrlsProvider.setLocalImageUrl('images/cdn/');
    } else {
      UrlsProvider.setLocalImageUrl(cdnImageURL);
    }

    // http://angular-translate.github.io/docs/#/guide/19_security
    $translateProvider.useSanitizeValueStrategy('escape');
    //$translateProvider.useMissingTranslationHandlerLog();

    /** 
    /** Global angular data functions **/
    /***********************************/

    // serialize
    DSProvider.defaults.serialize = function(resourceName, data) {
      return {_request: data};
    };

    // deserialize
    DSProvider.defaults.deserialize = function(resourceName, response) {
      return response.data._data;
    };

    DSProvider.defaults.baseUrl = UrlsProvider.apiUrl;

    $ionicConfigProvider.views.forwardCache(true);
    /**
    * Disable iosSwipe to go back feature
    */ 
    $ionicConfigProvider.views.swipeBackEnabled(false);

    /** 
    * Remove slide transitions: Games have cool transitions of their own! 
    */
    $ionicConfigProvider.views.transition('none');
    /**
    * Enable native scrolling, (which allow scrolling on your web browser)
    * Setting this to false has the same effect as setting each ion-content to have overflow-scroll='true'.
    * http://ionicframework.com/docs/api/provider/$ionicConfigProvider/
    */
    $ionicConfigProvider.scrolling.jsScrolling(false);
    /**
    * This interceptor inject the language code into all the request we make to our api
    */
    $httpProvider.interceptors.push('LanguageHttpInterceptor');

    /**
    * Configure Paching for managing apps
    **/
    pachingConfigProvider.configure({
        token: clubProvider.club.pachingAppKey,
        timeout:1000,
        threshold:1,
        domain: "pcg.io",
        protocol:"http"
      });
  }
]);
'use strict';
angular.module('shell.constants').constant('DIR', {});
angular.module('shell.constants')
    .constant('ASSET_PATH', {
            'CLUBS': 'sports/football/clubs/',
            'USERS': 'users/',
            'DEFAULT_USER_IMAGE': 'defaultUser.svg',
            'FACEBOOK_IMAGE': '/100x100/profilepic.jpg'
    });

angular.module('shell.constants')
  .constant('COUNTRY_FLAG_RESOLVER', {
    'ja': 'jp',
    'ns': 'cn',
    'ko': 'kr'
  });

/**
 * ==================  angular-ios9-uiwebview.patch.js v1.1.1 ==================
 *
 * This patch works around iOS9 UIWebView regression that causes infinite digest
 * errors in Angular.
 *
 * The patch can be applied to Angular 1.2.0 – 1.4.5. Newer versions of Angular
 * have the workaround baked in.
 *
 * To apply this patch load/bundle this file with your application and add a
 * dependency on the "ngIOS9UIWebViewPatch" module to your main app module.
 *
 * For example:
 *
 * ```
 * angular.module('myApp', ['ngRoute'])`
 * ```
 *
 * becomes
 *
 * ```
 * angular.module('myApp', ['ngRoute', 'ngIOS9UIWebViewPatch'])
 * ```
 *
 *
 * More info:
 * - https://openradar.appspot.com/22186109
 * - https://github.com/angular/angular.js/issues/12241
 * - https://github.com/driftyco/ionic/issues/4082
 *
 *
 * @license AngularJS
 * (c) 2010-2015 Google, Inc. http://angularjs.org
 * License: MIT
 */

angular.module('ngIOS9UIWebViewPatch', ['ng']).config(['$provide', function($provide) {
  'use strict';

  $provide.decorator('$browser', ['$delegate', '$window', function($delegate, $window) {

    if (isIOS9UIWebView($window.navigator.userAgent)) {
      return applyIOS9Shim($delegate);
    }

    return $delegate;

    function isIOS9UIWebView(userAgent) {
      return /(iPhone|iPad|iPod).* OS 9_\d/.test(userAgent) && !/Version\/9\./.test(userAgent);
    }

    function applyIOS9Shim(browser) {
      var pendingLocationUrl = null;
      var originalUrlFn= browser.url;

      browser.url = function() {
        if (arguments.length) {
          pendingLocationUrl = arguments[0];
          return originalUrlFn.apply(browser, arguments);
        }

        return pendingLocationUrl || originalUrlFn.apply(browser, arguments);
      };

      window.addEventListener('popstate', clearPendingLocationUrl, false);
      window.addEventListener('hashchange', clearPendingLocationUrl, false);

      function clearPendingLocationUrl() {
        pendingLocationUrl = null;
      }

      return browser;
    }
  }]);
}]);
angular.module('shell.constants').constant('DEFAULT_BLADES', {
  payment: {},
  fixture: {
    numberOfFutureFixtures: 3,
    numberOfPastFixtures: 2
  },
  howToPlay: {},
  challenge: {}
});

angular.module('shell.services').provider('club', [
	'clubCst',
	function(clubCst) {
		if (window.currentClub) {
  			this.club =  window.currentClub;
	  	} else {
		  	this.club = clubCst;
	  	}
		this.$get = function() {
			if (window.currentClub) {
	  			return window.currentClub;
		  	} else {
			  	return clubCst;
		  	}
		}
  	}
]);
(function(angular, undefined) {
  angular.module("shell.gen_constants", [])

.constant("clubCst", {
	"clubId": "0f7e9e3e-236e-431a-8b54-240402cd25c0",
	"subdomain": "wolves",
	"fbAppId": "143265769342570",
	"languageCode": "en",
	"gAnalyticsTrackerId": "UA-66153548-2",
	"batchKeys": {
		"android": {
			"batch_dev_api_key": "DEV55DC3A49E3DE6C4AE905B78618F",
			"batch_api_key": "55DC3A49DFC2E1B3A9E50F1F93A5E2"
		},
		"ios": {
			"batch_dev_api_key": "DEV55F828C1BC528E68DCC8508D239",
			"batch_api_key": "55F828C1BB761390C57A69E20651C7"
		}
	},
	"gcmId": "90116444685",
	"debugApp": false,
	"appVersion": "2.4",
	"supersonic": {
		"android": "40f48dcd",
		"ios": "40f41abd"
	},
	"chineseVersion": false,
	"appsflyerKey": {
		"android": "dbxyZzM8xxePE2RQyLhGRd",
		"ios": "dbxyZzM8xxePE2RQyLhGRd"
	},
	"itunesAppId": "1025336814",
	"branchKey": "202439272811221404",
	"pachingAppKey": "igtqt2hh"
})

;
})(angular);
angular.module("shell.gen_languages", [])

.constant("languages", {
	"ja": {
		"name": "ja",
		"displayName": "日本語",
		"path": "app/config/generated/languages/ja.js"
	},
	"pt-pt": {
		"name": "pt-pt",
		"displayName": "Português (Portugal)",
		"path": "app/config/generated/languages/pt-pt.js"
	},
	"en-gb": {
		"name": "en-gb",
		"displayName": "English (UK)",
		"path": "app/config/generated/languages/en-gb.js"
	},
	"it-it": {
		"name": "it-it",
		"displayName": "Italiano",
		"path": "app/config/generated/languages/it-it.js"
	},
	"fr-fr": {
		"name": "fr-fr",
		"displayName": "Français (France)",
		"path": "app/config/generated/languages/fr-fr.js"
	},
	"ko": {
		"name": "ko",
		"displayName": "한국어",
		"path": "app/config/generated/languages/ko.js"
	},
	"es-es": {
		"name": "es-es",
		"displayName": "Español (España)",
		"path": "app/config/generated/languages/es-es.js"
	},
	"id": {
		"name": "id",
		"displayName": "Indonesia",
		"path": "app/config/generated/languages/id.js"
	},
	"th": {
		"name": "th",
		"displayName": "ภาษาไทย",
		"path": "app/config/generated/languages/th.js"
	},
	"zh-hans": {
		"name": "zh-hans",
		"displayName": "中文(简体)",
		"path": "app/config/generated/languages/zh-hans.js"
	},
	"pt-br": {
		"name": "pt-br",
		"displayName": "Português (Brasil)",
		"path": "app/config/generated/languages/pt-br.js"
	}
})

;
angular.module("shell.gen_faq", [])

.constant("faq", [
	{
		"answer": "Yes - Tap on the \"shop\" icon in the top left hand corner and follow the steps to buying more golden balls.",
		"question": "Can I buy more golden balls?"
	},
	{
		"answer": "Yes, go to predictabl.com and follow the links.",
		"question": "Can I find more about the game online?"
	},
	{
		"answer": "Yes, however some of them may require you to hand over your precious golden balls to unlock them, or they may be too far in the future for the to be available yet. Please be patient, they'll be available soon!",
		"question": "Can I predict on all the fixtures?"
	},
	{
		"answer": "You can save your predictions by logging in. Don't forget you have to be registered to be able to log in.",
		"question": "Can I save my predictions?"
	},
	{
		"answer": "Unfortunately not - this does mean that other fans can't see your predictions either (unless you choose to share them!).",
		"question": "Can I see other fan's predictions?"
	},
	{
		"answer": "Yes you can. Tap a fixture and then tap on the info icon - there are also fan stats there to help you predict more accurately.",
		"question": "Can I see the stats and results for the previous fixtures between the two clubs?"
	},
	{
		"answer": "You can share your prediction by tapping on the share icon whenever it is available. This will allow you to share on all your available social networks.",
		"question": "Can I share my prediction?"
	},
	{
		"answer": "You can win tickets by entering and winning challenges that have tickets as a prize. You can then spend them to enter competitions to win the top prizes!",
		"question": "How do I get tickets and what do I do with them?"
	},
	{
		"question": "How do I know if I have won?",
		"answer": "You will receive an email that will let you know. You can also tap on a past challenge to see how you did."
	},
	{
		"answer": "Tap a fixture, tap a challenge, add players that you think will score – don’t forget to put them in the order that you think they will score.",
		"question": "How do I predict what I think the score will be?"
	},
	{
		"question": "How do I win prizes?",
		"answer": "You win prizes by winning a challenge. Predict in a challenge and get more points than anyone else entered into that challenge… Simple."
	},
	{
		"answer": "Tap a past fixture and you will see a list of challenges (there will be a notification number on the challenges that you have entered). Click on the challenge to see how many points you got for that challenge.",
		"question": "How many points did I get?"
	},
	{
		"question": "Where can I see the results of a challenge?",
		"answer": "Tap a past fixture and you will see a list of challenges. Click on the challenge to see the results."
	}
])

;
/**
* @ngdoc services
* @name Settings
* @description gather all settings in one place. The settings are loaded in the app.js
* @property {Fixture} fixture Array of fixtures settings
*/
angular.module('shell.services').factory('Settings', [
  '$q',
  '$rootScope',
  'club',
  'LanguageService',
  'fixturesService',
  'challengesService',
  'challengeMembersService',
  'productsService',
  'predictionsService',
  'endorsementsService',
  'teamsService',
  'clubsService',
  'usersService',
  'userChallengesService',
  'UserSettings',
  'userProducts',
  'DEFAULT_BLADES',
  'activitiesService',
  'userActivitiesService',
  function ($q, $rootScope, club, LanguageService, fixturesService, challengesService, challengeMembersService, productsService, predictionsService, endorsementService, teamsService, clubsService, usersService, userChallengesService, UserSettings, userProducts, DEFAULT_BLADES, activitiesService, userActivitiesService) {
    var Settings = {
      loading: false,
      fixtures: [],
      fixtureIdsString: '',
      clubIds: '',
      lang: ''
    };

    var loadFixturesData = function(fixtures) {
      var q = $q.defer();
        Settings.fixtures = fixtures;
        // create an array of fixture id
        var uids = [];
        var fixtureIds = [];
        for (var idx = fixtures.length - 1; idx >= 0; idx--) {
          var fixture = fixtures[idx];
          // we want to make sure there is no duplicata
          if (uids.indexOf(fixture.homeClubId) === -1) {
            uids.push(fixture.homeClubId);
          }
          if (uids.indexOf(fixture.awayClubId) === -1) {
            uids.push(fixture.awayClubId);
          }
          fixtureIds.push(fixture.id);
        }
        Settings.fixtureIdsString += fixtureIds.join(',');
        Settings.clubIds += uids.join(',');
        $q.all([
          clubsService.findAll({'ids': Settings.clubIds}),
          productsService.findAll({'clubId': club.clubId}),
          challengesService.findAll({'clubId': club.clubId, 'fixtureIds': Settings.fixtureIdsString}),
          endorsementService.findAll({'clubId': club.clubId})
        ]).then(function () {
          // attach the clubs details to the fixture
          for (var i = Settings.fixtures.length - 1; i >= 0; i--) {
            Settings.fixtures[i].attachClubsDetails();
          }
          if ($rootScope.user) {
            Settings.loadUserData();            
          }
          q.resolve();
        }, q.reject);
      return q.promise;
    };

    Settings.getMore = function() {
      var q = $q.defer();
      var allFixtures = fixturesService.getAll();
      var offset = allFixtures.length * -1;
      return fixturesService.searchAll({
        'homeOrAwayClub': club.clubId,
        'includeBefore': 0,
        'includeAfter': 5,
        'offset': offset
      }).remotePromise.then(function(fixtures) {
        loadFixturesData(fixtures).then(q.resolve, q.reject);
      }, q.reject);
      return q.promise;
    };

    var getFixtures = function () {
      var q = $q.defer();
      var fixturesPromises = fixturesService.searchAll({
        'homeOrAwayClub': club.clubId,
        'includeBefore': DEFAULT_BLADES.fixture.numberOfPastFixtures,
        'includeAfter': DEFAULT_BLADES.fixture.numberOfFutureFixtures
      });
      fixturesPromises.remotePromise.then(function (fixtures) {
        loadFixturesData(fixtures).then(q.resolve, q.reject);
      }, q.reject);
      return q.promise;
    };
    /***
    * fetch again the challenges, with the current user's token
    */
    Settings.loadUserData = function() {
      userProducts.findAll({ userId: $rootScope.user.id, 'clubId': club.clubId });
      $q.all([          
        activitiesService.findAll({ 'clubId': club.clubId }),
        userActivitiesService.findAll({'userId': $rootScope.user.id, 'clubId': club.clubId }),
      ]).finally(function() {
        $rootScope.$broadcast('user.activities.ready');
      });

      userChallengesService.findAll({ 
        'userId': $rootScope.user.id,
        'clubId': club.clubId, 
        'fixtureIds': Settings.fixtureIdsString 
      }).then(function getUserChallenges(userChallenges) {
          var challengeIds = userChallenges.map(function(userChallenge) { 
            return userChallenge.id;               
          });
          if (challengeIds.length) {
            challengesService.findAll({ 'clubId': club.clubId, 'ids': challengeIds.join(',') })
            .then(function () {
              challengesService.addFanVFan(fixturesService.getAll());
              var FanVFanChallenges = challengesService.filter({isFanVFan:true});
              for (var i = FanVFanChallenges.length - 1; i >= 0; i--) {
                if (FanVFanChallenges[i].membersCurrent > 0) {
                  challengeMembersService.findAll({challengeId:FanVFanChallenges[i].id});
                }
              }
            });
          } else {
            challengesService.addFanVFan(fixturesService.getAll());
          }
        });      
      if (Settings.fixtureIdsString) {
        predictionsService.findAll({ 'fixtureIds': Settings.fixtureIdsString });
      }
    };
    $rootScope.$on('user.logged.in', function () {
      if (Settings.loading === false) {
        Settings.loadUserData();
      }
    });

    /***
    * Remove User data
    */
    Settings.clear = function(all, storeReload) {
      //Ejecting the productsService throws an iTUnes login modal
      var clearProducts = storeReload !== undefined ? storeReload : true;

      if (all) {
        // change language
        endorsementService.ejectAll();
        teamsService.ejectAll();
      }

      //if (clearProducts) {
        productsService.ejectAll();
      //}
      // login
      challengesService.ejectAll();
      userProducts.ejectAll();
      userChallengesService.ejectAll();
      predictionsService.ejectAll();
    };

    /**
    * load
    * @memberOf Settings
    * @description Load blades & fixtures settings
    * @return promise, reject with an error, empty resolve
    */
    Settings.load = function () {
      var q = $q.defer();
      Settings.loading = true;
      UserSettings.init();
      LanguageService.init().then(function() {
        getFixtures().then(function() {
          Settings.loading = false;
          q.resolve();
        }, function(err) {
          Settings.loading = false;
          q.reject(err);
        });        
      }, function(err) {
        Settings.loading = false;
        q.reject(err);
      });
      return q.promise;
    };
    return Settings;
  }
]);
/**
* @ngdoc services
* @name UserSettings
* @description gather all user settings, and manage to save them in localstorage. such as sounds/pushnotification/...
*/
angular.module('shell.services').factory('UserSettings', [
  'CacheFactory',
  '$rootScope',
  function (CacheFactory, $rootScope) {
    var UserSettings = {};
    /***
    * set the cache private attribute, which store the user settings object
    */
    var _cache = CacheFactory.get('userSettings');
    if (!_cache) {
      _cache = new CacheFactory('userSettings', { storageMode: 'localStorage' });
    }
    UserSettings.saveChanges = function () {
      _cache.put('userSettings', UserSettings.data);
    };
    UserSettings.setData = function(data) {
      if (_.keys(data)[0] === 'soundEnable') {
        UserSettings.toggleSound(data.soundEnable);
      }
      angular.extend(UserSettings.data, data);
      UserSettings.saveChanges();
    };
    UserSettings.getData = function(item){
      return UserSettings.data[item];
    };
    UserSettings.toggleSound = function(enabled){
      if(enabled) {
        $rootScope.$broadcast('userSettings.sound.enabled');
      } else {
        $rootScope.$broadcast('userSettings.sound.disabled');
      }
    };
    UserSettings.init = function () {
      var userSettingsCachedData = _cache.get('userSettings');
      UserSettings.data = { 
        soundEnable: false,
        pushnotification: true,
        firstTime: true,
        neverPredicted: true,
        neverEnteredAChallenge: true,
        fixtureIds: [],
        endorsementIds: []
      };

      if (userSettingsCachedData) {
        UserSettings.data = angular.extend(
          UserSettings.data, 
          userSettingsCachedData
        );
      }
      
      UserSettings.saveChanges();
        // Set the default parameters, and save them in cache
    };
    UserSettings.init();
    return UserSettings;
  }
]);
/**
* @ngdoc services
* @name AppBuilder
* @description This service provider build function, which build the app according to the blades settings
* It provide some helper function, used in the blades.
* @property isBuilt {bool} false until the build is successful
* @property bladesCard {Array} hold the bladesCard that will be use in the fixture list
*/
'use strict';
angular.module('shell.services').factory('AppBuilder', [
  'Settings',
  'BackgroundService',
  '$rootScope',
  '$q',
  function (Settings, BackgroundService, $rootScope, $q) {
    var AppBuilder = {
      isBuilt: false,
      registeredMenu: {
        top: {
          left: [],
          default: [],
          right: []
        },
        bottom: {
          left: [],
          default: [],
          right: []
        }
      },
      settings: []
    };
    /**
    * getBladeSettingsTemplate
    * @function
    * memberOf AppBuilder
    * @description this getter gives you the full path of the blade's settings template
    * @return templatePath {string} full path to the blade card template. 
    * Example: '/app/blades/user/card/templates/userTemplate.html'
    */
    AppBuilder.getBladeSettingsTemplate = function (bladeName) {
      return 'app/blades/' + bladeName + '/settings/templates/' + bladeName + 'SettingsTemplate.html';
    };
    /**
    * getBladeCardTemplate
    * @function
    * memberOf AppBuilder
    * @description this getter gives you the full path the of blade's card template
    * @return templatePath {string} full path to the blade card template. 
    * Example: '/app/blades/user/card/templates/userTemplate.html'
    */
    AppBuilder.getBladeCardTemplate = function (bladeName) {
      return 'app/blades/' + bladeName + '/card/templates/' + bladeName + 'Template.html';
    };
    /**
    * getBladeCardTemplate
    * @function
    * memberOf AppBuilder
    * @description this getter gives you the full path the blade's parameter view template
    * @return templatePath {string} full path to the blade card template. Example: '/app/blades/user/view/templates/userTemplate.html'    
    */
    AppBuilder.getBladeViewTemplate = function (bladeName) {
      return 'app/blades/' + bladeName + '/view/templates/' + bladeName + 'Template.html';
    };
    /**
    * rebuild
    * @function
    * @memberOf AppBuilder
    * @description clear the settings & reload them
    * @return promise
    */
    AppBuilder.rebuild = function (all, storeReload) {
      var q = $q.defer();
      Settings.clear(all, storeReload);
      Settings.load().then(function() {
        $rootScope.$broadcast('build.rebuild.succeeded');
        q.resolve();
      }, function(err) {
        console.log(err);
        $rootScope.$broadcast('build.rebuild.failed');
        q.reject(err);
      });
      return q.promise;
    };
    /**
    * build
    * @function
    * @memberOf AppBuilder
    * @description Load blades & fixtures settings. On success, this function emit 'build.succeeded' event on the $rootScope
    * on fail, this function emit 'build.failed' event on the $rootScope.
    * @return void
    */
    AppBuilder.build = function () {
      // set background
      BackgroundService.init();
      // Load the required settings (blades and fixtures)      
      Settings.load().then(function () {
        console.log('build.succeeded');
        AppBuilder.isBuilt = true;
        // Broadcast to the listeners the 'build.succeeded' event
        $rootScope.$broadcast('build.succeeded');
      }, function (err) {
        // Broadcast to the listeners the 'build.failed' event
        $rootScope.$broadcast('build.failed', err);
      });
    };
    $rootScope.$on('user.logged.out', function() {
      // set to the default background
      BackgroundService.changeBackground();
    });
    /** 
    * registerCard
    * @function
    * @memberOf AppBuilder
    * @description register a new blade's card, which if matches with the blade settings fetched from the API,
    * will be render in the fixture scroll area
    * @return void
    */
    AppBuilder.registerMenu = function (options) {
      if (options.menu !== 'top' && options.menu !== 'bottom') {
        throw 'menu available: top and bottom';
      }
      var template = options.template ? options.template : options.templateUrl;
      if (!template) {
        throw 'You must specify either template or templateUrl';
      }
      var side = options.side ? options.side : 'default';
      AppBuilder.registeredMenu[options.menu][side].push(template);
    };
    /**
    * registerSettingsView
    * @function
    * @memberOf AppBuilder
    * @description register a setting view for other blades to use
    * @param name {string} blade name
    * @return void
    */
    AppBuilder.registerSettingsView = function (name) {
      AppBuilder.settings.push(name);
    };
    return AppBuilder;
  }
]);
/**
* @ngdoc services
* @name LanguageService
* @description manage the language into our app. Store in localstorage the default language, lazyload the laguage dictionary
* and update $translate on change.
*/
'use strict';
angular.module('shell.services').factory('LanguageService', [
  '$q',
  '$rootScope',
  'LanguagePluginService',
  'languages',
  '$ionicHistory',
  '$ocLazyLoad',
  '$translate',
  'CacheFactory',
  '$locale',
  '$timeout',
  'club',
  '$state',
  function ($q, $rootScope, LanguagePluginService, languages, $ionicHistory, $ocLazyLoad, $translate, CacheFactory, $locale, $timeout, club, $state) {
    /***
    * set the cache private attribute, which store the user default language
    */
    var _cache = CacheFactory.get('lang');
    if (!_cache) {
      _cache = new CacheFactory('lang', { storageMode: 'localStorage' });
    }
    var LanguageService = {};
    LanguageService.defaultLanguage = 'en-gb';

    var overrideLocalValues = function(oldObject, newObject) {
      angular.forEach(oldObject, function(value, key) {
        if (!newObject[key]) {
          delete oldObject[key];
        } else if (angular.isArray(newObject[key])) {
          oldObject[key].length = newObject[key].length;
        }
      });
      angular.forEach(newObject, function(value, key) {
        if (angular.isArray(newObject[key]) || angular.isObject(newObject[key])) {
          if (!oldObject[key]) {
            oldObject[key] = angular.isArray(newObject[key]) ? [] : {};
          }
          overrideLocalValues(oldObject[key], newObject[key]);
        } else {
          oldObject[key] = newObject[key];
        }
      });
    };

    var formatLangKey = function(lang, dontCheckClubLanguage) {
      lang = lang.replace('_', '-');
      if (languages[lang]) {
        // perfect match
        return lang;
      } else {
        var shortLang = lang.substring(0, 2);
        if (languages[shortLang]) {
          // match with the default langue code, eg: 'en-us' will fallback on 'en'
          return shortLang;
        } else {
          for (var k in languages) {
            if (k.substring(0,2) == shortLang) {
              return k;
            }
          }
          // no alternative found          
          if (club.languageCode && dontCheckClubLanguage != true) {            // use club default language, eg: Real Madrid will be Spanish
            return formatLangKey(club.languageCode, true);
          } else {
            // no match at all.. wtf is this language
            return 'en-gb';
          }
        }
      }
    };

    LanguageService.setLanguage = function (lang) {
      var q = $q.defer();
      var kLang = formatLangKey(lang);
      // load translation file & set the lang as current language.
      $q.all([
        $ocLazyLoad.load(languages[kLang].path),
        $ocLazyLoad.load('lib/angular-i18n/angular-locale_' + kLang + '.js')
      ]).then(function () {        
        LanguageService.current = kLang;
        // If the language has never been set on the rootscope during the current sesion
        // set the currentLanguage
        $rootScope.currentLanguage = kLang;

        var localInjector = angular.injector(['ngLocale']);
        var newLocale = localInjector.get('$locale');
        // $locale is read-only we can't use extend here
        overrideLocalValues($locale, newLocale);
        _cache.put('lang', kLang);
        $translate.use(kLang);        

        if ($rootScope.predictionToSave !== true) {
          $ionicHistory.clearCache();
          $ionicHistory.clearHistory();
        }
        $timeout(function() {
          if ($rootScope.predictionToSave !== true) {
            // use to make sure there is not view cache with the previous language
            $state.go('shell.fixtures', {}, {reload:true});  
          }
          // Broadcast the language.changed event
          $rootScope.$broadcast('language.changed');
        });
        q.resolve(kLang);
      }, function (err) {
        q.reject(err);
      });
      return q.promise;
    };
    LanguageService.getCurrentLanguage = function () {
      var q = $q.defer();
      if (LanguageService.current) {
        q.resolve(LanguageService.current);
      } else {
        var lang = _cache.get('lang');
        if (lang) {
          q.resolve(lang);
        } else {
          LanguagePluginService.getCurrentLanguage().then(q.resolve, q.reject);
        }
      }
      return q.promise;
    };
    LanguageService.init = function () {
      var q = $q.defer();
      LanguageService.getCurrentLanguage().then(function (lang) {
        LanguageService.setLanguage(lang).then(q.resolve, q.reject);
      }, q.reject);
      return q.promise;
    };
    return LanguageService;
  }
]);
/**
* @ngdoc services
* @name Urls
* @description gather all settings in one place. The settings are loaded in the app.js
* @property {Blades} blades blades settings
* @property {Fixture} fixture Array of fixtures settings
*/
'use strict';

angular.module('shell')
.provider('Urls', function () {
    
    // Base Url for images
    // defaults
    this.localImageUrl = 'images/cdn/';
    this.remoteImageUrl = '';
    this.apiUrl = 'https://engine.playabl.com/api';

    /*
    live: https://playabldemo.blob.core.windows.net/images/
    dev: https://predictrstorage.blob.core.windows.net/images/
    */

    this.$get = ['ImageCacheFactory', '$http', '$q', function(ImageCacheFactory, $http, $q) {
      var asyncImageExist = function(url) {
        var q = $q.defer();
        $http.head(url).then(q.resolve, function(response) {
         if (response.status === 404) {
            q.reject();
          } else {
            q.resolve();
          }
        });
        return q.promise;
      };
      var localImageUrl = this.localImageUrl;
      var remoteImageUrl = this.remoteImageUrl;
      var apiUrl = this.apiUrl;
      var imageExists = this.imageExists;
      return {
        localImageUrl: localImageUrl,
        remoteImageUrl: remoteImageUrl,
        doesImageExist: imageExists,
        asyncImageExist: asyncImageExist,
        apiUrl: apiUrl,
        setUrlPromise: function(url, fallback) {
          if (url) {
            if (fallback) {
              var q = $q.defer();
              var self = this;
              this.asyncImageExist(url).then(function() {
                q.resolve(url);
              }, function() {
                if ( Object.prototype.toString.call( fallback ) === '[object Array]' ) {
                  var fallbackUrl = fallback.shift();                      
                  self.setUrl(fallbackUrl, fallback).then(q.resolve, q.reject);
                } else {
                  q.resolve(fallback);
                }
              });
              return q.promise;
            } else {
              return this.asyncImageExist(url);              
            }
          } else {
            console.log('missing parameters');
          }
        },
        setUrl: function(object, key, url, fallback) {
          if (object && key && url) {                
            if (fallback) {
              var self = this;
              this.asyncImageExist(url).then(function() {
                object[key] = url;
              }, function() {
                if ( Object.prototype.toString.call( fallback ) === '[object Array]' ) {
                  var fallbackUrl = fallback.shift();                      
                  self.setUrl(object, key, fallbackUrl, fallback);
                } else {
                  object[key] = fallback;
                }
              });
            } else {
              console.log('no fallback, used: ' + url);
              object[key] = url;
            }
          } else {
            console.log('missing parameters');
          }
        },
        urlBuilder: function(urlEndPart) {
          var url = this.localImageUrl + urlEndPart;
          ImageCacheFactory.cacheUrl(url);
          return url;
        },
        cache: function(url) {
          ImageCacheFactory.cacheUrl(url);
          return url;
        },
        urlRemoteBuilder: function(urlEndPart){
          var url = this.remoteImageUrl + urlEndPart;
          ImageCacheFactory.cacheUrl(url);
          return url;
        }
      };
    }];
    this.imageExists = function(url) {        
        var http = new XMLHttpRequest();
        http.open('HEAD', url, false);
        http.send();
        return http.status != 404;
    };

    this.setLocalImageUrl = function(url) {
      this.localImageUrl = url;
    };

    this.setRemoteImageUrl = function (url) {
      this.remoteImageUrl = url;
    };
    this.setApiUrl = function (url) {
      this.apiUrl = url;
    };
  });
/**
* @ngdoc services
* @name DataToolService
* @description The kind of tools that you may find in underscore. 
*/
angular.module('shell.services').service('DataToolService', [
  function () {
    var DataToolService = {};

    /***
    * @description run functions on each element of array or a single object
    * @return Array || object (what was passed as response)
    */
    DataToolService.ifIsArray = function(response, functions){
        if (_.isArray(response)) {
            for (var i = response.length - 1; i >= 0; i--) {
                for (var f = functions.length - 1; f >= 0; f--) {
                    functions[f](response[i]);
                }
            }
        } else if (_.isObject(response)) {
            for (var f = functions.length - 1; f >= 0; f--) {
                functions[f](response);
            }
        }
    };

    return DataToolService;
  }
]);
/**
* @ngdoc services
* @name ImageCacheFactory
* @description This service provide an easy way to use the browser cache, to preload our images.
* @require $q
*/
angular.module('shell.services').factory('ImageCacheFactory', [
  '$q',
  function ($q) {
    var loadImage = function (imageUrl) {
      var q = $q.defer();
      var img = new Image();
      img.onload = q.resolve;
      img.onerror = q.reject;
      img.src = imageUrl;
      return q.promise;
    };
    /**
    * @description This function is used to extend the $q.all function, and run all the promises even if one of them fail
    * @param promises {Array} promises to resolve
    * @return promise
    */
    var allIgnoreError = function (promises) {
      var q = $q.defer();
      var len = promises.length;
      var result = [];
      var resolvedCount = 0;
      var handlePromiseResult = function (idx, err, res) {
        result[idx] = {
          succeed: !!res,
          error: err,
          result: res
        };
        resolvedCount++;
        if (resolvedCount === len) {
          q.resolve(result);
        }
      };
      for (var idx = len - 1; idx >= 0; idx--) {
        promises[idx].then(function (res) {
          handlePromiseResult(idx, null, res);
        }, function (err) {
          handlePromiseResult(idx, err, null);
        });
      }
      return q.promise;
    };
    return {
      /**
      * cacheUrl
      * @function
      * @description preload the param url
      * @param url {string}
      * @return promise
      */
      cacheUrl: function (url) {
        return loadImage(url);
      },
      /**
      * cacheUrls
      * @function
      * @description preload the param urls
      * @param urls {Array} of string
      * @param stopOnFail, reject as soon as one image fail
      * @return promise
      */
      cacheUrls: function (urls, stopOnFail) {
        var promises = [];
        var len = urls.length;
        for (var i = 0; i < len; i++) {
          var loadingPormise = loadImage(urls[i]);
          promises.push(loadingPormise);
        }
        if (stopOnFail) {
          return $q.all(promises);
        } else {
          return allIgnoreError(promises);
        }
      }
    };
  }
]);
/***
* This interceptor is used to inject the languageCode parameter in all request we make to our api
*/
angular.module('shell.services').factory('LanguageHttpInterceptor', [
  '$rootScope',
  function ($rootScope) {
    return {
      // optional method
      'request': function (request) {
        // Using the $rootScope to avoid circular dependencies
        if ($rootScope.currentLanguage) {
          // if the request goes to the api
          if (request.url.indexOf('/api/') !== -1) {
            request.url += (request.url.indexOf('?') === -1 ? '?' : '&') + 'languageCode=' + $rootScope.currentLanguage;
          }
        }
        return request;
      }
    };
  }
]);

angular.module('shell.services')
.factory('ExceptionService', [
  'club',
  '$injector',
  function(club, $injector) {
    var ExceptionService = {
      airbrake: new airbrakeJs.Client({projectId: 1, projectKey: 'cdb12637f71095d8d71892dd35fe6783'})
    };
    ExceptionService.airbrake._host = 'http://playablerrbit.cloudapp.net:3000/';

    ExceptionService.notify = function(params) {
      var rootScope = $injector.get('$rootScope');
      var id = rootScope.user ? rootScope.user.id : 'guess';
      if (window.cordova) {
        var parameters = angular.extend(params, {
          'userId': rootScope.id,
          'clubId': club.clubId,
          'appVersion': club.appVersion
        });
        ExceptionService.airbrake.notify(parameters);
      } else {
        console.log('ERROR WILL BE LOGGED ON ERRBIT');
      }
    };
    return ExceptionService;
  }
])
.config([
  '$provide',
  function($provide) {
    $provide.decorator('$exceptionHandler', [
      '$delegate',
      'ExceptionService',
      function ($delegate, ExceptionService) {
        return function (exception, cause) {
          ExceptionService.notify({
            'error': exception,              
            'params': {
              'cause': cause
            }
          });
          $delegate(exception, cause);
        }
      }
    ]);
  }
]);
angular.module('shell.services')
.factory('ModalService', [
  '$ionicModal',
  '$rootScope',
  '$q',
  '$injector',
  '$controller',
  function($ionicModal, $rootScope, $q, $injector, $controller) {
    var ModalService = {
      currentModalScore: null
    };

    var _cleanup = function (scope) {
      scope.$destroy();
      if (scope.modal) {
        scope.modal.remove();
      }
      ModalService.currentModalScore = null;
    };

    var _evalController = function (ctrlName) {
      var result = {
        isControllerAs: false,
        controllerName: '',
        propName: ''
      };
      var fragments = (ctrlName || '').trim().split(/\s+/);
      result.isControllerAs = fragments.length === 3 && (fragments[1] || '').toLowerCase() === 'as';
      if (result.isControllerAs) {
        result.controllerName = fragments[0];
        result.propName = fragments[2];
      } else {
        result.controllerName = ctrlName;
      }
      return result;
    };

    ModalService.showModal = function(templeteUrl, controller, parameters, options) {
      // Grab the injector and create a new scope
      var deferred = $q.defer();
      var ctrlInstance;
      var animation = options && options.animation ? options.animation : 'slide-in-up';
      var modalScope = $rootScope.$new();
      var thisScopeId = modalScope.$id;
      var defaultOptions = {
        animation: 'slide-in-up',
        focusFirstInput: false,
        backdropClickToClose: true,
        hardwareBackButtonClose: true,
        show: true,
        modalCallback: null
      };

      options = angular.extend({}, defaultOptions, options);

      $ionicModal.fromTemplateUrl(templeteUrl, {
        scope: modalScope,
        animation: options.animation,
        focusFirstInput: options.focusFirstInput,
        backdropClickToClose: options.backdropClickToClose,
        hardwareBackButtonClose: options.hardwareBackButtonClose
      }).then(function (modal) {
        modalScope.modal = modal;
        ModalService.currentModalScore = modalScope;
        modalScope.openModal = function () {
          modalScope.modal.show();
        };
        modalScope.closeModal = function (result) {
          deferred.resolve(result);
          return modalScope.modal.hide();
        };
        modalScope.$on('modal.hidden', function (thisModal) {
          if (thisModal.currentScope) {
            var modalScopeId = thisModal.currentScope.$id;
            if (thisScopeId === modalScopeId) {
              deferred.resolve(null);
              _cleanup(thisModal.currentScope);
            }
          }
        });
        if (controller) {
          // Invoke the controller
          var locals = { '$scope': modalScope, 'parameters': parameters };
          var ctrlEval = _evalController(controller);
          ctrlInstance = $controller(controller, locals);
          if (ctrlEval.isControllerAs) {
            ctrlInstance.openModal = modalScope.openModal;
            ctrlInstance.closeModal = modalScope.closeModal;
          }
        }
        if (options.show) {
          modalScope.modal.show()
            .then(function () {
            modalScope.$broadcast('modal.afterShow', modalScope.modal);
          });
        }
        if (angular.isFunction(options.modalCallback)) {
          options.modalCallback(modal);
        } else {
          deferred.resolve(modalScope);          
        }
      }, function (err) {
        deferred.reject(err);
      });
      return deferred.promise;
    }


    ModalService.show = function(templeteUrl, controller, parameters, options) {
      if (ModalService.currentModalScore) {
        var deferred = $q.defer();
        ModalService.currentModalScore.closeModal()
        .finally(function() {
          ModalService.showModal(templeteUrl, controller, parameters, options)
          .then(deferred.resolve, deferred.reject);
        });
        return deferred.promise;
      } else {
        return ModalService.showModal(templeteUrl, controller, parameters, options);
      } 
    };

    $rootScope.$on('$stateChangeStart', function() {
      if (ModalService.currentModalScore) {
        ModalService.currentModalScore.closeModal();
      }
    });

  return ModalService;
 
}]);
angular.module('shell.services')
.constant('TOAST', {
  types: {
    "info": {
      name: 'info',
      ionicon: 'ion-information-circled',
      type: 'STANDARD',
      icon: 2,
      bgColor: 2,
      color: "#ffffff",
      duration: 3000     
    },
    "loading": {
      name: 'loading',
      ionicon: '',
      type: 'STANDARD',
      icon: 4,
      bgColor: 2,
      color: "#ffffff",
      duration: 3000     
    },
    "loading_progress": {
      name: 'loading',
      type: 'PROGRESS_HORIZONTAL',
      icon: 4,
      bgColor: 2,
      color: "#ffffff",
      duration: -1    
    },
    "error": {
      name: 'error',
      ionicon: '',
      type: 'STANDARD',
      icon: 2,
      bgColor: 6,
      color: "#ffffff",
      duration: 3000
    },
    "success": {
      name: 'success',
      ionicon: 'ion-android-done',
      type: 'STANDARD',
      icon: 2,
      bgColor: 3,
      color: "#ffffff",
      duration: 3000
    },
    "save": {
      name: 'save',
      ionicon: 'ion-android-done',
      type: 'STANDARD',
      icon: 5,
      bgColor: 3,
      color: "#ffffff",
      duration: 3000
    }
  },
  icons: {
    "EDIT": 0,
    "EXIT": 1,
    "INFO": 2,
    "REDO": 3,
    "REFRESH": 4, 
    "SAVE": 5,
    "SHARE": 6,
    "UNDO": 7
  },
  type: {
    "STANDARD":0,
    "PROGRESS":1,
    "PROGRESS_HORIZONTAL":2,
    "BUTTON":3
  },
  bright: {
    "DARK": 100,
    "LIGHT": 1
  },
  color: {
    "BLACK": 0,
    "BLUE": 1,
    "GRAY": 2,
    "GREEN": 3,
    "ORANGE": 4,
    "PURPLE": 5,
    "RED": 6,
    "WHITE": 7
  }
})
.factory('ToastService', [
  '$ionicLoading',
  '$filter',
  '$timeout',
  function($ionicLoading, $filter, $timeout) {
    var ACTIVE_NATIVE_TOAST = false;
    var ToastService = {
      currentProgress: 0
    };

    ToastService.showNativeToast = function(title, message, toastInfo, lockBg, duration) {
      if (!duration) {
        duration = toastInfo.duration;
      }
      if (ACTIVE_NATIVE_TOAST && window.plugin && window.plugin.toastPlugin) {
        
        var translated_title = $filter('translate')(title);
        var translated_message = $filter('translate')(message);
        if (lockBg) {
          $ionicLoading.show({ template: '<div class="toast-content"></div>' });
        }
        window.plugin.toastPlugin.show([ translated_title, translated_message, toastInfo.icon, toastInfo.type, duration, toastInfo.bgColor, toastInfo.color ], 
          function() {            
            console.log('succes');
          }, function(err) { 
            console.log(err);
          })
      } else {
        ToastService.showToast(title, message, toastInfo.name, toastInfo.ionicon, duration);
      }
    };
    ToastService.hideNativeToast = function() {
      $ionicLoading.hide();
      if (ACTIVE_NATIVE_TOAST && window.plugin && window.plugin.toastPlugin) {
        window.plugin.toastPlugin.dismiss([], function() {console.log('hidden');}, function(err){ console.log(err); });
      }
    };
    ToastService.setProgress = function(value) {
      ToastService.currentProgress = value;
      if (value == 100) {
        ToastService.hideNativeToast();
      }
      if (window.plugin && window.plugin.toastPlugin) {
        return;
        window.plugin.toastPlugin.setProgress([value], function() {
          console.log('setProgress success: ' + value);
        }, function(err) {
          console.log(err);
        });
      }
    };

    ToastService.showToast = function (title, message, type, icon, hideTime) {
        hideTime = typeof hideTime !== 'undefined' ? hideTime : false;
        var loader = '';
        if (type.indexOf('loading') != -1) {
          loader = '<img class="rotating-loading go-clockwise" ng-src="images/assets/rotatingTimerEffect.svg">';
        }
        $ionicLoading.show({ template: '<div class="is-loading toast-content">' + loader + '<div class="col"><span class="row"><i class="' + icon + '"></i>{{"' + title + '"|translate}}</span><span class="row">{{"' + message + '"|translate}}</span></div>' });
        if (hideTime !== -1) {
          $timeout(function () {
            ToastService.hideToast();
          }, hideTime);
        }
    };
  
    ToastService.hideToast = function () {
      $ionicLoading.hide();
    };

    return ToastService;
  }
]);
angular.module('shell.services')
.factory('PopupService', [
  '$ionicPopup',
  function($ionicPopup) {
    var PopupService = {
      current: null
    };

    PopupService.show = function(params) {
      PopupService.hide();
      PopupService.current = $ionicPopup.show(params);
      return PopupService.current;
    };
    PopupService.confirm = function(params) {
      PopupService.hide();
      PopupService.current = $ionicPopup.show(params);
      return PopupService.current;

    };
    PopupService.alert = function(params) {
      PopupService.hide();
      PopupService.current = $ionicPopup.show(params);
      return PopupService.current;
    }
    PopupService.hide = function() {
      if (PopupService.current) {
        PopupService.current.close();
        PopupService.current = null;
      }
    };

    return PopupService; 
  }
]);

angular.module('shell.services').service('NetworkStateService', [
	'TOAST',
	'ToastService',
  	function (TOAST, ToastService) {
	
	    var NetworkStateService = {};

	    NetworkStateService.isOnline = function() {
	    	if (navigator && navigator.connection) {
		    	return navigator.connection.type != Connection.NONE;    		
	    	} else {
	    		return true;
	    	}
	    };

	    document.addEventListener('online', function() {
	    	console.log('IS ONLINE');
	    }, false);
	    document.addEventListener('offline', function() {
	    	console.log('IS OFFLINE');
	    	ToastService.showNativeToast('You appear offline. Try and find an internet connect to get full features.', '', TOAST.types.info, false);
	    }, false);
    	
    	return NetworkStateService;
  	}
]);

angular.module('shell.services').factory('NavigationService', [
    function () {
        var DeepLinkNavigationService = {
            routes: {}
        };

        DeepLinkNavigationService.registerRoute = function(route, navigateFunction) {
            if (typeof navigateFunction === 'function') {
                console.log('Route registered: ' + route);
                DeepLinkNavigationService.routes[route] = navigateFunction;
            } else {
                console.log('Second parameter needs to be a function');
            }
        };

        DeepLinkNavigationService.navigate = function(route, params) {
            if (DeepLinkNavigationService.routes[route]) {
                DeepLinkNavigationService.routes[route](params || {});
                return true;
            } else {
                return false;            
            }
        }

        return DeepLinkNavigationService;
    }
]);
/**
* @ngdoc services
* @name AudioToolService
* @description A wrapper for html5 audio.
*/
angular.module('shell.services').service('AudioToolsService', [
  function () {
    var AudioToolsService = {};

    /***
    * createSound
    * @description Public function, which creates an Audio Object with the right url for the right browser
    * @return Audio
    */
    AudioToolsService.createSound = function(file) {
      var sound = new Audio();
      if (sound.canPlayType('audio/ogg')) {
        sound.src = file + '.ogg';
      } else if (sound.canPlayType('audio/mp3')) {
        sound.src = file + '.mp3';
      } else {
        sound.src = file + '.wav';
      }
      return sound;
    };

    return AudioToolsService;
  }
]);
'use strict';
angular.module('shell.services').factory('BackgroundService', [
  '$rootScope',
  'club',
  'CacheFactory',
  function ($rootScope, club, CacheFactory) {
    var BackgroundService = {
      current: '',
      defaultBackground: 'images/cdn/sports/football/clubs/' + club.clubId + '/fixedBackground.jpg'
    };
    var _cache = CacheFactory.get('background');
    if (!_cache) {
      _cache = new CacheFactory('background', { storageMode: 'localStorage' });
    }
    var applyBackground = function() {
      if ($rootScope.currentBackgroundUrl !== BackgroundService.currentBackground) {
        $rootScope.currentBackgroundUrl = BackgroundService.currentBackground;        
        console.log('applyBackground: ' + $rootScope.currentBackgroundUrl);
      }
    };
    BackgroundService.init = function () {
      var currentBackground = _cache.get('current');
      if (currentBackground) {
        BackgroundService.currentBackground = currentBackground;
      } else {
        BackgroundService.currentBackground = BackgroundService.defaultBackground;
      }
      applyBackground();
    };
    BackgroundService.changeBackground = function(background) {
      console.log('changeBackground');
      if (!background) {
        _cache.put('current', BackgroundService.defaultBackground);                      
      } else {
        _cache.put('current', background);        
      }
      BackgroundService.init();
    };
    return BackgroundService;
  }
]);
/**
* @ngdoc services
* @name LanguagePluginService
* @description Give access to the languagePlugin. It allow to get the default local language on the user's device
* @require $q $window club $ionicPlatform
*/
angular.module('shell.services').factory('AppsflyerPluginService', [
  'club',
  '$rootScope',
  function (club, $rootScope) {
    var AppsflyerPluginService = {};

    $rootScope.$on('user.logged.in', function () {
      if (window.plugins && window.plugins.appsFlyer) {
        if ($rootScope.user) {
          window.plugins.appsFlyer.setAppUserId($rootScope.user.userId);          
        }
      }
    });

    AppsflyerPluginService.initialize = function() {
      console.log('AppsflyerPluginService.initialize');
      if (window.plugins && window.plugins.appsFlyer) {
        var args = [];
        if (ionic.Platform.isAndroid() == false) {
          args.push(club.appsflyerKey.ios);
          args.push(club.itunesAppId);          
        } else {
          args.push(club.appsflyerKey.android);
        }
        window.plugins.appsFlyer.initSdk(args);

        var getUserIdCallbackFn = function(id) {
            console.log('received id is: ' + id);
        }
        window.plugins.appsFlyer.getAppsFlyerUID(getUserIdCallbackFn);

      } else {
        console.log('appsflyer is not available');
      }
    };  

    return AppsflyerPluginService;
  }
]);
/**
* @ngdoc services
* @name LanguagePluginService
* @description Give access to the languagePlugin. It allow to get the default local language on the user's device
* @require $q $window club $ionicPlatform
*/
angular.module('shell.services').factory('LanguagePluginService', [
  '$q',
  '$window',
  'club',
  '$ionicPlatform',
  function ($q, $window, club, $ionicPlatform) {
    var LanguagePluginService = {};
    /***
    * getPlugin
    * @description Private function, which try to get the plugin
    * @return promise
    */
    var getPlugin = function () {
      var q = $q.defer();
      $ionicPlatform.ready(function () {
        if (!window.plugin) {
          q.reject('plugins have not been load...');
        } else if (!window.plugin.languagePlugin) {
          q.reject('languagePlugin has not been installed');
        } else {
          q.resolve(window.plugin.languagePlugin);
        }
      });
      return q.promise;
    };
    /**
    * @function getCurrentLanguage
    * @memberOf LanguagePluginService
    * @description This function use the languagePlugin to get the default local language on the user's device
    * @return promise
    */
    LanguagePluginService.getCurrentLanguage = function () {
      var q = $q.defer();
      getPlugin().then(function (languagePlugin) {
        languagePlugin.getCurrentLanguage(// success, we have the local default language
        function (lang) {
          console.log(lang);
          q.resolve(lang.toLowerCase());
        }, function (err) {
          console.log(err);
          console.log(club.defaultLanguageCode);
          // If we are now able to get the local language, we return the default club language 
          q.resolve(club.defaultLanguageCode);
        });
      }, function (err) {
        console.log(err);
        // If we are now able to get the local language, we return the default club language 
        q.resolve(club.languageCode ? club.languageCode : 'en-gb');
      });
      return q.promise;
    };
    return LanguagePluginService;
  }
]);
/**
* @ngdoc services
* @name LanguagePluginService
* @description Give access to the languagePlugin. It allow to get the default local language on the user's device
* @require $q $window club $ionicPlatform
*/
angular.module('shell.services').factory('PushNotificationPluginService', [
  '$timeout',
  'ToastService',
  'club',
  '$rootScope',
  'UserSettings',
  'NavigationService',
  'usersService',
  function ($timeout, ToastService, club, $rootScope, UserSettings, NavigationService, usersService) {
    var PushNotificationPluginService = {};

    $rootScope.$on('user.logged.out', function() {
      if (window.batch) {
        batch.userProfile.setLanguage(null);
        batch.userProfile.setRegion(null);
        batch.userProfile.setCustomIdentifier(null);
      }
    });
    $rootScope.$on('language.changed', function() {
      if (window.batch && $rootScope.user) {
        batch.userProfile.setLanguage($rootScope.currentLanguage);
      }
    });
    $rootScope.$on('user.logged.in', function() {
      if (window.batch && $rootScope.user) {
        batch.userProfile.setRegion($rootScope.user.country);
        batch.userProfile.setCustomIdentifier($rootScope.user.id);
      }
    });
    $rootScope.$on('pushnotification-settings-changed', function() {
      if (window.batch) {
        var notifTypes = batch.push.AndroidNotificationTypes.NONE;
        var enable = UserSettings.getData('pushnotification');
        if (enable) {
          notifTypes = notifTypes | batch.push.AndroidNotificationTypes.ALERT;
          notifTypes = notifTypes | batch.push.AndroidNotificationTypes.SOUND;
          notifTypes = notifTypes | batch.push.AndroidNotificationTypes.VIBRATE;
          notifTypes = notifTypes | batch.push.AndroidNotificationTypes.LIGHTS;
        }
        if (ionic.Platform.isAndroid()) {
          batch.push.setAndroidNotificationTypes(notifTypes);
        } else {
          batch.push.setiOSNotificationTypes(notifTypes);
        }
      }
    });

    var onPushReceived = function(event) {
      console.log(event);
      if (event.message) {
        console.log("Received push: " + event.message)
      } else {
        console.log("No incoming message")
      }
    };
     // Incoming message callback
    var onLaunchedFromPush = function(event) {
      console.log(event);
      if (event.payload && event.payload.route) {
        NavigationService.navigate(event.payload.route, event.payload);
      }
      if (event.message) {
        console.log("Launched from push: " + event.message)
      } else {
        console.log("No incoming message")
      }
    };
    // Registration callback
    var onRegistration = function(event)  {
      console.log(event);
      if (!event.error) {
        console.log("Reg Success: " + event.channelID)
      } else {
        console.log(event.error)
      }
    };
    document.addEventListener('batchPushReceived', onLaunchedFromPush, false);


    if (ionic.Platform.isAndroid() == false && window.batch) {
      $rootScope.$on('$stateChangeSuccess', function (evt, toState) {
        if (toState.name == 'shell.predict') {
          $timeout(function() {
            console.log('registerForRemoteNotifications');
            batch.push.registerForRemoteNotifications();
          });
        }
      });
    }

    var updateUserToken = function(token) {
      if (!$rootScope.user) {
        return;
      }

      var update = {
        'clubLoyalties': [
          {
            'clubId': club.clubId,
            'batchToken': token            
          }
        ]
      };
      usersService.update($rootScope.user.id, update)
      .then(function () {
        console.log('batch token updated on the user profile');
      }, function (err) {
        console.log(err);
      });
    }

    $rootScope.$on('user.logged.in', function() {
      if (window.batch) {
        window.batch.push.getLastKnownPushToken(function(token) {
          console.log(token);
          updateUserToken(token);
        });
      }
    });
    $rootScope.$on('user.logged.out', function() {
      updateUserToken(null);
    });

    PushNotificationPluginService.initialize = function() {
      console.log('PushNotificationPluginService.initialize');
      if (window.batch) {
        batch.setConfig({
          "androidAPIKey": club.batchKeys.android.batch_api_key,
          "iOSAPIKey": club.batchKeys.ios.batch_api_key
        });
        batch.push.setGCMSenderID(club.gcmId).setup();
        batch.start();        
      } else {
        console.log('batch is not available');
      }
    };  

    return PushNotificationPluginService;
  }
]);

angular.module('shell.services').factory('SupersonicAdsPluginService', [
  '$q',
  '$rootScope',
  '$window',
  '$ionicPlatform',
  'club',
  'ModalService',
  'activitiesService',
  'userActivitiesService',
  'usersService',
  'ExceptionService',
  'Urls',
  function ($q, $rootScope, $window, $ionicPlatform, club, ModalService, activitiesService, userActivitiesService, usersService, ExceptionService, Urls) {
    var SupersonicAdsPluginService = {
      preloaded: false,
      ready: false,
      countDownEndTime: null,
      loading: false
    };

    var videoClose = false;
    var rewarded = false;
    var DAY = 24 * 60 * 60 * 1000;

    $rootScope.$on('user.activities.ready', function() {
      var activities = activitiesService.filter({ title: 'Reward Video', 'isActive': true});
      if (activities && activities.length > 0) {
        //SupersonicAdsPluginService.ready = true;     
        SupersonicAdsPluginService.activity = activities[0];
        var userActivity = userActivitiesService.filter({'userId': $rootScope.user.id, 'activityId': activities[0].id});
        SupersonicAdsPluginService.rewardActivity = userActivity instanceof Array ? userActivity[0] : userActivity;    
      } else {
        console.log('no reward video user activity');
        SupersonicAdsPluginService.ready = false;
      }
    });

    SupersonicAdsPluginService.preload = function() {
      $ionicPlatform.ready(function () {
        if (window.SupersonicAds) {
          var id = ionic.Platform.isAndroid() ? club.supersonic.android : club.supersonic.ios;
          SupersonicAds.init(id, $rootScope.user ? $rootScope.user.id : '', SupersonicAds.types.REWARDED_VIDEO);
          //SupersonicAdsPluginService.ready = true;
        } else {
          SupersonicAdsPluginService.ready = false;
        }
      });
    };


    SupersonicAdsPluginService.rewardUser = function() {
      // call the action endpoint to validate and credit the user      
      var params = SupersonicAdsPluginService.rewardActivity;
      if (!SupersonicAdsPluginService.rewardActivity) {
        params = {
          userId: $rootScope.user.id,
          activityId: SupersonicAdsPluginService.activity.id
        };
      } 
      params.IncrementViewCount = '{$inc}';
      userActivitiesService.create(params).then(function(rewardActivity) {
        SupersonicAdsPluginService.rewardActivity = rewardActivity;
        usersService.updateCurrentUser();
        if (videoClose == false) {
          var reward = SupersonicAdsPluginService.activity.chipsOnCompletion;
          ModalService.show('app/blades/shop/popup/template/rewardsPopupTemplate.html', 'RewardsPopupController', { reward: reward }, { animation: 'animated zoomIn'});            
          rewarded = true;
        }
      }, function(err) {
        ExceptionService.notify(err);
      });
    };

    SupersonicAdsPluginService.play = function() {
        var q = $q.defer();
        if (window.SupersonicAds && SupersonicAdsPluginService.ready) {
          // preload the reward image
          Urls.urlRemoteBuilder('coins/freeBalls.png');
          // show a rewarded ad for placement RightHere
          SupersonicAds.showRewardedAd("DefaultRewardedVideo", function(info) {
            console.log(info);
            q.resolve();
          }, function(err) {
            console.log(err);
            q.reject(err);
          });                
        } else {
          q.reject();
        }
        return q.promise;
    };

    var log = function(event, data) {
      console.log('-------------------');
      console.log('===================');
      console.log(event);
      console.log(data);
      console.log('===================');
      console.log('-------------------');
    }
    $ionicPlatform.ready(function() {

      window.addEventListener("rewardedVideoRewardReceived", function(e) {
        var placement = e.placement;
        console.log(placement.id); // only available on android
        console.log(placement.name);
        console.log(placement.reward);
        console.log(placement.amount);
        SupersonicAdsPluginService.rewardUser();
      }, false);
      window.addEventListener("rewardedVideoAvailabilityChanged", function(e) {
        SupersonicAdsPluginService.ready = e.available;
        SupersonicAdsPluginService.loading = false;          
        // if (e.available) {
        //   SupersonicAdsPluginService.loading = false;          
        // } else {
        //   SupersonicAdsPluginService.loading = false;  
        // }
      }, false);
      window.addEventListener("rewardedVideoInitializationFailed", function(e) {
        SupersonicAdsPluginService.loading = false;
        SupersonicAdsPluginService.ready = false;
      }, false);
      window.addEventListener("rewardedVideoInitialized", function() {
      }, false);
      window.addEventListener("rewardedVideoClosed", function() {
        videoClose = true;
        if (rewarded == false) {
          var reward = SupersonicAdsPluginService.activity.chipsOnCompletion;
          ModalService.show('app/blades/shop/popup/template/rewardsPopupTemplate.html', 'RewardsPopupController', { reward: reward }, { animation: 'animated zoomIn'});
        }
      }, false);
      window.addEventListener("rewardedVideoOpened", function() {
        videoClose = false;
      }, false);



    window.addEventListener("rewardedVideoRewardReceived", function(e) { log('rewardedVideoRewardReceived', e); }, false);
    window.addEventListener("rewardedVideoEnded", function(e) { log('rewardedVideoEnded', e); }, false);
    window.addEventListener("rewardedVideoStarted", function(e) { log('rewardedVideoStarted', e); }, false);
    window.addEventListener("rewardedVideoAvailabilityChanged", function(e) { log('rewardedVideoAvailabilityChanged', e); }, false);
    window.addEventListener("rewardedVideoClosed", function(e) { log('rewardedVideoClosed', e); }, false);
    window.addEventListener("rewardedVideoOpened", function(e) { log('rewardedVideoOpened', e); }, false);
    window.addEventListener("rewardedVideoInitializationFailed", function(e) { log('rewardedVideoInitializationFailed', e); }, false);
    window.addEventListener("rewardedVideoInitialized", function(e) { log('rewardedVideoInitialized', e); }, false);
    window.addEventListener("rewardedVideoShowFail", function(e) { log('rewardedVideoShowFail', e); }, false);

    });

    return SupersonicAdsPluginService;
  }
]);

angular.module('shell.services').factory('DeepLinkService', [
  '$rootScope',
  '$state',
  'club',
  '$q',
  '$ionicPlatform',
  'activitiesService',
  'userActivitiesService',
  function ($rootScope, $state, club, $q, $ionicPlatform, activitiesService, userActivitiesService) {
    var DeepLinkService = {
      data: null,
      ready: false,
      rewardInvite: null,
      referralsUserActivity: null
    };
    var shareLinkFallBack = 'http://'+club.subdomain+'.playabl.com';
    var executePayload = function(data) {
      if (data.type == 'route' && data.route) {
        DeepLinkNavigationService.navigate(data.route, data.params);        
      }
    };

    var setIdentity = function() {
      if (window.branch) {
        window.branch.setIdentity($rootScope.user.id, function(err, data) {
          if (err) {
            console.log(err);
          } else {
            console.log(data);
          }
        });
      }
    };

    $rootScope.$on('user.logged.out', function() {
      if (window.branch) {
        window.branch.logout();
      }
    });
    $rootScope.$on('user.logged.in', function() {
      if (DeepLinkService.ready) {
        setIdentity();
      }
    });
    $rootScope.$on('build.succeeded', function() {
      if (DeepLinkService.data) {
        executePayload(DeepLinkService.data);
      }
      DeepLinkService.data = null;
      DeepLinkService.ready = true;
    });

    var checkCredits = function() {
      if (window.branch) {
        window.branch.credits(function(err, data) {
          console.log(err);
          console.log(data);
          if (!err) {
            var credits = data['default'];
            console.log('MY CREDIT: ' + credits);
            if (credits > 0) {
              // userActivity.update(...)
              var params = DeepLinkService.referralsUserActivity;
              if (!params) {
                params = {
                  userId: $rootScope.user.id,
                  activityId: DeepLinkService.rewardInvite.id
                };
              } 
              params.IncrementViewCount = '{$inc' + credits + '}';
              userActivitiesService.create(params)
              .then(function(referralsUserActivity) {
                DeepLinkService.referralsUserActivity = referralsUserActivity;
                usersService.updateCurrentUser();
                window.branch.redeem(credits, 'default');
              }, function(err) {
                ExceptionService.notify(err);
              });
            }
          }
        });
      }
    };

    $rootScope.$on('user.activities.ready', function() {
      var activities = activitiesService.filter({ title: 'Referrals', 'isActive': true});
      if (activities && activities.length > 0) {
        DeepLinkService.rewardInvite = activities[0];
        var userActivity = userActivitiesService.filter({'userId': $rootScope.user.id, 'activityId': activities[0].id});
        DeepLinkService.referralsUserActivity = userActivity instanceof Array ? userActivity[0] : userActivity;
        checkCredits();
      } else {
        console.log('no active referrals activity');
      }
    });

    /**
    * channel: sms/facebook/twitter/whatsapp/...
    * feature: share/invite
    **/
    DeepLinkService.createLink = function(channel, feature, data) {
      var q = $q.defer();
      if (window.branch) {
        branch.link({
          'channel': channel,
          'feature': feature,
          'data': data,
          'tags': [ionic.Platform.platform(), club.clubId]
        }, function(err, link) {
          console.log(err);
          console.log(link);
          q.resolve(err ? shareLinkFallBack : link);
        });
      } else {
        q.resolve(shareLinkFallBack);
      }
      return q.promise;
    };
    DeepLinkService.initialize = function() {
      if (club.branchKey) {
        console.log('DeepLinkService.initialize');
        if (window.branch) {
          $ionicPlatform.ready(function () {
            window.branch.init(club.branchKey, function(err, data) {
              console.log(err);
              console.log(data);
              if (!err && data.data) {
                if ($rootScope.user) {
                  setIdentity();
                }
                var parsed_data = JSON.parse(data.data);
                if (parsed_data['+clicked_branch_link']) {
                  // data are the deep linked params associated with the link that the user clicked -> was re-directed to this app
                  // data will be empty if no data found
                  // ... insert custom routing logic here ...
                  if (parsed_data.route == 'shell.challenges' && parsed_data.fixtureId) {
                    DeepLinkService.data = {
                      'type': 'route',
                      'route': parsed_data.route,
                      'params': { 'fixtureId': parsed_data.fixtureId }
                    }
                  }
                  if (DeepLinkService.ready) {
                    executePayload(DeepLinkService.data);
                  }
                }
              }
            });
          });
        } else {
          console.log('branch plugin not installed');
        }
      } else {
        console.log('missing branchKey on the club object');
      }
    }; 

    $ionicPlatform.on('resume', function(){
      DeepLinkService.initialize();
    }); 

    return DeepLinkService;
  }
]);
'use strict';
angular.module('shell.controllers').controller('SplashController', [
  '$scope',
  '$rootScope',
  '$state',
  'AppBuilder',
  'club',
  'Urls',
  '$timeout',
  '$ionicHistory',
  function ($scope, $rootScope, $state, AppBuilder, club, Urls, $timeout, $ionicHistory) {

    $scope.clubLogoImageUrl = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/clubLogo.svg');
    $scope.clubSplashTwo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/fixedBackground.jpg');
    $scope.scorePredictorLogo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/scorePredictorLogo.png');
    $scope.playerCluster = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/playerCluster.png');

    $scope.ready = false;

    var nextSplashState = function () {
      $timeout(function () {
        $scope.state = 'second';
        $timeout(function () {
          if (AppBuilder.build) {
            $timeout(function () {
              goToFixture();
            }, 1000);
          }
        }, 3000);
      }, 2000);
    };
    /***
    * Hide the native splash screen once the view is complelty loaded..
    */
    var listener = $rootScope.$on('$ionicView.loaded', function () {
      // deregister the ionivView.loaded event listener
      listener();
      ionic.Platform.ready(function () {
        if (navigator && navigator.splashscreen) {
          $timeout(function () {
            navigator.splashscreen.hide();
            nextSplashState();
          }, 2000);
        } else {
          $timeout(function () {
            nextSplashState();
          }, 1000);
        }
      });
    });

    $scope.$on('$ionicView.afterEnter', function() {
      $timeout(function() {
        $scope.ready = true;
        $scope.state = 'first';
      });
    });
    /***
		* Change state to 'fixtures' and reset history, to avoir user to go back on the splash screen once the app build
		*/
    var goToFixture = function () {
      if ($state.current.name == 'splash') {
        $ionicHistory.nextViewOptions({
          disableBack: true,
          historyRoot: true
        });
        $state.go('shell.fixtures');
      }
    };
    /**
		* Set this variable at true on build.failed event. Is used in the template to render either
		* the spinner or the retry button
		*/
    $scope.buildFailed = false;
    /***
		* If the app is already build, before the 'beforeEnter' event, then go to the 'fixtures' state
		*/
    $scope.$on('$ionicView.beforeEnter', function () {
      if (AppBuilder.isBuilt && $scope.state === 'second') {
        goToFixture();
      }
    });
    /**
		* This event is emitted when is app is succesfuly build and ready.
		* so we can go to the next state, 'fixtures'
		*/
    $rootScope.$on('build.succeeded', function () {
      if ($scope.state === 'second') {
        goToFixture();
      }
    });
    /**
		* This event emitted when is app build failed.
		* set the buidFailed scope variable at true, which will display the retry button
		*/
    $rootScope.$on('build.failed', function () {
      $scope.buildFailed = true;
    });
    /**
		* This function, attached to the scope is use to allow the user to retry building the app
		*/
    $scope.retry = function () {
      $scope.buildFailed = false;
      AppBuilder.build();
    };
  }
]);
'use strict';
angular.module('shell.directives').directive('stopEvent', [function () {
    return {
      restrict: 'A',
      link: function (scope, element, attr) {
        element.bind('tap', function (e) {
          e.stopPropagation();
        });
      }
    };
  }]);
angular.module('shell.directives')
  .directive('scaledText', scaledTextDirective);

//@TODO: These should be less magic and added to .run perhaps
var TEXT_SCALES = [{
    charMinCount: 12,
    charMaxCount: 15,
    className: 'small-text'
},{
  charMinCount: 15,
  charMaxCount: 20,
  className: 'smaller-text'
}];

function scaledTextDirective($interpolate) {
  return {
    restrict: 'A',
    link: function(scope, element) {
      var MULTI_SPACES = /\s{2,}/g;
      var elementText =
        $interpolate(element.text())(scope).trim().replace(MULTI_SPACES,'');
      var textLength = elementText.length;

      TEXT_SCALES.forEach(function(scale) {
        var minLength = scale.charMinCount;
        var maxLength = scale.charMaxCount;

        if (textLength >= minLength && textLength < maxLength) {
          element.addClass(scale.className);
        }
      });
    }
  };
}

scaledTextDirective.$inject = ['$interpolate'];

'use strict';
angular.module('shell.directives').directive('bladeCard', [
  'AppBuilder',
  function (AppBuilder) {
    return {
      restrict: 'E',
      link: function (scope, element, attrs) {
        scope.contentUrl = AppBuilder.getBladeCardTemplate(attrs.template);
      },
      template: '<div ng-include="contentUrl"></div>'
    };
  }
]);
'use strict';
angular.module('shell.directives').directive('bladeSettings', [
  'AppBuilder',
  function (AppBuilder) {
    return {
      restrict: 'E',
      link: function (scope, element, attrs) {
        scope.contentUrl = AppBuilder.getBladeSettingsTemplate(attrs.template);
      },
      template: '<div ng-include="contentUrl"></div>'
    };
  }
]);
'use strict';
angular.module('shell.directives').directive('bladeMenu', [
  '$compile',
  function ($compile) {
    return {
      restrict: 'E',
      link: function (scope, element, attrs) {
        element.html(scope.$eval(attrs.template));
        $compile(element.contents())(scope);
      }
    };
  }
]);
'use strict';
angular.module('shell.directives').directive('loader', [
  '$timeout',
  function ($timeout) {
    return {
      require: 'loader',
      controller: [
        '$scope',
        function ngSwitchController($scope) {
          $scope.loader = true;
          $scope.content = false;
          $scope.contentHide = false;
        }
      ],
      priority: 1200,
      link: function (scope, element, attr) {
        scope.$on('$ionicView.afterLeave', function () {
          scope.loader = true;
          scope.content = false;
          scope.contentHide = true;
        });
        scope.$on('$ionicView.afterEnter', function () {
          scope.content = true;
          $timeout(function () {
            scope.contentHide = false;
            scope.loader = false;
          });
        });
      }
    };
  }
]).directive('loaderLoading', [
  '$compile',
  function ($compile) {
    return {
      priority: 1000,
      require: '^loader',
      multiElement: false,
      link: function (scope, element, attrs) {
        var child = element.children().attr('ng-if', 'loader==true');
        $compile(child)(scope);
      }
    };
  }
]).directive('loaderContent', [
  '$compile',
  function ($compile) {
    return {
      priority: 1000,
      require: '^loader',
      multiElement: false,
      link: function (scope, element, attrs) {
        //$element.children().attr('ng-if', "loadingState=='loaded'"); 
        attrs.$set('ng-if', 'content==true');
        attrs.$set('ng-hide', 'contentHide==true');
        $compile()(scope);
      }
    };
  }
]);
'use strict';
angular.module('shell.directives')
.directive('myZoom', [
  '$window',
  '$timeout',
  function ($window, $timeout) {
    return {
      restrict: 'A',
      scope: {
        items: '=items',
        itemWidth: '@',
        margin: '@'
      },
      link: function (scope, element, attrs) {
        scope.scaleFactor = [];
        var onScroll = function(e) {
          var itemCount = scope.items.length;
          if (e.detail && itemCount > 0 && scope.itemWidth) {
            var itemWidth = $window.innerHeight * scope.itemWidth;
            var margin = ($window.innerHeight * scope.margin);
            var scroll = e.detail.scrollLeft - margin;
            var ci = ( scroll / itemWidth);
            scope.currentItem = Math.round(ci);
            for (var i = itemCount - 1; i >= 0; i--) {
              var vl = (i - ci);
              var scaleFactor = ((vl * vl) * -1) + 1;
              if (scaleFactor < 0.5 || itemCount * scope.itemWidth < 1.5) {
                scaleFactor = 0.5;
              }
              scope.items[i].scaleFactor = scaleFactor;
              scope.items[i].zIndex = Math.round(scaleFactor*100);
            };
            $timeout(function() {
              scope.$apply();
            });
          }
        };
        angular.element(element.parent()[0].parentElement).bind('scroll', onScroll);
      }
    };
  }
]);
'use strict';
angular.module('shell.directives')
.directive('myClick', [
  '$window',
  '$rootScope',
  'AudioToolsService',
  'UserSettings',
  function ($window, $rootScope, AudioToolsService, UserSettings) {
    return {
      restrict: 'A',
      link: function (scope, element, attrs) {
        var element = element;
        var parentScrollingElement = angular.element(element.parent().parent()[0].parentElement);
        var click = AudioToolsService.createSound('sounds/click1');
        var centerOfWindow = parseInt($window.innerWidth/2);
        var el = element.children().children()[0];
        var halfBox = $window.innerHeight*0.38;
        var position = 0;
        
        var onScrollClick = function(e) {
          var centerOfBox = parseInt( el.getBoundingClientRect().left + halfBox);
          var significantMovement = Math.abs(centerOfBox - position) > 70;
          if(significantMovement){
            position = centerOfBox;
          }
          if ( centerOfBox  < (centerOfWindow +35) && centerOfBox  > (centerOfWindow -35) && significantMovement ) {
            click.play();
          }         
        };
        
        function enableClickOnScroll(element, parentScrollingElement, onScrollClick){
          parentScrollingElement.on('scroll', onScrollClick);
        }

        if(UserSettings.getData('soundEnable')){
          enableClickOnScroll(element, parentScrollingElement, onScrollClick);
        }

        $rootScope.$on('userSettings.sound.enabled', function(){
          enableClickOnScroll(element, parentScrollingElement, onScrollClick);
        });

        $rootScope.$on('userSettings.sound.disabled', function(){
          parentScrollingElement.off('scroll', onScrollClick);
        });

        
      }
    };
  }
]);
angular.module('shell.directives')
.directive('inputKeyboard', [
	'$rootScope',
	'$document',
	'$window',
	function($rootScope, $document, $window) {
		var margin = 10;
		return {
			restrict: 'A',
			link: function(scope, element, attrs) {
				var hasFocus = false;
				element.bind('focus', function() {
					console.log(ionic.Platform.isIOS());
					hasFocus = true;
				});
				element.bind('blur', function() {
					hasFocus = false;
				});
				window.addEventListener('native.keyboardshow', function(e) {
					if(ionic && ionic.Platform && ionic.Platform.isAndroid() == false) {
						return;
					}
					if (hasFocus == true) {
						var kh = e.keyboardHeight;
						var rect = angular.element(element)[0].getBoundingClientRect();
						var bot = $window.innerHeight - rect.bottom;
						var os = Math.max(0, (kh - bot) + margin );
						if (os > 0) {
							document.documentElement.style[ionic.CSS.TRANSFORM] = 'translate3d(0, -' + os + 'px, 0)';
							//ß$document[0].body.style[ionic.CSS.TRANSFORM] = 'translate3d(0, -' + os + 'px, 0)';							
						}
					}
			    });
			    window.addEventListener('native.keyboardhide', function(e) {
					console.log('keyboardhide');
					if(ionic && ionic.Platform && ionic.Platform.isAndroid() == false) {
						return;
					}			
			    	if (hasFocus == false) {
			    		document.documentElement.style[ionic.CSS.TRANSFORM] = '';							
			    		//$document[0].body.style[ionic.CSS.TRANSFORM] = 'translate3d(0, 0, 0)';
			    		focus = false;
			    	}
			    });
			}
		};
	}
]);
angular.module('shell.directives').directive('focusMe', [
  '$timeout',
  '$parse',
  function($timeout, $parse) {
    return {
      link: function(scope, element, attrs) {
        var model = $parse(attrs.focusMe);
        scope.$watch(model, function(value) {
          if(value === true) { 
            $timeout(function() {
              element[0].focus(); 
            });
          }
        });
        element.bind('blur', function() {
           console.log('blur');
           scope.$apply(model.assign(scope, false));
        });
      }
    };
  }
]);
/*
 * SimplePubSub from https://github.com/mbenford/ngTagsInput/blob/master/src/util.js
 * */
'use strict';

function SimplePubSub() {
    var events = {};
    return {
        on: function(names, handler) {
            names.split(' ').forEach(function(name) {
                if (!events[name]) {
                    events[name] = [];
                }
                events[name].push(handler);
            });
            return this;
        },
        trigger: function(name, args) {
            angular.forEach(events[name], function(handler) {
                handler.call(null, args);
            });
            return this;
        }
    };
};

angular.module('shell.directives')
.directive('onFinishRender', function ($timeout) {
    return {
        restrict: 'A',
        link: function (scope, element, attr) {
            if (scope.$last === true) {
                $timeout(function () {
                    scope.$emit('ngRepeatFinished');
                });
            }
        }
    }
})
.directive('tabSlideBox', [ '$timeout', '$window', '$ionicSlideBoxDelegate', '$ionicScrollDelegate',
	function($timeout, $window, $ionicSlideBoxDelegate, $ionicScrollDelegate) {
		'use strict';

		return {
			restrict : 'A, E, C',
			link : function(scope, element, attrs, ngModel) {
				
				var ta = element[0], $ta = element;
				$ta.addClass("tabbed-slidebox");
				if(attrs.tabsPosition === "bottom"){
					$ta.addClass("btm");
				}
				
				//Handle multiple slide/scroll boxes
				var handle = ta.querySelector('.slider').getAttribute('delegate-handle');
				
				var ionicSlideBoxDelegate = $ionicSlideBoxDelegate;
				if(handle){
					ionicSlideBoxDelegate = ionicSlideBoxDelegate.$getByHandle(handle);
				}
				
				var ionicScrollDelegate = $ionicScrollDelegate;
				if(handle){
					ionicScrollDelegate = ionicScrollDelegate.$getByHandle(handle);
				}
				
				function renderScrollableTabs(){
					var iconsDiv = angular.element(ta.querySelector(".tsb-icons")), icons = iconsDiv.find("a"), wrap = iconsDiv[0].querySelector(".tsb-ic-wrp"), totalTabs = icons.length;
					var scrollDiv = wrap.querySelector(".scroll");
					
					angular.forEach(icons, function(value, key){
					     var a = angular.element(value);
					     a.on('click', function(){
					    	 ionicSlideBoxDelegate.slide(key);
					     });

						if(a.attr('icon-off')) {
							a.attr("class", a.attr('icon-off'));
						}
					});
					
					var initialIndex = attrs.tab;
					if (initialIndex && typeof initialIndex == 'string') {
						initialIndex = parseInt(initialIndex);
					}
					console.log(initialIndex);
					//Initializing the middle tab
					if(typeof attrs.tab === 'undefined' || (totalTabs <= initialIndex) || initialIndex < 0){
						initialIndex = Math.floor(icons.length/2);
					}
					
					//If initial element is 0, set position of the tab to 0th tab 
					if (initialIndex == 0) {
						setPosition(0);
					}
					
					$timeout(function() {
						ionicSlideBoxDelegate.slide(initialIndex);
					}, 0);
				}
				function setPosition(index) {					
					//$timeout(function() {
						var iconsDiv = angular.element(ta.querySelector(".tsb-icons")), icons = iconsDiv.find("a"), wrap = iconsDiv[0].querySelector(".tsb-ic-wrp"), totalTabs = icons.length;
						var scrollDiv = wrap.querySelector(".scroll");
						var curEl = angular.element(icons[index]);
						var prvEl = angular.element(iconsDiv[0].querySelector(".active"));
						if (curEl && curEl.length) {
							if (prvEl.attr('icon-off')) {
								prvEl.attr("class", prvEl.attr('icon-off'));
							} else {
								prvEl.removeClass("active");
							}
							if(curEl.attr('icon-on')) {
								curEl.attr("class", curEl.attr('icon-on'));
							}
							curEl.addClass("active");
							$timeout(function() {
								var middle = iconsDiv[0].offsetWidth/2;
								var curElWidth = curEl[0].offsetWidth;
								var curElLeft = curEl[0].offsetLeft;
								var leftStr = (middle  - (curElLeft) -  curElWidth/2 + 15);
								//If tabs are not scrollable
								if (!scrollDiv) {
									var leftStr = (middle  - (curElLeft) -  curElWidth/2 + 15) + "px";
									wrap.style.webkitTransform =  "translate3d("+leftStr+",0,0)" ;
								} else {
									//If scrollable tabs
									var wrapWidth = wrap.offsetWidth;
									var currentX = Math.abs(getX(scrollDiv.style.webkitTransform));
									var leftOffset = 100;
									var elementOffset = 40;
									//If tabs are reaching right end or left end
									if(((currentX + wrapWidth) < (curElLeft + curElWidth + elementOffset)) || (currentX > (curElLeft - leftOffset))){
										if(leftStr > 0){
											leftStr = 0;
										}
										//Use this scrollTo, so when scrolling tab manually will not flicker
										ionicScrollDelegate.scrollTo(Math.abs(leftStr), 0, true);
									}
								}
							}, 100);
						}
					//}, 10);
				};
				function getX(matrix) {
					matrix = matrix.replace("translate3d(","");
					matrix = matrix.replace("translate(","");
					return (parseInt(matrix));
				}
				var events = scope.events;
				events.on('slideChange', function(data){
					setPosition(data.index);
				});
				events.on('ngRepeatFinished', function(ngRepeatFinishedEvent) {
					renderScrollableTabs();
				});
				
				renderScrollableTabs();
			},
			controller : function($scope, $attrs, $element) {
				$scope.events = new SimplePubSub();
				
				$scope.slideHasChanged = function(index){
					$scope.events.trigger("slideChange", {"index" : index});
					$timeout(function(){if($scope.onSlideMove) $scope.onSlideMove({"index" : eval(index)});},100);
				};
				
				$scope.$on('ngRepeatFinished', function(ngRepeatFinishedEvent) {
					$scope.events.trigger("ngRepeatFinished", {"event" : ngRepeatFinishedEvent});
				});
			}
		};

	} 
]);

angular.module('shell.directives')
.directive('flippy', function() {
	return {
		restrict: 'EA',
		link: function($scope, $elem, $attrs) {
			$scope.flipped = false;
			var options = {
				flipDuration: ($attrs.flipDuration) ? $attrs.flipDuration : 400,
				timingFunction: 'ease-in-out',
			};

			// setting flip options
			angular.forEach(['flippy-front', 'flippy-back'], function(name) {
				var el = $elem.find(name);
				if (el.length == 1) {
					angular.forEach(['', '-ms-', '-webkit-'], function(prefix) {
						angular.element(el[0]).css(prefix + 'transition', 'all ' + options.flipDuration/1000 + 's ' + options.timingFunction);
					});
				}
			});

			var first = true;
			/**
			 * behaviour for flipping effect.
			 */
			$scope.flip = function() {
				if ($elem[0].classList.length == 0) {
					$elem.toggleClass($scope.flipped ? 'unflipped' : 'flipped');
				} else {
					$elem.toggleClass('unflipped');
					$elem.toggleClass('flipped');						
				}
				
				$scope.flipped = !$scope.flipped;
			}

		}
	};
});
/**
* @module blade.authentification
* @description This module contains the authentification view.
* @requires $stateProvider
*/
angular.module('blades.authentification', ['facebook']).config([
  '$stateProvider',
  '$httpProvider',
  'FacebookProvider',
  'clubProvider',
  function ($stateProvider, $httpProvider, FacebookProvider, clubProvider) {
    var club = clubProvider.$get();
    /***
  	* HERE IS THE WANKY WAY TO HANDLE THE TOKENS, WHICH REQUIRE A BACKEND UPDATE..
  	*/
    $httpProvider.interceptors.push('AuthHttpInterceptor');
    /**
  	* Configure facebook for development in browser
  	*/
    FacebookProvider.init({
      appId: club.fbAppId,
      version: 'v2.1'
    });
    $stateProvider.state('shell.login', {
      url: 'login',
      hideLayout: true,
      hideHeader: true,
      forceGoBack: true,
      views: {
        'tab-main': {
          templateUrl: 'app/blades/authentification/view/templates/loginView.html',
          controller: 'LoginController'
        }
      }
    }).state('shell.register', {
      url: 'register',
      hideLayout: true,
      hideHeader: true,
      forceGoBack: true,
      views: {
        'tab-main': {
          templateUrl: 'app/blades/authentification/view/templates/registerView.html',
          controller: 'RegisterController'
        }
      }
    }).state('shell.forgotPassword', {
      url: 'forgotpassword',
      hideLayout: true,
      hideHeader: true,
      views: {
        'tab-main': {
          templateUrl: 'app/blades/authentification/view/templates/forgotPasswordTemplate.html',
          controller: 'ForgotPasswordController'
        }
      }
    });
  }
]);
/**
* @ngdoc services
* @name AuthService
* @description Provide authentification functions (Facebook & playr)
* @property {object} currentUser hold the current user information, null if the current user is not login
*/
'use strict';
angular.module('blades.authentification').factory('AuthService', [
  '$q',
  '$timeout',
  '$resource',
  'CacheFactory',
  'FacebookAuthService',
  'UserNameAuthService',
  'usersService',
  'club',
  'Urls',
  '$rootScope',
  'AppBuilder',
  'ToastService',
  'TOAST',
  'pachingAdService',
  function ($q, $timeout, $resource, CacheFactory, FacebookAuthService, UserNameAuthService, usersService, club, Urls, $rootScope, AppBuilder, ToastService, TOAST, pachingAdService) {
    /***
    * set the cache private attribute, which store the playr token
    */
    var _cache = CacheFactory.get('auth');
    if (!_cache) {
      _cache = new CacheFactory('auth', { storageMode: 'localStorage' });
    }
    /***
    * $resources which point to the authentication endpoints
    * TODO Make resources ... 
    */
    var _authenticateResources = {
      'Logging in': $resource( Urls.apiUrl + '/authentications').save,
      'forgotPassword': $resource( Urls.apiUrl + '/authentications').save,
      'Signing in': $resource(Urls.apiUrl + '/users').save,
      'validateSession': $resource(Urls.apiUrl + '/authentications').get,
      'logout': $resource(Urls.apiUrl + '/authentications/:token').delete
    };
    /***
    * Dictionnary of authentification provider
    * @example: _provider['facebook'].login(...)
    */
    var providers = {
      facebook: FacebookAuthService,
      playr: UserNameAuthService
    };
    /***
    * @description private function, which pushes authentication token into the default http header in order
    * to authenticate all future requests. Also stores the token within
    * the factory in order to autoLogin the user next time he opens the app
    * @param token {string}
    * @return void
    */
    var setToken = function (token) {
      //$http.defaults.headers.common.token = token;
      // will be replace by the line above
      $rootScope.token = token;
      dataService.token = token;
      _cache.put('token', token);
    };

    var playrAuthenticate = function(request, provider, authData) {
      var q = $q.defer();

      if(ionic.Platform.isIOS()){
        authData._request.currentPlatform = "ios";
      }else if(ionic.Platform.isAndroid()){
        authData._request.currentPlatform = "android";
      }else{
        authData._request.currentPlatform = "web";
      }

      doAuth(q, request, authData, provider);

      function doAuth(q, request, authData, provider){
        // perform the appropriate request (login/signup) to our API with this authentification data
        _authenticateResources[request](authData).$promise.then(function (response) {
          if(request == 'Signing in' && !response._data.gatewayAd){
              pachingAdService.getLinkData()
                .then(function getLinkDataSuccess (link) {

                    usersService.update(response._data.id, {gatewayAd: link.linkId})
                      .then(function(user){
                        pachingAdService.regiserUser(user.id, {id:user.gatewayAd});
                      })

                });
          }
          // set the token as a default header for each http request
          setToken(response.token);
          // fetch the current user data
          usersService.find(response._data.id, {
            params: {
              'with': ['products']
            }
          })
          .then(function (user) {
            dataService.currentUser = user;

            usersService.bindOne($rootScope, 'user', user.id);
            var onBind = function() {
              if ($rootScope.user) {
                $rootScope.$broadcast('user.logged.in');
                AppBuilder.rebuild().finally(function() {
                  if (derigister) {
                    derigister();                
                  }
                  $timeout(function() {
                    ToastService.hideNativeToast();
                  }, 1000);
                  q.resolve(dataService.currentUser);                
                });
                return true;
              }
              return false;
            };
            if (onBind() == false) {
              var derigister = $rootScope.$watch(function() { return $rootScope.user; }, onBind);            
            }
          }, q.reject);
        }, function(err) {
          console.log(err);
          if (provider === 'facebook' && request === 'Logging in' && err.status === 401) {
            playrAuthenticate('Signing in', 'facebook', authData).then(q.resolve, q.reject);
          } else {
            q.reject(err);
          }
         });
      }

      return q.promise;
    };

    /**
    * authenticate
    * @function
    * @description this function allow to login or signup users using either facebook or playr (email/password)
    * @memberOf AuthService
    * @param request {string} can be either 'login' or 'signup'
    * @param provider {string} name of the authentification provider, so far only 'facebook' and 'playr'
    * are supported.
    * @param userData {object} data required by the API to authenticate a user. eg with playr provider, email,displayName and password must be set
    * @return promise reject with an error and resolve with the freshly logged in user
    */
    var authenticate = function (request, provider, userData) {
      var q = $q.defer();
      ToastService.showNativeToast(request, '', TOAST.types.loading_progress, true);

      userData.clubId = club.clubId;
      // get authentification from the authentification provider, ex: Facebook
      providers[provider].login(userData).then(function (authData) {
        playrAuthenticate(request, provider, authData).then(function() {
          q.resolve();
        }, function(err) {
          ToastService.hideNativeToast();
          q.reject(err);
        });
      }, function(err) {
        ToastService.hideNativeToast();
        q.reject(err);
      });

      return q.promise;
    };

    var getClubUser = function (users) {
      for (var i = users.length - 1; i >= 0; i--) {
        if (users[i].clubId === club.clubId) {
          return users[i];
        }
      }
      return null;
    };
    /**
    * autoLogin
    * @function
    * @memberOf AuthService
    * @description this function try to read the token stored in LocalStore, from a previous session,
    * and try to validate this token
    * @return promise reject with an error, resolve with the user authenticated with the token found
    */
    var autoLogin = function () {
      var q = $q.defer();
      var token = _cache.get('token');
      if (token) {
        $rootScope.token = token;
        // validate the token
        _authenticateResources.validateSession().$promise.then(function (response) {
          var user = getClubUser(response._data);
          if (user) {
            // fetch the user data with the validated token
            usersService.find(user.userId, {
              params: {
                'with': ['products']
              }
            }).then(function (user) {
              dataService.token = token;
              dataService.currentUser = user;
              usersService.bindOne($rootScope, 'user', user.id);  
              $timeout(function() {
                $rootScope.$broadcast('user.logged.in');
                q.resolve(dataService.currentUser);                
              });
            }, q.reject);
          } else {
            q.reject('No user found for this clubId');
          }
        }, q.reject);
      } else {
        q.reject('No session found');
      }
      return q.promise;
    };
    /**
    * logout
    * @function
    * @memberOf AuthService
    * @description logout the current user, and remove its token from localStorage
    * @return promise reject with an error, empty resolve
    */
    var logout = function () {
      var q = $q.defer();
      ToastService.showNativeToast('Logging out', '', TOAST.types.loading_progress, true);
      _authenticateResources.logout({ token: dataService.token }).$promise
      .finally(function () {
        $rootScope.token = null;
        dataService.token = null;
        dataService.currentUser = null;
        _cache.remove('token');
        usersService.ejectAll();
        AppBuilder.rebuild(false, false).then(function() {
          console.log('Reload done !');
          $rootScope.$broadcast('user.logged.out');
          ToastService.hideNativeToast();        
        }, function() {
          ToastService.hideNativeToast();
          console.log('fail to reload');
        });
        q.resolve();
      });
      return q.promise;
    };

    var forgotPassword = function(email) {
      return _authenticateResources.forgotPassword({
        '_request': {
          'clubId': club.clubId,
          'email': email,
          'forgotPassword': true
        }
      }).$promise;
    };
    /**
    * isAuthenticate
    * @function
    * @memberOf AuthService
    * @description is used to inform about the current user login state
    * @return {bool} true is the current user is login, false otherwise
    */
    var isAuthenticated = function () {
      return dataService.currentUser !== null;
    };
    var dataService = {
      /* PROPERTIES */
      currentUser: null,
      token: null,
      /* METHOD */
      authenticate: authenticate,
      autoLogin: autoLogin,
      isAuthenticated: isAuthenticated,
      logout: logout,
      forgotPassword: forgotPassword
    };

    return dataService;
  }
]);
/**
* @ngdoc services
* @name FacebookAuthService
* @description Provide facebook authentification using $cordovaFacebook's plugin or inappbrowser fallback
*/
'use strict';
angular.module('blades.authentification').factory('FacebookAuthService', [
  '$q',
  '$cordovaFacebook',
  'Facebook',
  '$ionicPlatform',
  function ($q, $cordovaFacebook, Facebook, $ionicPlatform) {
    var FB_FIELDS = 'name,first_name,last_name,id,gender,email';
    var getPlugin = function () {
      var q = $q.defer();
      $ionicPlatform.ready(function () {
        q.resolve($cordovaFacebook);
      });
      return q.promise;
    };
    var prepareProfile = function(userProfile) {
      var profile = {};
      if (userProfile.gender) {
        profile.gender = userProfile.gender.toUpperCase();
      }
      if (userProfile.name) {
        profile.username = userProfile.name;
      }
      if (userProfile.first_name) {
        profile.realFirstName = userProfile.first_name;
      }
      if (userProfile.last_name) {
        profile.realLastName = userProfile.last_name;
      }
      if(userProfile.email) {
        profile.email = userProfile.email;
      }
      return profile;
    };

    var webLoginFallback = function () {
      var q = $q.defer();
      Facebook.getLoginStatus(function (response) {
        if (response.status === 'connected') {
          Facebook.api('/me?fields=' + FB_FIELDS, function(me) {
            if (!me || me.error) {
              q.reject(me.error);
            } else {
              response.userProfile = prepareProfile(me);
              q.resolve(response);
            }
          });
        } else {
          Facebook.login(function (response) {
            if (response && response.authResponse) {
              Facebook.api('/me?fields=' + FB_FIELDS, function(me) {
                if (me.error) {
                  q.reject(me.error);
                } else {
                  response.userProfile = prepareProfile(me);
                  q.resolve(response);
                }
              });
            } else {
              q.reject();
            }
          }, {scope: 'public_profile,user_friends,email'});
        }
      });
      return q.promise;
    };
    var appLogin = function(facebookPlugin) {
      var q = $q.defer();
      facebookPlugin.login(['public_profile','user_friends', 'email']).then(function(response) {
        facebookPlugin.api('me?fields=' + FB_FIELDS).then(function(me) {
          response.userProfile = prepareProfile(me);
          q.resolve(response);
        }, q.reject);
      }, q.reject);
      return q.promise;
    };
    var login = function (userData) {
      var q = $q.defer();
      getPlugin().then(function (facebookPlugin) {
        var login;
        if (ionic.Platform.isWebView() || !facebookPlugin) {
          login = appLogin(facebookPlugin);
        } else {
          login = webLoginFallback();
        }
        login.then(function (response) {
          var res = response.authResponse;
          var expirationDate = new Date(new Date().getTime() + parseInt(res.expiresIn));
          var authData = {
            '_request': {
              'clubId': userData.clubId,
              'facebook': {
                'id': res.userID,
                'token': res.accessToken,
                'expiration_date': expirationDate
              }
            }
          };
          authData._request.acceptedChelseaPartnerTerms = !!userData.acceptedChelseaPartnerTerms;
          angular.extend(authData._request, response.userProfile);
          q.resolve(authData);
        }, function (error) {
          console.log(error);
          q.reject(error);
        });
      });
      return q.promise;
    };

    var dataService = {
      /* PROPERTIES */
      /* METHOD */
      login: login
    };
    return dataService;
  }
]);

/**
* @ngdoc services
* @name UserNameAuthService
* @description Service created for AuthService consistency
*/
'use strict';
angular.module('blades.authentification').factory('UserNameAuthService', [
  '$q',
  function ($q) {
    var login = function (userData) {
      var q = $q.defer();
      if (userData.email && userData.password) {
        q.resolve({ _request: userData });
      } else {
        q.reject('email or/and password are missing');
      }
      return q.promise;
    };
    var dataService = {
      /* PROPERTIES */
      /* METHOD */
      login: login
    };
    return dataService;
  }
]);
/***
* DIRTY WAY TO INJECT THE PLAYR'S TOKEN, WHICH EXIST WHILE THE BACKEND IS NOT UPDATED
*/
angular.module('blades.authentification').factory('AuthHttpInterceptor', [
  '$q',
  '$rootScope',
  function ($q, $rootScope) {
    return {
      // optional method
      'request': function (request) {
        // Using the dirty way to get the token, via the rootScope.
        if ($rootScope.token) {
          // if the request goes to the api
          if (request.url.indexOf('/api/') !== -1) {
            request.url += (request.url.indexOf('?') === -1 ? '?' : '&') + 'token=' + $rootScope.token;
          }
        }
        return request;
      }
    };
  }
]);
'use strict';
angular.module('blades.authentification').controller('LoginController', [
  '$scope',
  '$rootScope',
  'AuthService',
  'Urls',
  '$ionicHistory',
  'clubsService',
  'club',
  'ToastService',
  'TOAST',
  'ModalService',
  function ($scope, $rootScope, AuthService, Urls, $ionicHistory, clubsService, club, ToastService, TOAST, ModalService) {
    console.log('LoginController');
    $scope.scorePredictorLogo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/scorePredictorLogo.png');
    $scope.currentClub = clubsService.get(club.clubId);
    if ($scope.currentClub) {
      $scope.backGroundCss = { 'background-color': $scope.currentClub.properties.primaryColourHex };
    }
    $scope.chineseVersion = !!club.chineseVersion;
    $scope.hasAgreements = !!club.agreementsId;
    $scope.agreements = club.agreements;
    $scope.credentials = {};
    $scope.back = function () {
      $ionicHistory.goBack();
    };

    $scope.focusPasswordInput = false;
    $scope.focusEmailInput = false;
    $scope.submit = function() {
      if (!$scope.credentials.email || $scope.credentials.email.length === 0) {
        // set focus on the email input
        $scope.focusEmailInput = true;
      } else if (!$scope.credentials.password || $scope.credentials.password.length === 0) {
        // set focus on the password input
        $scope.focusPasswordInput = true;
      } else {
        // the email & password seems to be set, so close the keyboard and try to login..
        $scope.login('playr');
        if (typeof cordova !== 'undefined') {
          cordova.plugins.Keyboard.close();
        }
      }
    };

    $scope.terms = function() {
      var termsUrl = 'http://' + club.subdomain + '.playabl.com/terms.php';
      if (window.cordova && cordova.InAppBrowser) {
        cordova.InAppBrowser.open(termsUrl, '_system');
      } else {
        window.open(termsUrl, 'blank');
      }
    };
    $scope.hasAgrements = !!club.agreementsId;
    $scope.agreements = club.agreements;
    $scope.details = {
      acceptedChelseaPartnerTerms: true
    };
    $scope.clubTerms = function() {
      if ($scope.agreements.mainContentDisplayMethod == 'popup') {
        ModalService.show('app/blades/authentification/popups/templates/agreementsPopupTemplate.html', 'AgreementsPopupController', $scope.agreements);
      } else {
        var termsUrl = 'http://' + club.subdomain + '.playabl.com/terms.php';
        if (window.cordova && cordova.InAppBrowser) {
          cordova.InAppBrowser.open(termsUrl, '_system');
        } else {
          window.open(termsUrl, 'blank');    
        }
      }
    };

    $scope.login = function (provider) {
      if (provider === 'facebook') {
        $scope.credentials.acceptedChelseaPartnerTerms = $scope.details.acceptedChelseaPartnerTerms;        
      }
      AuthService.authenticate('Logging in', provider, $scope.credentials).then(function() {
        $rootScope.goBack();
      }, function (err) {
        if (typeof err == 'object') {
          if (err.data && err.data.userMessage) {
            err = err.data.userMessage;
          } else {
            err = 'Oups something wrong happened';
          }
        }
        ToastService.showNativeToast('ERROR!', err, TOAST.types.error);
      });
    };
  }
]);

'use strict';
angular.module('shell.controllers').controller('RegisterController', [
  '$scope',
  '$rootScope',
  'AuthService',
  'Urls',
  '$state',
  '$ionicHistory',
  'club',
  'clubsService',
  'ISO3166',
  'ToastService',
  'TOAST',
  'ModalService',
  function ($scope, $rootScope, AuthService, Urls, $state, $ionicHistory, club, clubsService, ISO3166, ToastService, TOAST, ModalService) {
    $scope.scorePredictorLogo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/scorePredictorLogo.png');
    $scope.currentClub = clubsService.get(club.clubId);
    $scope.backGroundCss = { 'background-color': $scope.currentClub.properties.primaryColourHex };
    $scope.hasAgrements = !!club.agreementsId;
    $scope.agreements = club.agreements;
    $scope.details = {
      acceptedChelseaPartnerTerms: true
    };
    $scope.currentClub = clubsService.get(club.clubId);

    $scope.submit = function() {
      if (!$scope.details.email || $scope.details.email.length === 0) {
        $scope.focusEmailInput = true;
      } else if (!$scope.details.displayName || $scope.details.displayName.length === 0) {
        $scope.focusUsernameInput = true;
      } else if (!$scope.details.password || $scope.details.password.length === 0) {
        $scope.focusPasswordInput = true;
      } else {
        if ($scope.hasAgrements) {
          $scope.details.agreements = [{
            'agreementId': club.agreementsId,
            'isAccepted': $scope.details.acceptedChelseaPartnerTerms
          }];
        }
        if ($scope.details.acceptedChelseaPartnerTerms !== false) {
          $scope.details.acceptedChelseaPartnerTerms = true;          
        }
        $scope.details.acceptedChelseaMainTerms = true;        
        $scope.signup('playr');
      }
    };

    $scope.terms = function() {
      if ($scope.agreements.mainContentDisplayMethod == 'popup') {
        ModalService.show('app/blades/authentification/popups/templates/agreementsPopupTemplate.html', 'AgreementsPopupController', $scope.agreements);
      } else {
        var termsUrl = 'http://' + club.subdomain + '.playabl.com/terms.php';
        if (window.cordova && cordova.InAppBrowser) {
          cordova.InAppBrowser.open(termsUrl, '_system');
        } else {
          window.open(termsUrl, 'blank');    
        }
      }
    };
    
    $scope.back = function() {
      $rootScope.goBack();
    };
    $scope.signup = function (provider) {
      AuthService.authenticate('Signing in', provider, $scope.details)
      .then(function () {
        ToastService.showNativeToast('Logged in successfully', '', TOAST.types.success);
        // go back 2 view away to skip the login view
        $rootScope.goBack(-2);
      }, function (err) {
        if (typeof err === 'object') {
          if (err.data && err.data.userMessage) {
            err = err.data.userMessage; 
          } else {
            err = 'Oups something wrong happened';            
          }
        }
        ToastService.showNativeToast('ERROR!', err, TOAST.types.error);
      });
    };
  }
]);

angular.module('blades.authentification').controller('ForgotPasswordController', [
  '$scope',
  '$rootScope',
  'AuthService',
  'Urls',
  'club',
  'ToastService',
  'TOAST',
  'clubsService',
  function ($scope, $rootScope, AuthService, Urls, club, ToastService, TOAST, clubsService) {
    console.log('ForgotPasswordController');
    $scope.scorePredictorLogo = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/scorePredictorLogo.png');
    $scope.currentClub = clubsService.get(club.clubId);
    if ($scope.currentClub) {
      $scope.backGroundCss = { 'background-color': $scope.currentClub.properties.primaryColourHex };
    }
    $scope.details = {};
    $scope.submit = function() {
      if ($scope.details.email && $scope.details.email.length > 0) {
        if (typeof cordova !== 'undefined') {          
          cordova.plugins.Keyboard.close();
        }
        AuthService.forgotPassword($scope.details.email)
        .then(function() {
          ToastService.showNativeToast('Reset password form has been sent to your email adress', '', TOAST.types.success);
          $rootScope.goBack();
        }, function(err) {
          console.log(err);
        });
      }
    };
    $scope.back = function() {
      $rootScope.goBack();
    }

  }
]);
'use strict';
angular.module('blades.authentification')

.directive('dynamicHtml', function ($compile) {
  return {
    restrict: 'A',
    replace: true,
    link: function (scope, ele, attrs) {
      scope.$watch(attrs.dynamicHtml, function(html) {
        ele.html(html);
        $compile(ele.contents())(scope);
      });
    }
  };
})

.controller('AgreementsPopupController', [
  '$scope',
  '$compile',
  'parameters',
  function ($scope, $compile, parameters) {
    
    $scope.agreements = parameters;

    $scope.goToUrl = function(link){
      if (window.cordova && cordova.InAppBrowser) {
        cordova.InAppBrowser.open(link, '_system');
      } else {
        window.open(link, 'blank');    
      }
    }

    $scope.close = function() {
      $scope.closeModal();
    };

  }
]);
/**
* @module blade.fixture
* @description This module contains the fixture card
*/
angular.module('blades.fixture', []).run(function () {
}).config([
  '$stateProvider',
  function ($stateProvider) {
    $stateProvider.state('shell.fixtures', {
      url: 'fixtures',
      requiresLogin: false,
      canGoBack: false,
      isRootView: true,
      showMainLogo: true,
      showNewsFeed: true,
      views: {
        'tab-main': {
          templateUrl: 'app/blades/fixture/views/templates/fixtureListTemplate.html',
          controller: 'FixtureListController',
          resolve: {
            fixtures: function($timeout, $q) {
              var q = $q.defer();
              $timeout(function() {
                q.resolve();
              }, 750);
              return q.promise;
            }
          }
        }
      }
    });
  }
]);
'use strict';
angular.module('blades.fixture').controller('FixtureListController', [
  '$scope',
  '$rootScope',
  '$timeout',
  '$state',
  'fixturesService',
  'Settings',
  '$ionicScrollDelegate',
  'ChallengeWonService',
  function ($scope, $rootScope, $timeout, $state, fixturesService, Settings, $ionicScrollDelegate, ChallengeWonService) {
    var CHALLENGE_WON_DELAY = 1000;
    // Call before we enter in this view, even if the view is cached, which is not the case for the code outside of this function
    var scrollToMiddle = function() {
      $timeout(function() {
        var handle = $ionicScrollDelegate.$getByHandle('fixture-scroll');
        if (handle && handle._instances && handle._instances[0].$element[0]) {
          var elementwidth = handle._instances[0].$element[0].offsetWidth;
          var scrollWidth = handle._instances[0].$element[0].children[0].scrollWidth;
          var scrollTo = (scrollWidth/2) - (elementwidth/2);
          $ionicScrollDelegate.$getByHandle('fixture-scroll').scrollTo(scrollTo, 0, true);
        }
      }, 200);
    };
    var first = true;

    $scope.getMore = function() {
      var handle = $ionicScrollDelegate.$getByHandle('fixture-scroll');

      Settings.getMore()
      .then(function() {
        for (var i = $scope.fixtures.length - 1; i >= 0; i--) {
          $scope.fixtures[i].attachClubsDetails();
        }
        handle.scrollTo(800, true);
      }, function(err) {
        console.log(err);
      });
    };

    $scope.goToChallenges = function(fixture) {
        $state.go('shell.challenges', {
            fixtureId: fixture.id
        });
    };


    $scope.show = false;
    $scope.$on('$stateChangeStart', function (event, toState) {
      if (toState.name !== 'shell.fixtures' && toState.name !== 'shell.login' ) {
        $scope.show = false;
      }
    });

    $scope.ready = false;
    $scope.$on('$ionicView.afterLeave', function () {
      $scope.ready = false;
    });

    $scope.$on('$ionicView.afterEnter', function () {
      $scope.ready = true;
      $scope.show = true;
      fixturesService.bindAll($scope, 'fixtures', {
        orderBy: [
          ['startTime', 'ASC']
        ]
      }, function() {
          if (first) {
            first = false;
            scrollToMiddle();

            if($rootScope.user){
              ChallengeWonService.checkWonChallenges($rootScope.user.id);

              if(ChallengeWonService.challengesWon) {
                $timeout(function(){
                  ChallengeWonService.fireChallengeWon();
                }, CHALLENGE_WON_DELAY);
              }
            }
          }
      });
    });
  }
]);

/**
* @name fixturesViewDirective
* @requires {Object} fixturesService Service get fixtures from api 
* @requires {Object} clubsService Service get clubs from api
**/
angular.module('blades.fixture').directive('fixtureCard', [
  'fixturesService',
  'clubsService',
  '$rootScope',
  'Settings',
  'challengesService',
  'Urls', 
  'club',
  'userChallengesService',
  '$timeout',
  function (fixturesService, clubsService, $rootScope, Settings, challengesService, Urls, club, userChallengesService, $timeout) {
    return {
      name: 'fixtureCard',
      restrict: 'E',
      priority: 100,
      templateUrl: 'app/blades/fixture/card/templates/fixtureCardTemplate.html',
      link: function (scope) {
        var timeSponsorUrl = 'sports/football/clubs/' + club.clubId + '/time.png';
        scope.backgroundTimeSponsor = {
          'background-image': 'url(' + Urls.urlBuilder(timeSponsorUrl) + ')'
        };
        
        $timeout(function () {  
          userChallengesService.bindAll(scope, 'userChallenges', {where: {fixtureId: scope.fixture.fixtureId}});
        });
      }
    };
  }
]);
angular.module('blades.share', []);
angular.module('blades.share').directive('shareButton', [
  'SharePopupService',
  'club',
  function (SharePopupService, club) {
    return {
      scope: {
        caption: '@',
        picture: '@',
        description: '@',
        title: '@',
        url: '@',
        size: '@'        
      },
      templateUrl: 'app/blades/share/shareButton/templates/shareButtonTemplate.html',
      link: function(scope) {
        scope.chineseVersion = club.chineseVersion;
        if(scope.size === 'small'){
          scope.big = false;
        }else{
          scope.big = true;
        }
        scope.openSharePopup = function () {
          SharePopupService.show({
            title: scope.title,
            caption: scope.caption,
            description: scope.description,
            url: scope.url,
            picture: scope.picture
          });
        };
      }
    };
  }
]);
angular.module('blades.share').factory('ShareService', [
  '$cordovaSocialSharing',
  '$cordovaFacebook',
  'Facebook',
  '$ionicPlatform',
  '$q',
  'DeepLinkService',
  '$rootScope',
  function ($cordovaSocialSharing, $cordovaFacebook, Facebook, $ionicPlatform, $q, DeepLinkService, $rootScope) {
    var ShareService = {
      useWebView: !ionic.Platform.isWebView()
    };
    var getSocialSharePlugin = function() {
      var q = $q.defer();
      $ionicPlatform.ready(function () {
        q.resolve($cordovaSocialSharing);
      });
      return q.promise;
    }
    var getFacebookPlugin = function () {
      var q = $q.defer();
      $ionicPlatform.ready(function () {
        q.resolve($cordovaFacebook);
      });
      return q.promise;
    };
    var inviteFacebook = function (shareData) {
      var q = $q.defer();
      if (window.facebookConnectPlugin) {
        window.facebookConnectPlugin.appInvite({ 
          caption: shareData.caption, 
          picture: shareData.picture,
          url: "https://fb.me/729824883828003"
        },function (result) {
          console.log(result);
          q.resolve();
        }, function (err) { 
          console.log(err);
          q.reject(err); 
        });
      } else {
        q.reject();
      }
      return q.promise;
    };
    var inviteWhatsapp = function(shareData) {
      var q = $q.defer();
      getSocialSharePlugin().then(function(sharePlugin) {
        sharePlugin.shareViaWhatsApp(shareData.caption + ' ' + shareData.url, null, null, q.resolve, q.reject);
      });
      return q.promise;
    };
    var shareFacebook = function (shareData) {
      var q = $q.defer();
      getFacebookPlugin().then(function (facebookPlugin) {
        facebookPlugin.showDialog({
          method: 'share',
          href: shareData.url,
          caption: shareData.caption,
          title: shareData.title,
          picture: shareData.picture,
          description: shareData.description
        }).then(function (result) {
          q.resolve(result);
        }, function (error) {
          q.reject(error);
        });
      }, q.reject);
      return q.promise;
    };
    ShareService.invite = function(provider, shareData) {
      // create a trackabl link 
      DeepLinkService.createLink(provider, 'invite', {})
      .then(function(link) {
        if (link) {
          shareData.url = link;
        }
        if (provider == 'facebook') {
          return inviteFacebook(shareData);
        } else if (provider === 'whatsapp') {
          return inviteWhatsapp(shareData);
        } else {
          return $q.reject('Not supported provider: ' + provider);
        }
      });
    };
    ShareService.share = function (provider, shareData) {
      DeepLinkService.createLink(provider, 'share', {})
      .then(function(link) {
        if (link) {
          shareData.url = link;
        }
        if (provider === 'facebook') {
          if (ShareService.useWebView) {
            Facebook.ui({
              method: 'feed',
              display: 'popup',
              link: shareData.url,
              redirect_url: shareData.url,
              name: shareData.title,
              caption: shareData.caption,
              description: shareData.description,
              picture: shareData.picture
            }, function afterShare(response) {
              console.log(response);
            });
          } else {
            shareFacebook(shareData);          
          }
        } else if (provider === 'twitter') {
          if (ShareService.useWebView) {
            window.open('https://twitter.com/intent/tweet?text=' + shareData.caption, '_blank');
          } else {
            $cordovaSocialSharing.shareViaTwitter(shareData.caption, shareData.picture, shareData.url);          
          }
        }
      });      
    };
    return ShareService;
  }
]);
angular.module('blades.share').factory('SharePopupService', [
  '$rootScope',
  'ModalService',
  function ($rootScope, ModalService) {
    var SharePopupService = {
      shareData: null
    };

    SharePopupService.show = function (shareData) {
      SharePopupService.shareData = shareData;
      ModalService.show('app/blades/share/sharePopup/template/sharePopupTemplate.html', 'SharePopupController', shareData);
    };

    return SharePopupService;
  }
]);

'use strict';
angular.module('blades.share').controller('SharePopupController', [
  '$scope',
  'ShareService',
  'parameters',
  '$timeout',
  function ($scope, ShareService, parameters, $timeout) {

    $scope.useWebView = ShareService.useWebView;
    
    $scope.shareData = parameters;

    console.log(parameters);
    $scope.showDelayed = true;
        
    $scope.share = function(provider) {
      console.log(parameters);
      ShareService.share(provider, $scope.shareData);
    };
  }
]);

angular.module('blades.newsFeed', [])
.run([
	'$rootScope',
	function($rootScope) {
		$rootScope.showNewsFeed = true;
		$rootScope.$on('$stateChangeSuccess', function (evt, toState) {
			$rootScope.showNewsFeed = !!toState.showNewsFeed;
    	});
	}
]);
angular.module('blades.newsFeed').directive('newsFeed', [
  function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/newsFeed/templates/newsFeedTemplate.html',
      controller: 'NewsFeedController'
    };
  }
]);

angular.module('blades.newsFeed').controller('NewsFeedController', [
  '$scope',
  '$rootScope',
  'AppBuilder',
  '$interval',
  '$ionicSlideBoxDelegate',
  'fixturesService',
  'fixtureStatisticsService',
  'challengesService',
  '$timeout',
  function ($scope, $rootScope, AppBuilder, $interval, $ionicSlideBoxDelegate, fixturesService, fixtureStatisticsService, challengesService, $timeout) {
    $scope.news = [
      { type: 'text', text: "Share your prediction and earn bonus XP" },
      { type: 'text', text: 'Golden tickets give you a chance to win some of the most exclusive prizes in the game.' }
    ];

    var insertRandomly = function(item) {
      var len = $scope.news.length;
      var ran = Math.floor(Math.random() * len);
      $scope.news.splice(ran, 0, item);
    }

    var init = function() {
      fixturesService.currentFixture.attachClubsDetails();
      fixtureStatisticsService.find('user', {params: {'fixtureId':fixturesService.currentFixture.id}})
      .then(function(stats) {
        if (stats && stats.topPredictedScore) {
          insertRandomly({
            type: 'stats',
            text: 'TOP PREDICTED SCORE',
            score: stats.topPredictedScore.homeTeam + ' - ' + stats.topPredictedScore.awayTeam,
            logoHome: fixturesService.currentFixture.homeClubDetails.imgUrl,
            logoAway: fixturesService.currentFixture.awayClubDetails.imgUrl
          });
          var handle = $ionicSlideBoxDelegate.$getByHandle('newsSlider');
          if (handle && handle._instances.length) {
            handle.update();
          }
        }
      });

      $scope.ready = true;
      var challenges = challengesService.filter({
        fixtureId: fixturesService.currentFixture.id,
        isPhysicalPrize:true
      });

      for (var i = challenges.length - 1; i >= 0; i--) {
        var challenge = challenges[i];
        insertRandomly({
          type: 'imageAndText',
          image: challenge.prizeImageUrl,
          title: challenge.name,
          text: challenge.winningPosition
        });
      };

      $timeout(function() {
        var handle = $ionicSlideBoxDelegate.$getByHandle('newsSlider');
        if (handle && handle._instances.length) {
          handle.update();
        }
      });
    };

    if (AppBuilder.isBuilt) {
      init();
    } else {
      $rootScope.$on('build.succeeded', function() {
        init();   
      });
    }

    if (!$rootScope.sliderInterval) {
      $rootScope.sliderInterval = $interval(function() {
        var handle = $ionicSlideBoxDelegate.$getByHandle('newsSlider');
        if (handle && handle._instances.length) {
          handle.update();
          $timeout(function() {
            handle.next(500);
          });
        }
      }, 5000);
    }
  }
]);
/**
* @module blade.static
* @description This module contains the predict card
* @requires $AppBuilder
*/
angular.module('blades.predict', ['timer']).config([
  '$stateProvider',
  function ($stateProvider) {
    $stateProvider.state('shell.predict', {
      cache: false,
      requiresLogin: false,
      hideLayout: true,
      hideHeader: true,
      hideFooter: true,
      url: 'predict/:challengeId',
      views: {
        'tab-main': {
          templateUrl: 'app/blades/predict/view/templates/predictViewTemplate.html',
          controller: 'predictViewController'
        }
      },
      resolve: {
        predict: function($timeout, $q, $state) {
          var q = $q.defer();
          console.log($state);
          console.log('start resolve');
          $timeout(function() {
            console.log('resolved')
            q.resolve();
          }, 750);
          return q.promise;
        }
      }
    });
  }
]);
angular.module('blades.predict').factory('PredictService', [
  '$rootScope',
  '$q',
  '$state',
  'club',
  'userChallengesService',
  'challengesService',
  'usersService',
  'challengeMembersService',
  'predictionsService',
  function ($rootScope, $q, $state, club, userChallengesService, challengesService, usersService, challengeMembersService, predictionsService) {
    var PredictService = {
      currentPrediction: null
    }; 

    var createUserChallenge = function() {
      var payload = {
        "clubId": club.clubId,
        "userId": $rootScope.user.id,
        "fixtureId": PredictService.currentPrediction.challenge.fixtureId,
        "createOrJoin": true,
        "stake": PredictService.currentPrediction.challenge.stake
      };
      if (PredictService.currentPrediction.challenge.challengeType) {
        payload.challengeType = PredictService.currentPrediction.challenge.challengeType;
      }
      if (PredictService.currentPrediction.challenge.exclusiveMembers) {
        payload.exclusiveMembers = PredictService.currentPrediction.challenge.exclusiveMembers;
      }
      return userChallengesService.create(payload, {
        "params": {
          "userId": $rootScope.user.id
        }
      });
    }

    PredictService.saveFanVFanChallenge  = function() {
      var q = $q.defer();
      createUserChallenge()
      .then(function(newChallengeMember) {
        challengesService.find(newChallengeMember.id)
        .then(function(newChallenge) {
          // remove the old challenge as we should now have a joined one
          challengesService.eject(PredictService.currentPrediction.challenge.id);
          // set on the scope the new challenge
          PredictService.currentPrediction.challenge = newChallenge;
          PredictService.currentPrediction.prediction.challengeId = newChallenge.id;
          // save the prediction for this challenge!
          PredictService.savePrediction(PredictService.currentPrediction.prediction).then(q.resolve, q.reject);
        }, q.reject);
      }, function(err) {
        console.log(err);
        if (err.status == 304) {
          console.log('Mo, we need to talk');
        }
        q.reject(err);          
      });
      return q.promise;
    };

    PredictService.joinChallenge = function() {
      return challengeMembersService.create({
          userId: $rootScope.user.id
        }, {
          params: {
            challengeId: PredictService.currentPrediction.challenge.id
          }
        }
      );
    }
    
    PredictService.savePrediction = function(challenge, prediction, createFunc) {
      var q = $q.defer();

      if (!createFunc) {
        createFunc = 'create';
      }
      if (challenge && prediction) {
        PredictService.currentPrediction = {
          challenge: angular.copy(challenge),
          prediction: angular.copy(prediction)
        };
      } else {
        prediction = PredictService.currentPrediction.prediction;
        challenge = PredictService.currentPrediction.challenge;
      }
      if (!$rootScope.user || !$rootScope.user.id) {
        $state.go('shell.login');
        $rootScope.predictionToSave = true;
        q.reject('need to be logged in, redirect');
      } else if (challenge.isFanVFan && challenge.isCreateButton) {
        PredictService.saveFanVFanChallenge().then(q.resolve, q.reject);
      } else {
        predictionsService[createFunc](PredictService.currentPrediction.prediction).then(function(savedPrediction) {
          predictionsService.setScorers(savedPrediction);
          if (!challenge.isFanVFan || (challenge.exclusiveMembers && $rootScope.user.id && challenge.exclusiveMembers.indexOf($rootScope.user.id) != -1) ) {
            PredictService.joinChallenge().then(function() {
              challengeMembersService.findAll({challengeId:challenge.id},{bypassCache:true});
              usersService.updateCurrentUser();
            });
          } else {
            challengeMembersService.findAll({challengeId:challenge.id}, {bypassCache:true})
            .then(function() { 
              console.log('challengeMembersService findAll done');
            }, function(err) {
              console.log(err);
            });
          }
          usersService.updateCurrentUser();
          userChallengesService.find(challenge.id)
          .finally(function() {
            q.resolve();
          });
        }, function(err) {
          console.log(err);
          q.reject(err);
        }).finally(function() {
          $rootScope.predictionToSave = false;          
        });
      }
      return q.promise;
    };

    return PredictService;
  }
]);
'use strict';
angular.module('blades.share').controller('PredictionSavedPopupController', [
  '$scope',
  'ShareService',
  'parameters',
  'club',
  function ($scope, ShareService, parameters, club) {
    $scope.chineseVersion = club.chineseVersion;
    $scope.useWebView = ShareService.useWebView;
    
    $scope.shareData = parameters;

    $scope.share = function(provider) {
      ShareService.share(provider, $scope.shareData);
    };
  }
]);

angular.module('blades.predict').controller('StatisticsPopupController', [
  '$scope',
  'parameters',
  'fixtureStatisticsService',
  'teamsStatisticsService',
  'teamsService',
  function ($scope, parameters, fixtureStatisticsService, teamsStatisticsService, teamsService) {
  	var fixture = parameters.fixture;
  	var optaId  = fixture.competition.optaId;

  	var statsBroken = false;

    var homeTeamStats = teamsStatisticsService.get(fixture.homeTeamId + '-' + optaId,{params: {'teamId':fixture.homeTeamId}});
    var awayTeamStats = teamsStatisticsService.get(fixture.awayTeamId + '-' + optaId,{params: {'teamId':fixture.awayTeamId}});

    $scope.homeClub = parameters.homeClub;
    $scope.awayClub = parameters.awayClub;

  	$scope.stats = {
        'averageGoalsPerGame': {
        	'background': 'app/blades/predict/images/stats-teams.png',
            'homeTeam':homeTeamStats.$getAverageGoalsPerGame(),
            'awayTeam':awayTeamStats.$getAverageGoalsPerGame(),
            'homeTeamCss': {
                'background-color': $scope.homeClub.properties.primaryColourHex,
                'width': homeTeamStats.$getAverageGoalsPerGameBar(awayTeamStats.$getAverageGoalsPerGame())
            },
            'awayTeamCss': {
                'background-color': $scope.awayClub.properties.primaryColourHex,
                'width': awayTeamStats.$getAverageGoalsPerGameBar(homeTeamStats.$getAverageGoalsPerGame())
            }  
        },
        'totalGoalsThisSeason': {
        	'background': 'app/blades/predict/images/stats-teams.png',
            'homeTeam':homeTeamStats.$getTotalGoalsThisSeason(),
            'awayTeam':awayTeamStats.$getTotalGoalsThisSeason(),
            'homeTeamCss': {
                'background-color': $scope.homeClub.properties.primaryColourHex,
                'width': homeTeamStats.$getTotalGoalsBar(awayTeamStats.$getTotalGoalsThisSeason())
            },
            'awayTeamCss':{
                'background-color': $scope.awayClub.properties.primaryColourHex,
                'width': awayTeamStats.$getTotalGoalsBar(homeTeamStats.$getTotalGoalsThisSeason())
            }
        }
    };

    var userStats = fixtureStatisticsService.getAll();
    if (!userStats[0]) {
        statsBroken = true;
    }
    userStats = userStats[0];

    $scope.currentStat = 'averageGoalsPerGame';

    if (!homeTeamStats || !awayTeamStats) {
        statsBroken = true;
    } else if (!userStats.topPredictedGoalScorer || !userStats.topPredictedScore) {
        statsBroken = true;
    }
	if (!$scope.statsBroken) {
        $scope.stats['fanStats'] = {
        	'background': 'app/blades/predict/images/stats-fans.png',
            'topPredictedScorer': {
              'raw': userStats.topPredictedGoalScorer,
              'player': teamsService.getPlayerById(userStats.topPredictedGoalScorer.id)
            },
            'topPredictedScore': {
              'data':userStats.topPredictedScore
            }
        };
      };

  }
]);
//Real Madrid
var CLUBS_WITH_TITLE_BG = ['b59d4512-89e9-4135-9e6b-b9cc89277ff2'];
var BACKGROUND_NEEDING_TITLE_BG = ['fixedBackground.jpg'];

angular.module('blades.predict').controller('predictViewController', [
  '$scope',
  '$rootScope',
  '$timeout',
  '$q',
  '$filter',
  'ModalService',
  '$stateParams',
  'PredictService',
  '$state',
  'UserSettings',
  'AudioToolsService',
  'club',
  'Urls',
  'usersService',
  'challengesService',
  'challengeMembersService',
  'userChallengesService',
  'predictionsService',
  'fixturesService',
  'fixtureStatisticsService',
  'clubsService',
  'teamsService',
  'teamsStatisticsService',
  'ToastService',
  'TOAST',
  'PopupService',
  'SharePopupService',
  'BackgroundService',
  function ($scope, $rootScope, $timeout, $q, $filter, ModalService, $stateParams, PredictService, $state, UserSettings, AudioToolsService, club, Urls, usersService, challengesService, challengeMembersService, userChallengesService, predictionsService, fixturesService, fixtureStatisticsService, clubsService, teamsService, teamsStatisticsService, ToastService, TOAST, PopupService, SharePopupService, BackgroundService) {
    $scope.loading = true;
    $scope.ready = false;

    $scope.chineseVersion = !!club.chineseVersion;

    var currentClub = clubsService.get(club.clubId);
    var currentChallenge = null;

    $scope.$on('$ionicView.beforeEnter', function() {
      $scope.loading = true;
    });
    $scope.$on('$ionicView.afterLeave', function() {
      // detach team's data from $scope
      $scope.ready = false;
      $scope.prediction = null;
      $scope.loading = true;
      $timeout.cancel($scope.timeoutStartTime);
    });

    var challenge = null;
    $scope.show = true;
    $scope.$on('$stateChangeStart', function (event, toState) {
      if (toState.name !== 'shell.predict') {
        $scope.show = false;
      }
    });

    var getChallenge = function() {
      var q = $q.defer();
      var challenge = challengesService.get($state.params.challengeId);
      if (challenge) {
        q.resolve(challenge);
        return q.promise;
      } if (currentChallenge) {
        q.resolve(currentChallenge);
        return q.promise;
      } else {
        return challengesService.find($state.params.challengeId)
      }
    };

    var modals = {};
    var initTeamModal = function(team, show) {
      return ModalService.show('app/blades/predict/view/templates/predictTeamModal.html', 'PredictModalController', {
        team: team,
        fixture: $scope.fixture,
        style: $scope.style[team],
        prediction: $scope.prediction
      }, {
        show: !!show
      }).then(function(modal) {
        modals[team] = modal;
      });
    };

    $scope.showTeamModal = function(team) {
      if (modals[team] && modals[team].$parent) {
        modals[team].modal.show();
        modals[team].$on('modal.hidden', function (thisModal) {
          initTeamModal(team);
        });
      } else {
        initTeamModal(team, true)
        .then(function(modal) {
          modals[team].$on('modal.hidden', function (thisModal) {
            initTeamModal(team);
          });
        });
      }
    };

    $scope.showTitleBG = function showTitleBG() {
      var currentBG = BackgroundService.currentBackground;
      var needsTitle = BACKGROUND_NEEDING_TITLE_BG.filter(function(background){
        var bgRegExp = new RegExp(background, 'i');
        return bgRegExp.test(currentBG);
      });
      return _.includes(CLUBS_WITH_TITLE_BG, club.clubId) && needsTitle.length;
    };

    $scope.$on('$ionicView.afterEnter', function () {
      $scope.ready = true;
      $scope.show = true;
      getChallenge().then(function(challenge) {
        currentChallenge = challenge;
        $scope.challengeLogo = challenge.logo;
        var now = new Date().getTime();
        var fixture = fixturesService.get(challenge.fixtureId);
        $scope.flashing = fixture.startTime < (now - 60000);

        var endIn = Math.max(0, fixture.startTime - now);
        if (endIn < 0x7FFFFFFF) {
          //setTimeout limit is MAX_INT32=(2^31-1))
          $scope.timeoutStartTime = $timeout(function() {
            // make sure we go back all the way to the first view
            $rootScope.goBack(-42);
            ToastService.showNativeToast('ERROR!', 'Sorry, you cannot predict on this match any more', TOAST.types.error);
          }, endIn);
        }

        var optaId = fixture.competition.optaId;

        $scope.fixture = fixture;
        $scope.homeClub = clubsService.get($scope.fixture.homeClubId);
        $scope.awayClub = clubsService.get($scope.fixture.awayClubId);

        $scope.compLogoUrl = Urls.urlBuilder('sports/football/competitions/' + optaId + '/compLogo.svg');

        var dataToLoad = [];
        dataToLoad.push(teamsService.find(fixture.awayTeamId));
        dataToLoad.push(teamsService.find(fixture.homeTeamId));
        dataToLoad.push(teamsStatisticsService.find(fixture.homeTeamId + '-' + optaId, {params: {'teamId':fixture.homeTeamId}}));
        dataToLoad.push(teamsStatisticsService.find(fixture.awayTeamId + '-' + optaId, {params: {'teamId':fixture.awayTeamId}}));
        dataToLoad.push(fixtureStatisticsService.find('user', {params: {'fixtureId':fixture.id}}));
        // Need to load all player images here too!
        $timeout(function () {
          $q.all(dataToLoad).then(function loadingSuccess(argument) {
            initPrediction();
            if ($rootScope.predictionToSave) {
              console.log('wait for the prediction to be saved and leave');
            } else {
              $scope.loading = false;
            }
          }).catch(function loadingFailure(argument) {
            $rootScope.goBack();
          });
        }, 2500);
      }, function(err) {
        console.log(err);
      });
    });

    var initPrediction = function() {
      $scope.predictionToSave = false;
      $scope.timeSponsorImage = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/time.png');
      $scope.style = {
        home: clubsService.getStyle($scope.homeClub, 'home'),
        away: clubsService.getStyle($scope.awayClub, 'home') // we use the home style anyway..
      };
      if (!$scope.style.away) {
        $scope.style.away = $scope.style.home; 
      }
      if (!$scope.style.home) {
        $scope.style.home = $scope.style.away;  
      }

      $scope.getPrediction();
    }

    // Get the prediction and bind it to the view in
    // case it changes elsewhere
    $scope.getPrediction = function() {

      var newPredictionConfig = {
        fixtureId: $scope.fixture.id,
        clubId: club.clubId,
        challengeId: currentChallenge.id,
        homeTeamId: $scope.fixture.homeTeamId,
        awayTeamId: $scope.fixture.awayTeamId
      };

    	$scope.prediction = predictionsService.getPredictionByIds(newPredictionConfig);

      if ($scope.prediction.id) {
        predictionsService.bindOne($scope, 'prediction', $scope.prediction.id);
      }

      predictionsService.setScorers($scope.prediction);
      initTeamModal('home');
      initTeamModal('away');
    };

    var getShareData = function() {
      try {
        if (!$scope.prediction) {
          return null;
        }
        var homeScore = $scope.prediction.homeTeamScore == '+' ? '0' : $scope.prediction.homeTeamScore;
        var awayScore = $scope.prediction.awayTeamScore == '+' ? '0' : $scope.prediction.awayTeamScore;
        var caption = $filter('translate')('I predicted %%PREDICTED_SCORE%% for the %%HOME_CLUB%% vs %%AWAY_CLUB%% on %%CLUB_NAME%% %%APP_NAME%%');
        caption = caption.replace('%%PREDICTED_SCORE%%', homeScore + ' - ' + awayScore);
        caption = caption.replace('%%HOME_CLUB%%', $scope.homeClub.name);
        caption = caption.replace('%%AWAY_CLUB%%', $scope.awayClub.name);
        caption = caption.replace('%%CLUB_NAME%%', currentClub.name);
        caption = caption.replace('%%APP_NAME%%', club.id == '9764489e-c45c-431d-a0f2-7e092c7495db' ? 'Score Predictor' : 'Predictabl'); // Eww, I know
        if (currentClub.properties && currentClub.properties.hashtag) {
          var hashtagedCaption = caption + ' ' + currentClub.properties.hashtag;
          var twitterShortUrlMaxLength = 20;
          if (hashtagedCaption.length < ( 140 - twitterShortUrlMaxLength) ) {
            caption = hashtagedCaption;
          }
        }
        var baseUrl = 'https://playabllive.blob.core.windows.net/';
        var pathToClub = 'images/sports/football/clubs/' + currentClub.id;
        var pathToSharePic = '/icons/android/icon.png';
        var pictureUrl = baseUrl + pathToClub + pathToSharePic;

        return {
          title: "PredictablTitle",
          description: "Predictabl",
          url: 'http://'+currentClub.subdomain+'.playabl.com',
          picture: pictureUrl,
          caption: caption
        };
      } catch (err) {
        console.log(err);
        return null;
      }
    };

    $scope.fireSharePopup = function() {
      var shareData = getShareData();
      if (shareData) {
        SharePopupService.show(shareData);
      }
    };

    $scope.showStats = function() {
      ModalService.show('app/blades/predict/view/templates/popups/statistics.html', 'StatisticsPopupController', {
        fixture: fixturesService.get(currentChallenge.fixtureId),
        homeClub: $scope.homeClub,
        awayClub: $scope.awayClub
      });
    };

    $scope.showSavedPredictionPopup = function() {
      var shareData = getShareData();
      if (!shareData) {
        $rootScope.goBack();
      } else {
        ModalService.show('app/blades/predict/view/templates/popups/predictionSaved.html', 'PredictionSavedPopupController', getShareData(), {
          modalCallback: function() {
            console.log($scope.prediction);
          }
        })
        .finally(function() {
          $rootScope.goBack();
        });
      }
    }

  	$scope.savePrediction = function(prediction) {
      PredictService.savePrediction(currentChallenge, $scope.prediction)
      .then(function() {
        $scope.showSavedPredictionPopup();
      }, function(err) {
        console.log(err);
        initPrediction();
      });
  	};

    $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
      // If we have come back from login, go go go!
      if ($rootScope.predictionToSave && (fromState.name == 'shell.login' || fromState.name == 'shell.register') ){
        $scope.savePrediction();
      }
    });
  }
]);


angular.module('blades.predict').controller('PredictModalController', [
  '$scope',
  'parameters',
  '$ionicSlideBoxDelegate',
  'teamsService',
  'teamsStatisticsService',
  '$timeout',
  '$filter',
  function ($scope, parameters, $ionicSlideBoxDelegate, teamsService, teamsStatisticsService, $timeout, $filter) {
    var MAX_PLAYERS_TO_PREDICT = 9;
    $scope.pteam = parameters.team;
    $scope.style = parameters.style;
    $scope.goals = parameters.prediction[$scope.pteam  + 'Goals'];
    $scope.slideIndex = 0;
    $scope.team = teamsService.get(parameters.fixture[$scope.pteam  + 'TeamId']);
    $scope.teamBadge = parameters.fixture[$scope.pteam  + 'ClubDetails'].imgUrl;
    $scope.showPlayerGoals = false;
    $scope.currentFilter = 'All';
    $scope.placeholders = [];

    function buildPlaceHolders(placeHoldersToBuild){
      while(placeHoldersToBuild--) {
        $scope.placeholders.push(buildPlaceholder());
      }
    }

    function buildPlaceholder(){
      var id = 'id-' + new Date().getTime();
      return {
        id: id
      };
    }

    var orderPlayersForView = function(players, fixture) {
        var availablePlayers = teamsService.availablePlayers(players,fixture);
        var PLAYERS_PER_ROWS = 5;
        var NUM_ROWS = 2;
        var PLAYERS_PER_PAGES = PLAYERS_PER_ROWS * NUM_ROWS;
        var row = 0;
        var page = 0;

        $scope.players = [];

        for (var i = 0; i < availablePlayers.length; i++) {
            if (!(i%PLAYERS_PER_PAGES)) {
                $scope.players.push({
                    0:[],
                    1:[]
                });
            }

            if (!(i%PLAYERS_PER_PAGES) && (i !== 0)) {
                page++;
                row = 0;
            } else if(!(i%PLAYERS_PER_ROWS) && (i !== 0)) {
                row++;
            }

            $scope.players[page][row].push(availablePlayers[i]);
        }
    };

    $scope.init = function init() {
      var placeHoldersToBuild = $scope.goals.length === 0 ?
        MAX_PLAYERS_TO_PREDICT : MAX_PLAYERS_TO_PREDICT - $scope.goals.length;

      orderPlayersForView($scope.team.players, parameters.fixture);
      buildPlaceHolders(placeHoldersToBuild);
    };

    $scope.init();

    $scope.addGoal = function(team, player) {
        if (parameters.prediction.$getTeamScore(team) === MAX_PLAYERS_TO_PREDICT) {
          return;
        }
        parameters.prediction.$addGoal(team, player);
        $scope.placeholders.pop();
      };

    $scope.removeGoal = function(team, playerOrder) {
        parameters.prediction.$removeGoal(team, playerOrder);
        $scope.placeholders.push(buildPlaceholder());
    };

    $scope.nilGoals = function(team) {
        parameters.prediction.$nilGoals(team);
        $scope.goals = parameters.prediction[$scope.pteam  + 'Goals'];
        $scope.placeholders = [];
        buildPlaceHolders(MAX_PLAYERS_TO_PREDICT);
    };

    $scope.onSlideChanged = function(index) {
      $scope.slideIndex = index;
    };

    $scope.totalGoalsPredicted = function totalGoalsPredicted() {
        return parameters.prediction[$scope.pteam + 'Goals'].length;
    };

    $scope.togglePlayerInfo = function togglePlayerInfo(){
      $scope.showPlayerGoals = !$scope.showPlayerGoals;
    };

    $scope.playerInfoButtonText = function playerInfoButtonText(){
      var buttonText = $scope.showPlayerGoals ? 'NAME' : 'GOALS';
      return $filter('translate')(buttonText);
    };

    $scope.nextPlayerPage = function() {
        $ionicSlideBoxDelegate.$getByHandle('playersSlidebox').next();
    };

    $scope.previousPlayerPage = function() {
        $ionicSlideBoxDelegate.$getByHandle('playersSlidebox').previous();
    };

    $scope.playerPositionBadge = function(player) {
        return teamsService.playerPositionShortCode(player.position);
    };

    $scope.readableShirtNumber = function(player) {
        var DEFAULT_SHIRT_REGEX = /default\-team\/headShot.svg/;
        var UNREADABLE_COLOUR_REGEX = /\#000/;
        var READABLE_TEXT_COLOUR = {color: '#FFF'};
        var playerTextColour = $scope.style.playerText.color;

        return DEFAULT_SHIRT_REGEX.test(player.imgUrl) &&
        UNREADABLE_COLOUR_REGEX.test(playerTextColour) ?
        READABLE_TEXT_COLOUR : $scope.style.shirtNumberColor;
    };

    $scope.getPlayerGoals = function getPlayerGoals(playerId){
        var playerTotalGoals = 0;
        var fixture = parameters.fixture;
        var optaId  = fixture.competition.optaId;
        var teamId = fixture[$scope.pteam +'TeamId'];
        var teamStats = teamsStatisticsService.get(
            teamId + '-' + optaId, {
                params: {
                    'teamId':teamId
                }
            });

        var playerStats = teamStats.players.filter(function(player) {
            return player.id === playerId;
        })[0];

        return playerStats.statistics.reduce(function(prevTotal, player){
            return prevTotal += player.totalGoals;
        }, playerTotalGoals);
    };

    $scope.filterPlayers = function filterPlayers(playerType) {
        var filteredPlayers;

        if (playerType) {
            filteredPlayers = $scope.team.players.filter(function(player) {
                return player.position === playerType;
            });
        } else {
            filteredPlayers = $scope.team.players;
        }

        orderPlayersForView(filteredPlayers, parameters.fixture);
        $scope.currentFilter = playerType || 'All';
        $ionicSlideBoxDelegate.update();
        $ionicSlideBoxDelegate.slide(0);
    };
  }
]);

angular.module('blades.predict')
    .directive('playerPosition', playerPositionDirective);

function playerPositionDirective(){
    return {
        templateUrl: 'app/blades/predict/directives/playerPosition/template.html',
        transclude: true
    }
}

angular.module('blades.predict')
  .directive('playerAvatar', playerAvatarDirective);

function playerAvatarDirective() {
  return {
    templateUrl: 'app/blades/predict/directives/playerAvatar/template.html',
    transclude: true,
    scope: {
      player: '=',
      club: '='
    },
    controller: playerAvatarDirectiveController
  };
}

function playerAvatarDirectiveController($scope, club) {
  $scope.playerOutlineImage = 'images/player-outline.svg';
  $scope.shirtOutlineImage = 'images/shirt-outline.svg';

  $scope.isCurrentClub = function isCurrentClub() {
    return club.clubId === $scope.club.clubId;
  };
}

playerAvatarDirectiveController.$inject = ['$scope','club'];

/**
* @module blade.shop
* @description This module contains the shop modal.
* @requires $AppBuilder
*/
angular.module('blades.shop', []).run([
  'AppBuilder',
  'NavigationService',
  '$state',
  '$rootScope',
  function (AppBuilder, NavigationService, $state, $rootScope) {    
    AppBuilder.registerMenu({
      menu: 'top',
      side: 'left',
      template: '<shop-menu-button type="background"></shop-menu-button>'
    });
    AppBuilder.registerMenu({
      menu: 'top',
      side: 'right',
      template: '<shop-menu-button type="coins"></shop-menu-button>'
    });
    NavigationService.registerRoute('shell.shop', function(args) {
      // navigate to the shop, if we are login
      if ($rootScope.user) {
        $state.go('shell.shop', args);
      }
    });
  }
]).config([
  '$stateProvider',
  function ($stateProvider) {
  $stateProvider.state('shell.shop', {
    url: 'shop?type',
    requiresLogin: true,
    showMainLogo: false,
    hideLayout: true,
    hideFooter: true,
    hideHeader: true,
    views: {
      'tab-main': {
        templateUrl: 'app/blades/shop/modal/templates/shopModalTemplate.html',
        controller: 'ShopModalController',
          resolve: {
            shop: function($timeout, $q) {
              var q = $q.defer();
              console.log('start resolve');
              $timeout(function() {
                console.log('resolved')
                q.resolve();
              }, 750);
              return q.promise;
            }
          }
      }
    }
  });
}]);
angular.module('blades.shop').factory('PurchasePluginService', [
  '$q',
  'ToastService',
  'TOAST',
  'ModalService',
  'usersService',
  'userPayments',
  '$rootScope',
  function ($q, ToastService, TOAST, ModalService, usersService, userPayments, $rootScope) {
    var PurchasePluginService = { 
      isAvailable: false,
      paymentProvider: ionic.Platform.isAndroid() ? 'google' : 'apple',
      platform: ionic.Platform.isAndroid() ? 'android' : 'apple' 
    };
    
    var finishPurchase = function(order) {
      console.log(order);
      var userPaymentsData = {
        "paymentProvider": PurchasePluginService.paymentProvider,
        "productId": PurchasePluginService.currentTransaction.productId,
        "transactionId": PurchasePluginService.currentTransaction.transactionId,
        "paymentProviderData": {}
      };
      var device = ionic.Platform.device();
      if (device && device.name) {
        userPaymentsData.deviceMakingTransaction = device.name;
      }
      if (ionic.Platform.isAndroid()) {
        userPaymentsData.paymentProviderData.googleIap = {
          'purchaseData': order.transaction.receipt,
          'dataSignature': order.transaction.signature
        }
      } else {
        userPaymentsData.paymentProviderData.appleIap = {
          'id': order.transaction.id
        }
      }
      if (PurchasePluginService.currentProduct && PurchasePluginService.currentProduct.storeProduct.price) {
        userPaymentsData.price = PurchasePluginService.currentProduct.storeProduct.price;
      }
      console.log(userPaymentsData);
      userPayments.create(userPaymentsData).then(function(result) {
        console.log(result);
        
        order.finish();
        console.log('stu currentProduct: ', PurchasePluginService.currentProduct);

        if($rootScope.user.gatewayAd){

          // This WILL FAIL
          // please add prductName and productType
          /*
          PurchasePluginService.currentProduct Object looks like .. 

            appProductData.: Object
            id: "47b29252-3468-4a1a-819f-9ba2bfe3ed03"
            numberOfItems: 10000
            orderNumber: 1
            priceCurrency: "GBP"
            productDescription: "Buy 10,000 coins to enter cool challenges and buy awesome extras!"
            productImage: "images/cdn/coins/tenthousandBalls.png"
            productImageIsLocal: false
            productName: "Buy 10,000 coins"
            properties: Object
            proposedPriceGBP: 0
            published: true
            storeProduct: Object
            totalPrice: 1.49
            type: "balls"
          */

          pachingAdService.regiserPayment(
            $rootScope.user.id, 
            PurchasePluginService.currentProduct.id, 
            PurchasePluginService.currentProduct.productName, 
            PurchasePluginService.currentProduct.type, 
            false
          );
        }
        
        PurchasePluginService.currentProduct = null;
        PurchasePluginService.currentTransaction = null;
        // refresh user's data
        usersService.updateCurrentUser();

        // Register Payment with add service
        ModalService.show('app/blades/shop/popup/template/rewardsPopupTemplate.html', 'RewardsPopupController', { bought: result.chipsPaidFor}, { animation: 'animated zoomIn'});
        //ToastService.showNativeToast('Congrats! You just bought ' + result.chipsPaidFor + ' balls!', '', TOAST.types.success);
      }, function(err) {
        ToastService.showNativeToast('ERROR!', err, TOAST.types.error);
      });
    };

    var handleError = function(err) {
      console.log(err);
      ToastService.showNativeToast('ERROR!', err, TOAST.types.error);
    };

    PurchasePluginService.init = function(products) {
      if (!window.store) {
        PurchasePluginService.isAvailable = false;
       return;
      }
      store.verbosity = store.DEBUG;
      var refreshStore = false;
      for (var i = products.length - 1; i >= 0; i--) {
        if (products[i].appProductData && products[i].appProductData[PurchasePluginService.platform] && products[i].appProductData[PurchasePluginService.platform].id) {
          var id = products[i].appProductData[PurchasePluginService.platform].id;

          var registeredProduct = store.get(id);
          if (registeredProduct) {
            products[i].storeProduct = registeredProduct;
          } else {
            refreshStore = true;
            store.register({
              id: id,
              alias: products[i].productName,
              type: store.CONSUMABLE
            });
            console.log(products[i].appProductData[PurchasePluginService.platform].id);
            var p = store.get(products[i].productName);
            products[i].storeProduct = p;
            store.error(function(err) {
              console.log(err);
            });
            store.when(p.id)
            .updated(function(p) {
              console.log('id: ' + p.id);
              console.log('id: ' + p.price);
            })
            .approved(finishPurchase)
            .error(function(err) {
              console.log(err);
              //handleError(err);
            })
            .cancelled(function(err) {
              console.log(err);
            });
          }
        }
      }
      if (refreshStore) {
        store.refresh();        
      }
      PurchasePluginService.isAvailable = true;
    };
    PurchasePluginService.buy = function (product, transaction) {
      var q = $q.defer();

      PurchasePluginService.currentProduct = product;
      PurchasePluginService.currentTransaction = transaction;
      console.log('buy');

      var id = transaction.storeId;
      store.order(id).then(function(data) {
        console.log(data);
      }, function(err) {
        console.log(err);
        q.reject(err);
      });
      return q.promise;
    };
    return PurchasePluginService;
  }
]);

angular.module('blades.shop')
.controller('TicketsPopupController', [
  '$scope',
  'PopupService',
  function ($scope, PopupService) {
    $scope.close = function() {
      PopupService.hide();
    };
  }
]);

angular.module('blades.shop')
.controller('RewardsPopupController', [
  '$scope',
  'parameters',
  'Urls',
  function ($scope, parameters, Urls) {
    $scope.bgImage = Urls.urlRemoteBuilder('coins/freeBalls.png'); 
  	if (parameters.reward) {
  		$scope.description = "YOU WON";
  	  	$scope.value = parameters.reward;
	}
  	if (parameters.bought) {
  		$scope.description = "YOU BOUGHT"; 
	  	$scope.value = parameters.bought;
  	}
  }
]);
angular.module('blades.shop').controller('ShopModalController', [
  '$scope',
  '$rootScope',
  '$stateParams',
  'productsService',
  '$timeout',
  '$ionicSlideBoxDelegate',
  'userProducts',
  'ToastService',
  'TOAST',
  'usersService',
  'Transactions',
  'BackgroundService',
  'PurchasePluginService',
  '$ionicLoading',
  'SupersonicAdsPluginService',
  'ModalService',
  function ($scope, $rootScope, $stateParams, productsService, $timeout, $ionicSlideBoxDelegate, userProducts, ToastService, TOAST, usersService, Transactions, BackgroundService, PurchasePluginService, $ionicLoading, SupersonicAdsPluginService, ModalService) {
    $scope.$on('$ionicView.beforeEnter', function() {
      $scope.loadingBalls        = false;     
      $scope.loadingBackgrounds  = false;   
    });
    $scope.show = true;    
    $scope.$on('$stateChangeStart', function (event, toState) {
      if (toState.name !== 'shell.shop') {
        $scope.show = false;
      }
    });

    $scope.ready = false;
    $scope.$on('$ionicView.afterLeave', function () {
      $scope.ready = false;
    });
    $scope.$on('$ionicView.afterEnter', function () {
      $scope.ready = true;
      $scope.show = true;

      $timeout(function() {
        $scope.video = SupersonicAdsPluginService;

        $scope.balls = productsService.filter({ 
          type: 'balls'
        });
        $scope.backgrounds = productsService.filter({ 
          type: 'backgrounds',
          orderBy: [
            ['totalPrice', 'ASC']
          ]
        });

        userProducts.bindAll($scope, 'userProducts');
        $timeout(function() {
          $scope.activeTab = $stateParams.type === 'background' ? 0 : 1;
      
          $scope.changeTab($scope.activeTab);
          if ($scope.activeTab === 1) {
            $scope.loadingBalls = true;
          } else {
            $scope.loadingBackgrounds = true;
          }
          $timeout(function() {
            $scope.loadingBackgrounds = true;
            $scope.loadingBalls = true;
          }, 0);
        }, 0);
      });
    });

    $scope.preview = function(item) {
        var previewImage = '<img class="preview"' +
                            'ng-click="hidePreview()"' +
                            'style="background-image:url(' +
                            item.productImage + ')" />';

        $ionicLoading.show({
          template: previewImage,
          scope: $scope
        });
    };
    $scope.hidePreview = function() {
      $ionicLoading.hide();
    };

    $scope.changeTab = function(tabIndex) {
      var slidebox = $ionicSlideBoxDelegate.$getByHandle('shop-slidebox');
      $timeout(function() {
        if (slidebox) {
          var slidesCount = slidebox.slidesCount();
          console.log('slidesCount: ' + slidesCount);
          slidebox.slide(tabIndex);
        }
      });
    };

    $scope.isOwned = function(item) {
      for (var i = $scope.userProducts.length - 1; i >= 0; i--) {
        if ($scope.userProducts[i].id === item.id) {
          return item.numberOfItems === $scope.userProducts[i].numberPurchased;
        }
      }
      return false;
    };

    $scope.watchVideo = function() {
      SupersonicAdsPluginService.play()
      .then(function() {
        console.log('resolved')
      }, function(err) {
        console.log('err')
      }, function() {
        console.log('notify');
      });
    };

    /**
    * Open a confirmation popup and procced to the transaction
    */
    $scope.buy = function (item) {      
      userProducts.update(item.id, {id: item.id, 'numberPurchased': 1}, {'params': {'userId': $rootScope.user.id} })
      .then(function(result) {
        console.log(result);
        // refresh user's data
        usersService.updateCurrentUser();
        //ToastService.showNativeToast('Background Theme purchased successfully.', '', TOAST.types.success);
        $scope.use(item);
      }, function(err) {
        console.log(err);
        ToastService.showNativeToast('ERROR!', '', TOAST.types.error);
      });
    };
    
    $scope.use = function(item) {
      BackgroundService.changeBackground(item.productImage);
    };

    $scope.startPayment = function (ball) {
      Transactions.create({
        paymentProvider: PurchasePluginService.platform,
        productId: ball.id
      }).then(function(transaction) {
        transaction.storeId = ball.appProductData[PurchasePluginService.platform].id;
        transaction.alias = ball.productName;
        console.log(transaction);
        PurchasePluginService.buy(ball, transaction);
      }, function(err) {
        console.log(err);
      });
    };

  }
]);
/**
* @name userPayment
* @description Service to ...
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('blades.shop')
.service('userPayments', [
  'DS',
  function(DS) {
    var userPayments = DS.defineResource({
      name: 'userPayments',
      idAttribute: 'newUserChipCount',
      deserialize: function (resourceName, response) {
        var result = response.data._data;
        console.log(result);
        return result;
      }
    });
    return userPayments;
  }
]);

/**
* @name usersProductService
* @description Service to ...
* @requires {Object} DS Service / Factory for HTTP calls {@link http://angular-data.pseudobry.com}
**/
angular.module('blades.shop')
.service('userProducts', [
  'DS',
  'productsService',
  function(DS, productsService) {
    var userProducts = DS.defineResource({
      name: 'users/products',
      idAttribute: 'id',
      endpoint: 'products',
      relations: {
        belongsTo: {
          users: {
            parent: true,
            enumerable: true,
            localField: 'user',
            localKey: 'userId'
          }
        }
      },
      serialize: function(resourceName, data) {
        return data;
      },
      deserialize: function (resourceName, response) {
        var prod;
        if (response.data._data) {
          prod = response.data._data; 
        } else {
          prod = response.data;
        }
        if (prod) {
          for (var i = prod.length - 1; i >= 0; i--) {
            var product = productsService.get(prod[i].id);
            if (product && product.productImageIsLocal) {
              prod[i].numberPurchased = 0;
            }
          }
        }
        return prod;
      }
    });
    return userProducts;
  }
]);


angular.module('blades.shop').factory('Transactions', [
  'DS',
  function (DS) {
    var Transactions = DS.defineResource({
      name: 'transactions',
      serialize: function(resourceName, data) {
        return data;
      },
      deserialize: function (resourceName, response) {
        console.log(response);
        var result = response.data;
        result.id = result.productId;
        return result;
      }
    });
    return Transactions;
  }
]);
/**
* @ngdoc directive
* @name paymentMenu
* @description encapsulate the user profile card, which contains the main user data
* displayName, profile picture, ...
* @example
* Usage:
* <payment-menu-button></payment-menu-button>
*/
angular.module('blades.shop')
.directive('shopMenuButton', [
	function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/shop/menuButton/templates/shopMenuButtonTemplate.html',
      controller: 'ShopMenuButtonController',
      link: function ($scope, $element, $attrs) {
        $scope.type = $attrs.type;
      }
    };
  }]);

angular.module('blades.shop').controller('ShopMenuButtonController', [
  '$scope',
  '$rootScope',
  '$state',
  'ModalService',
  function ($scope, $rootScope, $state, ModalService) {
    $scope.onClick = function () {
      if ($rootScope.user) {
        $state.go('shell.shop', {type: $scope.type});
      } else {
        $state.go('shell.login');
      }
    };
    $scope.openTicketPopup = function() {
      ModalService.show('app/blades/shop/popup/template/ticketsPopupTemplate.html', 'TicketsPopupController');
    };
  }
]);
/**
* Define the blade user profile module
*/
angular.module('blades.user', [])
.run([
  'AppBuilder',
  'NavigationService',
  '$state',
  '$rootScope',
  function (AppBuilder, NavigationService, $state, $rootScope) {
    AppBuilder.registerMenu({
      menu: 'bottom',
      side: 'right',
      template: '<user-menu-button></user-menu-button>'
    });
	NavigationService.registerRoute('shell.user', function(args) {
		// navigate to the user page, if we are login and if a userId parameter is provided
		if (args.userId && $rootScope.user) {
			$state.go('shell.user', args);
		}
	});
  }
])
.config([
 	'$stateProvider',
  	function ($stateProvider, NavigationProvider, $state) {
	  	$stateProvider.state('shell.user', {
		    url: 'user/:userId',
		    requiresLogin: true,
		    showMainLogo: false,
		    hideLayout: true,
		    hideFooter: true,
		    hideHeader: true,
		    views: {
		      'tab-main': {
		        templateUrl: 'app/blades/user/modal/templates/userPopupTemplate.html',
		        controller: 'UserModalController',
		          	resolve: {
		            	shop: function(userStatisticsService, $stateParams, club, $q, $timeout) {
		            		var q = $q.defer();
		            		userStatisticsService.findAll({
  						        'userId': $stateParams.userId,
  						        'clubId': club.clubId,
  						        'includeTotals': false
    						    });
		            		$timeout(function() {
		            			q.resolve()
		            		}, 750);
		            		return q.promise;
			            }
		          	}
		      	}
		    }
		});
	}
]);


angular.module('blades.user').controller('UserModalController', [
  '$scope',
  '$rootScope',
  'ModalService',
  'userStatisticsService',
  '$state',
  'UserModalService',
  'AppBuilder',
  'usersService',
  'Urls',
  'club',
  '$stateParams',
  'PopupService',
  function ($scope, $rootScope, ModalService, userStatisticsService, $state, UserModalService, AppBuilder, usersService, Urls, club, $stateParams, PopupService) {
    $scope.settings = AppBuilder.settings;
    $scope.defaultUserImage = Urls.urlRemoteBuilder('sports/football/clubs/' + club.clubId + '/defaultUser.svg');

	  $scope.statisticsLoaded = false;

    $scope.show = true;
    $scope.$on('$stateChangeStart', function (event, toState) {
      if (toState.name !== 'shell.user') {
        $scope.show = false;
      }
    });

    $scope.ready = false;
    $scope.$on('$ionicView.afterLeave', function () {
      $scope.ready = false;
    });

    $scope.$on('$ionicView.afterEnter', function () {
      $scope.ready = true;
      $scope.isMyProfile = $rootScope.user.id === $stateParams.userId;
      if ($scope.isMyProfile) {
        $scope.profileUser = $rootScope.user;
      }
      $scope.show = true;
      userStatisticsService.findAll({
        'userId': $stateParams.userId,
        'clubId': club.clubId,
        'includeGlobals': true
      }).then(function(user) {
        var userData = userStatisticsService.filter({userId: $stateParams.userId});
        var userProfileData = userData instanceof Array ? userData[0] : userData;
        $scope.statisticsLoaded = true;
        $scope.statistics = userProfileData.clubStats;
        $rootScope.user.statistics = userProfileData.clubStats;
        
        if (!$scope.profileUser) {
          $scope.profileUser = userProfileData;
        } else {
          $scope.profileUser.stats = userProfileData;
        }
      });
    });

    /**
    * goToLoginState
    * @function
    * @description this function is used  in the template if the user is not yet authentication
    * to redirect him on the login page
    * @return void
    */
    $scope.goToLoginState = function () {
      UserModalService.hidePopup();
      $state.go('shell.login');
    };

    $scope.showEditProfile = function() {
      ModalService.show('app/blades/user/modal/templates/edituserPopupTemplate.html', 'UserEditModalController');
    };

	$scope.changeAvatar = function changeAvatar() {
    if ($scope.isMyProfile === true) {
      var avatarTemplate = 'app/blades/user/avatarPopup/template.html';
      PopupService.show({
        templateUrl: avatarTemplate,
        cssClass: 'avatar-selector--container'
      });
    }
	};

  $scope.showFBModal = function showFBModal() {
    if (club.chineseVersion != true) {
      ModalService.show('app/blades/user/modal/templates/facebookFriendsLeagueTemplate.html', 'UserFacebookLeagueModalController');      
    }
  };


  $scope.isChineseVersion = function userHasFacebook() {
    return club.chineseVersion;
  };

}]);

angular.module('blades.user').factory('FacebookFriendsService', [
  '$q',
  '$cordovaFacebook',
  'Facebook',
  '$ionicPlatform',
  function ($q, $cordovaFacebook, Facebook, $ionicPlatform) {
    var FacebookFriendsService = {};

    var getPlugin = function () {
      var q = $q.defer();
      $ionicPlatform.ready(function () {
      	if (window.facebookConnectPlugin) {
			// Use the plugin
	        q.resolve($cordovaFacebook);      		
      	} else {
      		// use Facebook js sdk ( should only be for dev in browser )
      		q.resolve(Facebook); 
      	}
      });
      return q.promise;
    };

    var facebookApi = function(path) {
    	var q = $q.defer();
    	getPlugin().then(function(plugin) {
    		if (window.facebookConnectPlugin) {
				plugin.api(path, ['public_profile', 'user_friends', 'email'])
				.then(q.resolve, q.reject);
			} else {
				plugin.api(path, function(response) {
					if (response && !response.error) {
						q.resolve(response);
					} else {
						q.reject();
					}
				});
			}
    	});
	    return q.promise;
    }

    FacebookFriendsService.hasPermission = function(permission) {
	    var q = $q.defer();
	    getPlugin().then(function(facebookPlugin) {
	    	facebookApi('/me/permissions')
	    	.then(function(response) {
		    	if (response && !response.error) {
		        	console.log(response.data);
		        	for (var i = response.data.length - 1; i >= 0; i--) {
		          		if (response.data[i].permission == permission) {
		            		if (response.data[i].status == 'granted') {
		             			 q.resolve();
		             	 		return;
		            		}
		          		}
		        	}
		        	q.reject();
		      	} else {
		        	q.reject(response.error);
		      	}
		    }, q.reject);
	    });
	    return q.promise;
	}

	FacebookFriendsService.getFacebookFriends = function() {
		var q = $q.defer();
		getPlugin().then(function(facebookPlugin) {	    
		    facebookApi('/me/friends').then(function(response) {
		      if (response && !response.error && response.data && response.data instanceof Array) {
		          q.resolve(response.data);
		      } else {
		        q.reject(response.error);
		      }
		    }, q.reject);
		});
	    return q.promise;
	}


    return FacebookFriendsService;
  }
]);

angular.module('blades.user').controller('UserEditModalController', [
  '$scope',
  '$rootScope',
  'AuthService',
  'ModalService',
  '$state',
  'UserModalService',  
  'AppBuilder',
  'usersService',
  'ISO3166',
  'ToastService',
  'TOAST',
  '$timeout',
  'Urls',
  'club',
  'PopupService',
  '$filter',
  function ($scope, $rootScope, AuthService, ModalService, $state, UserModalService, AppBuilder, usersService, ISO3166, ToastService, TOAST, $timeout, Urls, club, PopupService, $filter) {
    $scope.settings = AppBuilder.settings;
    $scope.user = $rootScope.user;
    $scope.defaultUserImage = Urls.urlRemoteBuilder('sports/football/clubs/' + club.clubId + '/defaultUser.svg');
    
    $scope.hasAgrements = !!club.agreementsId && club.agreements.showEditProfile;
    $scope.agreements = club.agreements;

    $scope.terms = function() {
      if ($scope.agreements.mainContentDisplayMethod == 'popup') {
        ModalService.show('app/blades/authentification/popups/templates/agreementsPopupTemplate.html', 'AgreementsPopupController', $scope.agreements);
      } else {
        var termsUrl = 'http://' + club.subdomain + '.playabl.com/terms.php';
        if (window.cordova && cordova.InAppBrowser) {
          cordova.InAppBrowser.open(termsUrl, '_system');
        } else {
          window.open(termsUrl, 'blank');    
        }
      }
    };
    /**
    * close
    * @function
    * @description Close this modal
    */
    $scope.close = function() {
      $scope.closeModal();
    };

    /**
    * logout
    * @description this function call the AuthService logout function
    */
    $scope.logout = function () {
      AuthService.logout().then(function () {
        $rootScope.user = null;  // the view should be update, thanks to the $scope.user binding
      }, function (err) {
         if (typeof err === 'object') {
          if (err.data && err.data.userMessage) {
            err = err.data.userMessage; 
          } else {
            err = 'Oups something wrong happened';            
          }
        }
        ToastService.showNativeToast('ERROR!', err, TOAST.types.error);    
      })
      .finally(function(){
        $scope.close(true);
      });
    };

  // Name field
  $scope.showEditNameBlock = false;
  $scope.editData = {};
  $scope.showEditName = function() {
    $scope.editData.realFirstName = $rootScope.user.realFirstName;
    $scope.editData.realLastName = $rootScope.user.realLastName;
    $scope.showEditNameBlock = true;
  };

  $scope.cancelEditName = function(){
    $scope.showEditNameBlock = false;
  };

  $scope.saveName = function(){
    var update = {
      'realFirstName': $scope.editData.realFirstName,
      'realLastName': $scope.editData.realLastName
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $rootScope.user.realFirstName = $scope.editData.realFirstName;
      $rootScope.user.realLastName = $scope.editData.realLastName;
    }, function (err) {
      console.log(err);
    });
    $scope.showEditNameBlock = false;
  };

  // Username field
  $scope.showEditUsernameBlock = false;

  $scope.showEditUsername = function(){
    $scope.editData.displayName = $rootScope.user.displayName;
    $scope.showEditUsernameBlock = true;
  };

  $scope.cancelEditUsername = function(){
    $scope.showEditUsernameBlock = false;
  };

  $scope.saveUsername = function(){
    var update = {
     'displayName': $scope.editData.displayName
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $rootScope.user.displayName = $scope.editData.displayName;
      console.log('done');
    }, function (err) {
      console.log(err);
    });
    $scope.showEditUsernameBlock = false;
  };

  // Email field
  $scope.showEditEmailBlock = false;

  $scope.showEditEmail = function(){
    $scope.editData.email = $rootScope.user.email;
    $scope.showEditEmailBlock = true;
  };

  $scope.cancelEditEmail = function(){
    $scope.showEditEmailBlock = false;
  };

  $scope.saveEmail = function(){
    var update = {
     'email': $scope.editData.email
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $rootScope.user.email = $scope.editData.email;
      console.log('done');
    }, function (err) {
      console.log(err);
    });
    $scope.showEditEmailBlock = false;
  };

  // Gender field
  if ($rootScope.user.gender) {
    $scope.editData.genderStr = $rootScope.user.gender === 'm' ? 'MALE' : 'FEMALE';    
  }
  $scope.showEditGenderBlock = false;
  $scope.showEditGender = function() {
    $scope.editData.gender = $rootScope.user.gender;
    $scope.showEditGenderBlock = true;
  };

  $scope.cancelEditGender = function(){
    $scope.showEditGenderBlock = false;
  };

  $scope.saveGender = function() {
    var translatedValue = $filter('translate')('MALE');
    var update = {
      'gender': $scope.editData.genderStr == translatedValue ? 'm' : 'f'
    };
    usersService.update($rootScope.user.id, update).then(function () {
      //$scope.editData.genderStr = $scope.editData.gender === 'm' ? 'MALE' : 'FEMALE';
      $rootScope.user.gender = $scope.editData.genderStr == translatedValue ? 'm' : 'f';
    }, function (err) {
      console.log(err);
    });
    $scope.showEditGenderBlock = false;
  };

  // Country field
  $timeout(function() {
    $scope.country = ISO3166.getCountryName($rootScope.user.country);
    $scope.countryList = Object.keys(ISO3166.countryToCode);   
  });
  $scope.showEditCountryBlock = false;

  $scope.showEditCountry = function() {
    $scope.editData.country = $rootScope.user.country;
    $scope.showEditCountryBlock = true;
  };

  $scope.cancelEditCountry = function() {
    $scope.showEditCountryBlock = false;
  };

  $scope.saveCountry = function() {
    var update = {
      'country': ISO3166.getCountryCode($scope.editData.country)
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $rootScope.user.country = ISO3166.getCountryCode($scope.editData.country);
      $scope.country = $scope.editData.country;
    }, function (err) {
      console.log(err);
    });
    $scope.showEditCountryBlock = false;
  };

  // DOB field
  $scope.showEditDobBlock = false;
  var dob = new Date($rootScope.user.dateOfBirth);
  $scope.day = dob.getDate();
  $scope.month = dob.getMonth();
  $scope.year = dob.getYear();
  $scope.showEditDob = function(){
    $scope.editData.dob = $rootScope.user.dateOfBirth;
    $scope.showEditDobBlock = true;
  };

  $scope.cancelEditDob = function(){
    $scope.showEditDobBlock = false;
  };

  $scope.saveDob = function() {
    console.log($scope.editData.dob);
    var update = {
      'dateOfBirth': $scope.editData.dob
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $rootScope.user.dateOfBirth = $scope.editData.dob;
      $scope.day = $scope.editData.dob.getDate();
      $scope.month = $scope.editData.dob.getMonth();
      $scope.year = $scope.editData.dob.getYear();
    }, function (err) {
      console.log(err);
    });
    $scope.showEditDobBlock = false;
  };

  // ACCEPT TERMS
  var getUserAgreements = function() {
    if ($rootScope.user.agreements) {
      for (var i = $rootScope.user.agreements.length - 1; i >= 0; i--) {
        var agr = $rootScope.user.agreements[i];
        if (agr.agreementId == club.agreementId) {
          return agr;
        }
      };
    }  
  };
  var isAgreementsAccepeted = function() {
    var userAgr = getUserAgreements()
    if (userAgr) {
      return userAgr.isAccepted;
    } else {
      return $rootScope.user.acceptedChelseaPartnerTerms;
    }
  };
  $scope.editData.acceptedChelseaPartnerTerms = isAgreementsAccepeted();
  
  $scope.partnerTermsUpdate = function() {

    if ($scope.editData.acceptedChelseaPartnerTerms === isAgreementsAccepeted()) {
      return;
    }
    var update = {
     'acceptedChelseaPartnerTerms': $scope.editData.acceptedChelseaPartnerTerms
    };
    usersService.update($rootScope.user.id, update).then(function () {
      $scope.editData.acceptedChelseaPartnerTerms = isAgreementsAccepeted();
    }, function () {
      $scope.editData.acceptedChelseaPartnerTerms = isAgreementsAccepeted();
    });      
  };
}]);
angular.module('blades.user')
  .controller('UserFacebookLeagueModalController', UserFacebookLeagueModalController);

function UserFacebookLeagueModalController(
  $scope,
  $rootScope,
  $q,
  $filter,
  Facebook,
  usersService,
  userStatisticsService,
  friendsService,
  club,
  clubsService,
  ShareService,
  FacebookAuthService,
  AuthService,
  FacebookFriendsService,
  $ionicSlideBoxDelegate,
  fixturesService,
  challengesService,
  $state,
  ModalService,
  challengeMembersService,
  ToastService,
  TOAST
) {
  var user = $rootScope.user;
  // @TODO this should be an app wide const
  var PREDICTION_ACCURACY = 2;
  var STATS_TO_USE = 'statistics';
  var currentClub = clubsService.get(club.clubId);

  $scope.activeSlide = 0;
  $scope.title = [
    "Facebook Friends",
    "Select the stake"
  ];
  $ionicSlideBoxDelegate.$getByHandle('facebook-slidebox').enableSlide(false);
  $scope.placementBannerCount = 2;
  $scope.statToOrderBy = STATS_TO_USE + '.predictionAccuracy';
  $scope.statsLoaded = false;  
  
  function getCurrentFriends() {
    return friendsService.findAll({userId: $rootScope.user.id })
    .then(function(friends) {
      return friendsService.getAll();
    });
  }

  function refreshFacebookFriends(friendsToFilter) {
    var q = $q.defer();
    FacebookFriendsService.getFacebookFriends()
    .then(function(friendsData) {
      console.log('friendsData');
      var filteredFriends = [];
      if (friendsToFilter) {
        var filterField = _.pluck(friendsToFilter, 'facebookId');
        filteredFriends = friendsData.filter(function(FBFriend) {
          return !_.contains(filterField, FBFriend.id);
        });
      } else {
        filteredFriends = friendsData;
      }
      if (filteredFriends.length) {
        var promises = [];
        for (var i = filteredFriends.length - 1; i >= 0; i--) {
          var filterFriend = filteredFriends[i];
          var createPromise = friendsService.create({
            displayName: filterFriend.name,
            facebookId: filterFriend.id
          },{
            params: {userId: user.id}
          });
          promises.push(createPromise);
        }
        $q.all(promises).finally(q.resolve);
      } else {
        q.resolve();              
      }
    }, q.reject);
    return q.promise;
  }

  function getFBFriends(friendsToFilter) {
    return FacebookFriendsService.hasPermission('user_friends')
    .then(function() {
      return refreshFacebookFriends(friendsToFilter);
    }, function(err) {
      return FacebookAuthService.login({})
      .then(function() {
        return refreshFacebookFriends(friendsToFilter);
      });
    }); 
  }

  function getFriendStats(friends) {
    friends.map(function(friend) {
      userStatisticsService.findAll({
        'userId': friend.userId,
        'clubId': club.clubId,
        'includeGlobals': true
      }).then(function(friendStats) {
        if (friendStats && friendStats[0]) {
          friendsService.bindStats(friendStats[0]);          
        }
      });
    });
  }

  function init() {
    var allFriends = friendsService.getAll();

    if (!allFriends.length) {
      getCurrentFriends().then(function(friends) {
        getFBFriends(friends).then(function() { 
          friendsService.inject(currentUserAsFriend);
          allFriends = friendsService.getAll();
          $q.all(getFriendStats(allFriends)).then(function(){
            $scope.statsLoaded = true;
            friendsService.bindAll($scope, 'userFriends', {
              'orderBy': [
                ['predictionAccuracy', 'DESC']
              ]
            });
            setTimeout(function() {
              console.log($scope.userFriends);
            }, 2000);
          });
        });
      });
      var currentUserAsFriend = angular.extend({
        userId: user.id
      }, user);
    } else {
      $scope.statsLoaded = true;
      friendsService.bindAll($scope, 'userFriends', {
        'orderBy': [
          ['predictionAccuracy', 'ASC']
        ]
      })
    }
  }

  $scope.isFacebookUser = function() {
    return (!!$rootScope.user.facebook && !!$rootScope.user.facebook.id);
  };

  if ($scope.isFacebookUser()) {
    init();    
  }

  $scope.back = function() {
    if ($scope.activeSlide != 0) {
      $scope.activeSlide = 0;
      $ionicSlideBoxDelegate.$getByHandle('facebook-slidebox').slide($scope.activeSlide);
    } else {
      $scope.closeModal();
    }
  }


  var createChallenge = function(fixture, friend, stake) {
    return {
      "logo": "images/fbChallengeLogo.png",
      "backgroundImageUrl": "fvf/" + stake + "/challenge.jpg",
      "prizeImageUrl": 'images/balls.png',
      "style": {
        'background-image': 'url(' + 'images/fvf/' + stake + '/challenge.jpg)'
      },
      "challengeType": "fb-fvf",
      "fixtureId": fixture.id,
      "id": stake + '-' + friend.userId,
      "exclusiveMembers": [friend.userId],
      "isCreateButton": true,
      "isFanVFan":true,
      "isAuto": false,
      "isGlobal": false,
      "isNational": false,
      "isPromo": false,
      "isPublic": true,
      "isPublished": true,
      "isRegional": false,
      "isSettled": false,
      "membersCurrent": 0,
      "membersMax": 2,
      "stake": stake,
      "name": "Facebook Challenge",
      "prizeName": "balls",
      "shareText": "I am set to win 1000 balls in a fan v fan challenge",
      "losingPosition": "",
      "winningPosition": "Play one vs one against a real fan and win double your winnings! The more you put down, the more you can win back!"
    };
  };

  var availableStakes = [50, 100, 500, 1000, 2500, 10000, 50000, 100000];

  $scope.selectFriend = function(friend) {
    var invitedChallenges = challengesService.filter({
      'where': {
        'userId': friend.userId,
        'exclusiveMembers': {
          'contains': $rootScope.user.id
        },
        fixtureId: {
          '==': fixturesService.currentFixture.id
        },
        challengeType: {
          '==': 'fb-fvf'
        }
      }
    });
    console.log(invitedChallenges);
    var challenges = challengesService.filter({
      'where': {
        'exclusiveMembers': {
          'contains': friend.userId
        },
        fixtureId: {
          '==': fixturesService.currentFixture.id
        },
        challengeType: {
          '==': 'fb-fvf'
        }
      }
    });
    console.log(challenges)
    challenges = invitedChallenges.concat(challenges);
    console.log(challenges);
    $scope.existingChallenge = {}
    for (var i = challenges.length - 1; i >= 0; i--) {
      $scope.existingChallenge["" + challenges[i].stake ] = {
        'challenge': challenges[i],
        'prediction': challengeMembersService.filter({ 'challengeId': challenges[i].id })
      }
    };
    $scope.challenges = [];
    for (var i = availableStakes.length - 1; i >= 0; i--) {
      if ($scope.existingChallenge[availableStakes[i]]) {
        $scope.challenges.unshift($scope.existingChallenge[availableStakes[i]].challenge);
      } else {
        var challenge = createChallenge(fixturesService.currentFixture, friend, availableStakes[i]);
        $scope.challenges.unshift(challenge);        
      }
    };
    $scope.selectedFriend = friend;
    $scope.activeSlide = 1;
    $ionicSlideBoxDelegate.$getByHandle('facebook-slidebox').slide($scope.activeSlide);
  };

  $scope.selectChallenge = function(challenge) {
    var now = new Date();
    var stake = challenge.stake;
    var fixture = fixturesService.currentFixture;
    if (fixture && fixture.startTime < now.getTime()) {
      ToastService.showNativeToast('ERROR!', 'Sorry, you cannot predict on this match any more', TOAST.types.error);
      fixturesService.refresh(fixture);
      HelpService.finish();
      $rootScope.goBack();
      return;
    }
    var _challenge = challengesService.get(challenge.id);
    if (!_challenge) {
      if (stake > $rootScope.user.chips) {
        ModalService.show('app/blades/challenge/card/templates/popups/aintGotTheBalls.html');
        return;
      }
      challengesService.inject(challenge);
    } else {
      challenge = _challenge;
    }
    // if ($scope.existingChallenge["" + stake] && $scope.existingChallenge["" + stake].challenge) {
    //   challenge = $scope.existingChallenge["" + stake].challenge;
    // } else if (stake > $rootScope.user.chips && !$scope.prediction) {
    //   ModalService.show('app/blades/challenge/card/templates/popups/aintGotTheBalls.html');
    //   return;
    // } else {
    //   challenge = createChallenge(fixture, $scope.selectedFriend, stake);
    //   challengesService.inject(challenge);      
    // }
    $state.go('shell.predict', {
      challengeId: challenge.id
    });
  }

  $scope.linkWithFacebook = function() {
    FacebookAuthService.login({})
    .then(function(authData) {
      var update = {};
      authData = authData._request;
      authData.displayName = authData.username;
      delete authData.username;
      for (var key in authData) {
        if (authData[key] && !$rootScope.user[key]) {
          update[key] = authData[key];
        }
      }
      update.facebook = authData.facebook;
      usersService.update($rootScope.user.id, update)
      .then(function () {
        init();
      }, function(err) {
        //show error toast
        console.log(err);
        if (err && err.data && err.data.userMessage) {
          ToastService.showNativeToast('ERROR!', err.data.userMessage, TOAST.types.error);            
        }
      });
    }, function(err) {
      if (err && err.userMessage) {
        ToastService.showNativeToast('ERROR!', err.userMessage, TOAST.types.error);            
      } else {
        ToastService.showNativeToast('ERROR!', err.userMessage, TOAST.types.error);    
      }

      // show error toast
      console.log(err);
    }); 
  };

  $scope.inviteFriends = function inviteFriends(provider) {
    var baseUrl = 'https://playabllive.blob.core.windows.net/';
    var pathToClub = 'images/sports/football/clubs/' + currentClub.id;
    var pathToSharePic = '/icons/android/icon.png';
    var pictureUrl = baseUrl + pathToClub + pathToSharePic;
    var caption = $filter('translate')('See if you can outscore me on Predictabl!');

    var shareInfo = {
      url: 'http://'+currentClub.subdomain+'.playabl.com',
      title: 'PredictablTitle',
      caption: caption,
      description: 'Predictabl',
      picture: pictureUrl
    };
    ShareService.invite(provider, shareInfo);
  };


  $scope.isCurrentUser = function isCurrentUser(friend) {
    return friend.userId === user.id;
  };

  $scope.friendPredictionAccuracy = function friendPredictionAccuracy(friend) {
    if (!friend) return 0;
    var predictionAccuracy = friend[STATS_TO_USE] ?
      friend[STATS_TO_USE].predictionAccuracy : 0;
    return predictionAccuracy.toFixed(PREDICTION_ACCURACY);
  };

  $scope.onSlideChanged = function() {
    $scope.activeSlide = $ionicSlideBoxDelegate.$getByHandle('facebook-slidebox').currentIndex();
  };

  $scope.friendChipsWon = function friendChipsWon(friend) {
    if (!friend) return 0;
    var chipsWon = friend[STATS_TO_USE] ?
      friend[STATS_TO_USE].totalChipsWon : 0;

    return $filter('number')(chipsWon);
  };

  $scope.friendTotalPoints = function friendTotalPoints(friend) {
    if (!friend) return 0;
    var totalPoints = friend[STATS_TO_USE] ?
      friend[STATS_TO_USE].totalPoints : 0;
    return $filter('number')(totalPoints);
  };

  $scope.removeFriend = function removeFriend(friend) {
    if (friend) {
      var friendToRemove = friendsService.filter({
        where: {
          id: friend.id
        }
      });

      friendsService.destroy(friendToRemove, {
        params: {
          userId: user.id
        },
      });
    }
  };
}

UserFacebookLeagueModalController.$inject = [
  '$scope',
  '$rootScope',
  '$q',
  '$filter',
  'Facebook',
  'usersService',
  'userStatisticsService',
  'friendsService',
  'club',
  'clubsService',
  'ShareService',
  'FacebookAuthService',
  'AuthService',
  'FacebookFriendsService',
  '$ionicSlideBoxDelegate',
  'fixturesService',
  'challengesService',
  '$state',
  'ModalService',
  'challengeMembersService',
  'ToastService',
  'TOAST'
];

angular.module('blades.user').factory('UserModalService', [
  '$ionicModal',
  function ($ionicModal) {
    var UserModalService = {};
    UserModalService.showModal = function () {
      $ionicModal.fromTemplateUrl('app/blades/user/modal/templates/userPopupTemplate.html').then(function (modal) {
        UserModalService.userModal = modal;
        UserModalService.userModal.show();
      });
    };
    UserModalService.hideModal = function (callback) {
      UserModalService.userModal.hide()
      .then(function () {
        UserModalService.userModal.remove();
      })
      .then(callback);
    };

    UserModalService.showEditProfileModal = function () {
      $ionicModal.fromTemplateUrl('app/blades/user/modal/templates/edituserPopupTemplate.html').then(function (modal) {
        UserModalService.userModal = modal;
        UserModalService.userModal.show();
      });
    };
    return UserModalService;
  }
]);
/**
* @ngdoc directive
* @name userMenuButton
* @description this directive encapsulate the user menu button
* @example
* Usage:
* <user-menu-button></user-menu-button>
*/
angular.module('blades.user')
.directive('userMenuButton', [
	function () {
    	return {
	      restrict: 'E',
	      templateUrl: 'app/blades/user/menuButton/templates/userMenuButtonTemplate.html',
	      controller: 'UserMenuButtonController'
	    };
  	}
]);
/**
* ngdoc controller
* @name UserMenuButtonController
* @requires $scope $state Urls club UserModalService
* @description write the description here ;)
*/
angular.module('blades.user').controller('UserMenuButtonController', [
  '$scope',
  '$state',
  'Urls',
  'club',
  'UserModalService',
  'ModalService',
  function ($scope, $state, Urls, club, UserModalService, ModalService) {
    $scope.defaultUserImage = Urls.urlRemoteBuilder('sports/football/clubs/' + club.clubId + '/defaultUser.svg');
    $scope.goToLogin = function () {
      $state.go('shell.login');
    };
    $scope.goToUserProfile = function () {
      ModalService.show('app/blades/user/modal/templates/userPopupTemplate.html', 'UserModalController');
      // is it supposed to be a popup ? a view ? anyway, do that here :)
      //UserModalService.showModal();
    };
  }
]);
angular.module('shell.directives')
	.directive('userStatBox', userStatBoxDirective);

function userStatBoxDirective(){
	return {
		templateUrl: 'app/blades/user/userStatBox/template.html',
		scope: {
			iconClass: '@icon',
			total: '@',
			otherTotal: '@',
			label: '@'
		}
	}
}

angular.module('shell.directives')
	.directive('banner', bannerDirective);

function bannerDirective() {
	return {
		templateUrl: 'app/blades/user/banner/template.html',
		transclude: true
	};
}

angular.module('blades.user')
	.directive('avatar', avatarDirective)
	.controller('AvatarController', AvatarController);

function avatarDirective() {
	return {
		templateUrl: 'app/blades/user/avatar/template.html',
		scope: {
			info: '=',
			user: '=',
			mode: '@',
			select: '&',
			owned: '&'
		},
		controller: 'AvatarController'
	};
}

function AvatarController($scope, $filter, ASSET_PATH) {
	$scope.isSelectMode = function isSelectMode() {
		return $scope.mode && $scope.mode === 'select';
	};

	$scope.avatarImage = function avatarImage() {
		if ($scope.isSelectMode()) {
			return $scope.info.productImage;
		} else {
			if ($scope.user) {
				if ($scope.user.avatarUrl) {
					return $scope.user.avatarUrl;
				}
				if ($scope.user.imageUrl) {
					return $scope.user.imageUrl;
				}
			}
			return '';			
		}
	};

	$scope.selectAvatar = function selectAvatar() {
		if ($scope.avatarIsLocked() || $scope.isCurrentAvatar()) {
			return;
		}
		$scope.select($scope.info);
	};

	$scope.avatarIsLocked = function avatarIsLocked() {
		return $scope.info &&
			$scope.user.currentClubLoyalty.level < $scope.info.level;
	};

	$scope.isDefaultAvatar = function isDefaultAvatar(item) {
		var defaultAvatarRegExp = new RegExp(ASSET_PATH.DEFAULT_USER_IMAGE, 'i');
		var facebookAvatarRegExp = new RegExp(ASSET_PATH.FACEBOOK_IMAGE,'i');

		return defaultAvatarRegExp.test(item.productImage) ||
			facebookAvatarRegExp.test(item.productImage);
	};

	$scope.isCurrentAvatar = function isCurrentAvatar() {
		$scope.user.avatarUrl = $scope.user.avatarUrl || '';

		return $scope.info.productImage === $scope.user.avatarUrl;
	};

	$scope.avatarHasCost = function avatarHasCost() {
		return $scope.info.totalPrice > 0;
	};

	$scope.avatarAvailableToUser = function avatarAvailableToUser() {
		return !$scope.isCurrentAvatar() && !$scope.avatarIsLocked();
	};

	$scope.displayItemCost = function displayItemCost(item){
		if(item.totalPrice === 0 ) {
			return $filter('translate')('FREE');
		} else {
			return $filter('number')(item.totalPrice, 0);
		}
	};
}

AvatarController.$inject = ['$scope','$filter', 'ASSET_PATH'];

angular.module('blades.user')
	.controller('AvatarPopupController', avatarPopupController);

function avatarPopupController($scope, $rootScope, $state, $q, PopupService, usersService, productsService, Urls, club, ASSET_PATH, userProducts, ToastService, TOAST, $timeout) {
	$scope.avatars = [];
	var defaultAvatar = !$rootScope.user.facebook.id ?
		ASSET_PATH.CLUBS + club.clubId + '/' + ASSET_PATH.DEFAULT_USER_IMAGE :
		ASSET_PATH.USERS + $rootScope.user.id + ASSET_PATH.FACEBOOK_IMAGE;
	$scope.avatars = productsService.filter({
		type: 'avatars',
		orderBy: [
            ['totalPrice', 'ASC']
        ]
	});

	$scope.avatars.unshift({
		id: null,
		productImage: Urls.urlRemoteBuilder(defaultAvatar)
	});

	userProducts.bindAll($scope, 'userProducts');

	$scope.isOwned = function isOwned(avatar) {
		for (var i = $scope.userProducts.length - 1; i >= 0; i--) {
        	if ($scope.userProducts[i].id === avatar.id) {
          		return avatar.numberOfItems === $scope.userProducts[i].numberPurchased;
        	}
      	}
      	return false;
    };


	$scope.closePopup = function closePopup() {
		PopupService.hide();
	};

	function makePromise(object, attrToReturn) {
		var deferred = $q.defer();
		deferred.resolve(object[attrToReturn]);
		return deferred.promise;
	};
	var buyAvatar = function(avatar) {
		var deferred = $q.defer();
		if (avatar.totalPrice > $rootScope.user.chips) {
			$scope.pulse = true;
			$timeout(function() {
				$scope.pulse = false;
			}, 1000);
		} else {
			userProducts.update(avatar.id, {id: avatar.id, 'numberPurchased': 1}, {'params': {'userId': $rootScope.user.id} })
		    .then(function(result) {
		        setAvatar(result);
		        deferred.resolve();
		    }, function(err) {
		        console.log(err);
		        ToastService.showNativeToast('ERROR!', '', TOAST.types.error);
		        deferred.reject(err);
		    });
		}
		return deferred.promise;
	};

	var setAvatar = function(avatar) {
		var productUrl;
		if (avatar.id) {
			productUrl = productsService.find(avatar.id)
				.then(function(product){
					return product.productImage;
				});
		} else {
			productUrl = makePromise(avatar, 'productImage');
		}

		productUrl.then(function(avatarImageUrl) {
			usersService.inject({
				id: $rootScope.user.id,
				'avatarUrl': avatarImageUrl
			});

			PopupService.hide();

			usersService.update($rootScope.user.id, {
				'avatarUrl': avatarImageUrl
			});
		})
		.catch(function(err){
			console.log('PROD ERR', err);
		});
	};

	$scope.selectAvatar = function setAvatarImage(avatar) {
		if ($scope.isOwned(avatar) == true || !avatar.id) {
			setAvatar(avatar);
		} else {
			buyAvatar(avatar).then(function() {
				setAvatar(avatar);
			}, function(err) {
				console.log(err);
			});
		}
	};

	$scope.getMore = function getMore() {
		PopupService.hide();
		$state.go('shell.shop', {type: 'coins'});
	};
}

avatarPopupController.$inject = ['$scope', '$rootScope','$state','$q','PopupService','usersService','productsService', 'Urls', 'club','ASSET_PATH', 'userProducts', 'ToastService', 'TOAST', '$timeout'];

/**
* @module blade.challenge
* @description This module contains the challenge view. It register to the side menu and add a new view, shell.challenge
* @requires $stateProvider
*/
angular.module('blades.challenge', []).run([
  '$rootScope',
  'challengerFoundPopupService',
  'NavigationService',
  'fixturesService',
  '$state',
  function($rootScope, challengerFoundPopupService, NavigationService, fixturesService, $state) {
    $rootScope.$on('user.logged.in', function () {
      challengerFoundPopupService.startLookingForChallengers();
    });
    NavigationService.registerRoute('shell.challenges', function(args) {
      if (args.fixtureId) {
        // if a specific fixtureId is provided
        $state.go('shell.challenges', args);
      } else {
        // else try to navigate to the current fixture challenges
        if (fixturesService.currentFixture) {
          $state.go('shell.challenges', { 'fixtureId': fixturesService.currentFixture.id });          
        }
      }
    });
  }
]).config([
  '$stateProvider',
  function ($stateProvider) {    
    $stateProvider.state('shell.challenges', {
      url: 'challenges/:fixtureId/:predictionId',
      showMatchReport: true,      
      showNewsFeed: true,
      canGoBack: true,
      views: {
        'tab-main': {
          requiresLogin: false,
          templateUrl: 'app/blades/challenge/view/challengeList/templates/challengeListTemplate.html',
          controller: 'ChallengeListController',
          resolve: {
            challenges: function($timeout, $q) {
              var q = $q.defer();
              $timeout(function() {
                q.resolve();
              }, 750);
              return q.promise;
            }
          }
        }
      }
    });
  }
]);

'use strict';
angular.module('blades.challenge').controller('ChallengeListController', [
  '$scope',
  '$state',
  '$rootScope',
  '$location',
  'challengesService',
  '$stateParams',
  '$ionicScrollDelegate',
  'fixturesService',
  '$timeout',
  'club',
  'clubsService',
  'ModalService',
  function ($scope, $state, $rootScope, $location, challengesService, $stateParams, $ionicScrollDelegate, fixturesService, $timeout, club, clubsService, ModalService) {
    var handle;
    $scope.isIOS = true || ionic.Platform.isIOS();

    $scope.show = false;
    $scope.$on('$stateChangeStart', function (event, toState) {
      if (toState.name !== 'shell.challenges') {
        $scope.show = false;
      }
    });

    $scope.isCurrentFixture = false;

    $scope.goToFacebookChallenges = function() {
      ModalService.show('app/blades/user/modal/templates/facebookFriendsLeagueTemplate.html', 'UserFacebookLeagueModalController');
    };

    $scope.selectChallenge = function(challenge) {
        var now = new Date();
        if (fixture && fixture.startTime < now.getTime()) {
          ToastService.showNativeToast('ERROR!', 'Sorry, you cannot predict on this match any more', TOAST.types.error);
          fixturesService.refresh(fixture);
          HelpService.finish();
          $rootScope.goBack();
          return;
        }
        if (challenge.stake > 0) {
          if (!$rootScope.user) {
            $state.go('shell.login');
            return;
          } else if (challenge.stake > $rootScope.user.chips && !$scope.prediction) {
            $scope.aintGotTheBalls();
            return;
          }          
        }

        $state.go('shell.predict', {
          challengeId: challenge.id
        });
      };

    $scope.ready = false;
    $scope.$on('$ionicView.afterLeave', function () {
      $scope.ready = false;
    });
    
    $scope.$on('$ionicView.afterEnter', function () {
      $scope.ready = true;
      $scope.show = true;
      $scope.fixture = fixturesService.get($stateParams.fixtureId);
      $scope.isCurrentFixture = fixturesService.currentFixture.id == $scope.fixture.id;
      var params = {
        fixtureId: $stateParams.fixtureId,
        isPublished: true,
        orderBy: [
          ['isFanVFan', 'ASC'],
          ['isGlobal', 'DESC'],
          ['stake', 'ASC']
        ]
      };
      var currentClub = clubsService.get(club.clubId);
      if (currentClub.disableFanVsFan == true && false) {
        params.isFanVFan = false;
      }
      /**
      * This is a hack to hide challenges with physical prizes during the apple review
      */
      if ($scope.isIOS !== true && (!$rootScope.user || ($rootScope.user && $rootScope.user.email === 'review@apple.com') ) ) {
        params.isPhysicalPrize = false;
        params.isNationnal = false;
      }
      challengesService.bindAll($scope, 'challenges', params, function() {
        if ($scope.loading === true) {
          $timeout(function() {
            handle = $ionicScrollDelegate.$getByHandle('challenge-scroll');
            handle.scrollTo(100, false);
          }, 0);
        }
        //$scope.show = false;
      });
    });
  }
]);

'use strict';

/**
* @ngdoc directive
* @name challangeCard
* @description encapsulate the challenge card
* @example
* Usage:
* <challenge-card></challenge-card>
*/
angular.module('blades.challenge').directive('challengeCard', [// Inject the dependencies you need
  function () {
    return {
      restrict: 'EA',
      templateUrl: 'app/blades/challenge/card/templates/challengeCardTemplate.html',
      scope: { challenge: '=', 'joinChallenge': '&', 'useDefault': '='},
      controller: 'challengeDirectiveController'
    };
  }]).controller('challengeDirectiveController', [
  '$scope',
  '$rootScope',
  '$timeout',
  '$state',
  'ModalService',
  'UserSettings',
  'Urls',
  'club',
  'fixturesService',
  'predictionsService',
  'userChallengesService',
  'challengeMembersService',
  'ToastService',
  'TOAST',
  'clubsService',
  'HelpService',
  'ASSET_PATH',
  function ($scope, $rootScope, $timeout, $state, ModalService, UserSettings, Urls, club, fixturesService, predictionsService, userChallengesService, challengeMembersService, ToastService, TOAST, clubsService, HelpService, ASSET_PATH) {

      var challenge = $scope.challenge;
      //@TODO: This should be an App Const
      var MINIMUM_PLAYERS_TO_SHOW = 50;
      $scope.challengeOptions = {
        minimumPlayersToShow: MINIMUM_PLAYERS_TO_SHOW
      };

      //$scope.clubMascott = Urls.urlBuilder('sports/football/clubs/' + club.clubId + '/clubMascott.jpg');

      $rootScope.$on('$ionicView.afterEnter', function () {
        initialiseChallenges();
      }, false);
      $scope.challengeMembers = challengeMembersService.filter({'challengeId': challenge.id});
      if (challenge.membersMax == 2) {
        console.log(challenge);
      }
      function initialiseChallenges() {
        var userChallenge;
        if ($rootScope.user && $rootScope.user.id){
          userChallenge = userChallengesService.get(
            challenge.id, 
            {params:{userId:$rootScope.user.id}}
          );
        }

        $scope.defaultUserImage = Urls.urlRemoteBuilder('sports/football/clubs/' + club.clubId + '/defaultUser.svg');
        if (challenge.exclusiveMembers && challenge.exclusiveMembers.length >= 1) {
          var member = challenge.exclusiveMembers[0];
          console.log(member);
          $scope.exclusifMember = Urls.urlRemoteBuilder(ASSET_PATH.USERS + member + '/100x100/profilepic.jpg');          
        }
        $scope.isChallengeMember =  false;

        if (userChallenge) {
          $scope.isChallengeMember =  true;

          $scope.userChallenge = userChallenge;
          var prediction = predictionsService.filter({
            challengeId:challenge.id
          });

          if(prediction && prediction.length > 0){
            $scope.prediction = prediction[0];
            if($scope.prediction.userResults){
              $scope.userResults = $scope.prediction.userResults;
            }
          }

        }
      }
      initialiseChallenges();


      var fixture = fixturesService.get(challenge.fixtureId);
      $scope.fixture = fixture;

      var enterChallenge = function() {
        var now = new Date();
        if (fixture && fixture.startTime < now.getTime()) {
          ToastService.showNativeToast('ERROR!', 'Sorry, you cannot predict on this match any more', TOAST.types.error);
          fixturesService.refresh(fixture);
          HelpService.finish();
          $rootScope.goBack();
          return;
        }
        if (challenge.stake > 0) {
          if (!$rootScope.user) {
            $state.go('shell.login');
            return;
          } else if (challenge.stake > $rootScope.user.chips && !$scope.prediction) {
            $scope.aintGotTheBalls();
            return;
          }          
        }

        $state.go('shell.predict', {
          challengeId: challenge.id
        });
      };
      $scope.enterChallenge = function() {
        if ($scope.useDefault) {
          enterChallenge();
        } else {
          $scope.joinChallenge(challenge);
        }
      }
      $scope.aintGotTheBalls = function(){
        ModalService.show('app/blades/challenge/card/templates/popups/aintGotTheBalls.html');
      };   

      $scope.challengePrizeImage = challenge.prizeImageUrl;
      $scope.challengeLogo = challenge.logo;
      
      $scope.homeClub = clubsService.get(fixture.homeClubId);
      $scope.awayClub = clubsService.get(fixture.awayClubId);
      
  }
]);
angular.module('blades.challenge').factory('challengerFoundPopupService', [
  '$q',
  'ModalService',
  '$rootScope',
  'challengeMembersService',
  'challengesService',
  function ($q, ModalService, $rootScope, challengeMembersService,challengesService) {
    var challengerFoundPopupService = {
      looking: null,
      newChallengers: [],
      popup: null
    };
    var REFRESH_INTERVAL = 40000; // 40 sec

    challengerFoundPopupService.stopLookingForChallengers = function() {
      clearInterval(challengerFoundPopupService.looking);
      challengerFoundPopupService.looking = null;
    };

    challengerFoundPopupService.startLookingForChallengers = function() {
      if (!challengerFoundPopupService.looking) {
        challengerFoundPopupService.looking = setInterval(function() {
          lookForChallengers();
        }, REFRESH_INTERVAL);
      }
    };

    // get challenges that are FAN V FAN
    // 
    function lookForChallengers() {
      var challenges = challengesService.filter({
        isFanVFan:true, isCreateButton:false, membersCurrent:1
      });

      // get a list of all the challenges awaiting Challengers
      var challengeMemberCounts = [];
      var challengePromoises = [];
      var newChallengers = [];
      for (var i = challenges.length - 1; i >= 0; i--) {
        challengeMemberCounts.push(challenges[i].id);
        challengePromoises.push(challengesService.find(challenges[i].id, {bypassCache:true}));
      }


      $q.all(challengePromoises)
      .then(function() {
        var newChallenge = null;
        var newMembersPromises = [];
        for (var i = challengeMemberCounts.length - 1; i >= 0; i--) {
           newChallenge = challengesService.get(challengeMemberCounts[i]);
           if(newChallenge && newChallenge.membersCurrent === 2) {
            
            newChallengers.push(newChallenge.id);

            newMembersPromises.push(
              challengeMembersService.findAll(
                {'challengeId': newChallenge.id},
                {bypassCache:true}
              )
            );
           }
        }

        $q.all(newMembersPromises)
          .then(function(){
            if(newChallengers.length > 0){
              // reset new global challengers
              challengerFoundPopupService.newChallengers = newChallengers;
              challengerFoundPopupService.showPopup();            
            }
          });
      });
    };

    challengerFoundPopupService.getNewChallengers = function() {
      return challengerFoundPopupService.newChallengers;
    };

    challengerFoundPopupService.showPopup = function () {
      challengerFoundPopupService.stopLookingForChallengers();
      ModalService.show('app/blades/challenge/popups/templates/challengerFoundPopupTemplate.html', 'ChallengerFoundPopupController')
      .finally(function() {
        challengerFoundPopupService.startLookingForChallengers();
      });
    };
    
    return challengerFoundPopupService;
  }
]);
'use strict';
angular.module('blades.challenge').controller('ChallengerFoundPopupController', [
  '$rootScope',
  '$scope',
  '$state',
  'challengesService',
  'challengeMembersService',
  'fixturesService',
  'clubsService',
  'challengerFoundPopupService',
  function ($rootScope, $scope, $state, challengesService, challengeMembersService, fixturesService, clubsService, challengerFoundPopupService) {
    
    $scope.close = function () {
      $scope.closeModal();
    };
    if(!$rootScope.user || !$rootScope.user.id){
      $scope.close();
      return;
    }

    var challengers = challengerFoundPopupService.getNewChallengers();

    var viewChallengers = [];

    for (var i = challengers.length - 1; i >= 0; i--) {
      
      var challengeMembers = challengeMembersService.filter({challengeId:challengers[i]});
      var challengeMember = null;

      for (var j = challengeMembers.length - 1; j >= 0; j--) {
        if (challengeMembers[j].userId != $rootScope.user.id) {
          challengeMember = challengeMembers[j];
        }
      };

      if (challengeMember) {
        viewChallengers.push({
          displayName: challengeMember.displayName,
          imgUrl: challengeMember.imgUrl,
          challengeName: getChallengeName(challengers[i]),
          challengePot: getChallengePot(challengers[i]),
          homeClubLogo: getHomeClubImgUrl(challengers[i]),
          awayClubLogo: getAwayClubImgUrl(challengers[i]),
          fixtureId: getFixtureByChallengeId(challengers[i]).id
        });
      }
      

    };
    $scope.challengers = viewChallengers;

    function getChallengeName(challengeId) {
      var challenge = challengesService.get(challengeId);
      return challenge.name;
    }
    
    function getChallengePot(challengeId) {
      var challenge = challengesService.get(challengeId);
      return challenge.pot;
    }
    function getHomeClubImgUrl(challengeId) {
      var data = getFixtureByChallengeId(challengeId);
      data = clubsService.get(data.homeClubId);
      return data.imgUrl;
    }
    function getAwayClubImgUrl(challengeId) {
      var data = getFixtureByChallengeId(challengeId);      
      data = clubsService.get(data.awayClubId);
      return data.imgUrl;
    }

    function getFixtureByChallengeId(challengeId){
      var challenge = challengesService.get(challengeId);
      return fixturesService.get(challenge.fixtureId);
    }

    $scope.goToChalenge = function(fixtureId){
      $scope.close();
      $state.go('shell.challenges', {
        fixtureId: fixtureId
      });
    }    
  }
]);


  


angular.module('blades.challenge').factory('ChallengeWonService', [
  '$rootScope',
  'ModalService',
  'fixturesService',
  'challengesService',
  'UserSettings',
  function($rootScope, ModalService, fixturesService, challengesService, UserSettings) {
    var ChallengeWonService = {
      challengesWon: [],
      lastFixture: null
    };

    function userHasSeenLastChallenge() {
      var challengeWon = UserSettings.getData('challengeWon');
      return !challengeWon ||
          challengeWon.fixtureId !== ChallengeWonService.lastFixture.id;
    }

    ChallengeWonService.checkWonChallenges = function(userId) {
      var challengeLastFixture, challengesWon;

      var fixtures = fixturesService.filter({
        where: {
          state: {
            '==': 'fulltime',
          },
          winnerId: {
            '==': userId
          }
        },
        orderBy: [
          ['startTime', 'DESC']
        ]
      });

      if (fixtures.length) {
        challengeLastFixture = fixtures[0];
        challengesWon = challengesService.filter({
          fixtureId: challengeLastFixture.id
        });
      } else {
        challengeLastFixture = null;
        challengesWon = null;
      }

      ChallengeWonService.lastFixture = challengeLastFixture;
      ChallengeWonService.challengesWon = challengesWon;
    };


    ChallengeWonService.fireChallengeWon = function() {
      var userSeenChallengeWon = userHasSeenLastChallenge();

      if (ChallengeWonService.challengesWon && userSeenChallengeWon) {
        ModalService.show(
          'app/blades/challenge/popups/templates/challengeWonPopupTemplate.html',
          'ChallengeWonPopupController',
          ChallengeWonService.challengesWon
        ).finally(function() {
          UserSettings.updateData('challengeWon', {
            fixtureId: ChallengeWonService.lastFixture.id
          });
        });
      }
    };

    return ChallengeWonService;
  }
]);

angular.module('blades.challenge')
  .controller('ChallengeWonPopupController', [
  '$scope',
  'parameters',
  'ShareService',
  function ($scope, parameters, ShareService) {

  	$scope.challengesWon = parameters;

  	$scope.share = function(provider) {
  		var shareData = {};
      	ShareService.share(provider, shareData);
    };

    $scope.close = function () {
      $scope.closeModal();
    };
  }
]);

angular.module('blades.challenge')
  .directive('titleRibbon', titleRibbonDirective);

function titleRibbonDirective() {
  return {
    templateUrl: 'app/blades/challenge/directives/ribbon/template.html',
    transclude: true,
    scope: {
      type: '@'
    }
  };
};

angular.module('blades.challenge')
  .directive('prizeBox', prizeBoxDirective);

function prizeBoxDirective($filter){
  return {
    templateUrl: 'app/blades/challenge/directives/prizeBox/template.html',
    scope: {
      prize: '=',
      type: '@',
      options: '='
    },
    link: function($scope) {
      var FAN_STAKE_MULTIPLIER = 2;

      function fanStakeValue() {
        var fanStakeAmount = $scope.prize.stake * FAN_STAKE_MULTIPLIER;
        return $filter('number')(fanStakeAmount, 0);
      }

      $scope.entryFee = function entryFee(){
        if($scope.prize.stake){
          return $filter('number')($scope.prize.stake, 0);
        } else {
          return $filter('translate')('FREE');
        }
      };

      $scope.prizeTitle = function prizeTitle(){
        if($scope.type === 'ticket') {
          return $filter('translate')('GOLDEN TICKET');
        } else if($scope.type === 'fan'){
          return $filter('translate')('PRIZE') + ' ' + fanStakeValue();
        } else {
          return $scope.prize.name;
        }
      };

      $scope.isTicket = function isTicket(){
        return $scope.type === 'ticket';
      };
    }
  };
}

prizeBoxDirective.$inject = ['$filter'];

  
angular.module('blades.endorsement', [])
.run([
  'AppBuilder',
  function (AppBuilder) {
    AppBuilder.registerMenu({
      menu: 'top',
      side: 'left',
      template: '<endorsement-menu-button></endorsement-menu-button>'
    });
  }
]);
angular.module('blades.endorsement').controller('EndorsementPopupController', [
  '$scope',
  'parameters',
  '$ionicSlideBoxDelegate',
  '$timeout',
  '$rootScope',
  'pachingAdService',
  function ($scope, parameters, $ionicSlideBoxDelegate, $timeout, $rootScope, pachingAdService) {

  	$scope.endorsements = parameters;

    $scope.activeTab = 0;

    $timeout(function() {
      $ionicSlideBoxDelegate.update();
    });

    $scope.next = function() {
      $ionicSlideBoxDelegate.next();
    };

  	$scope.click = function(endorsement) {
  		var url = endorsement.externalOptInURL;
      console.log(endorsement);

      if(!url){
        return;
      }

      if($rootScope.user && $rootScope.user.gatewayAd){
        pachingAdService.regiserPayment(
          $rootScope.user.id, 
          'inApp.playabl.notification', 
          endorsement.title, 
          endorsement.captureRegion, 
          0.003
        );
      }
      if (window.cordova && cordova.InAppBrowser) {
          cordova.InAppBrowser.open(url, '_system');
      } else {
          window.open(url, 'blank');    
      }
    };


    $scope.close = function () {
      $scope.closeModal();
    };
  }
]);

/**
* ngdoc controller
* @name PaymentCardController
* @requires $scope
* @description write the description here ;)
*/
angular.module('blades.endorsement').controller('EndorsementMenuButtonController', [
  '$scope',
  'UserSettings',
  '$timeout',
  'ModalService',
  'endorsementsService',
  function ($scope, UserSettings, $timeout, ModalService, endorsementsService) {
    
    var self =  this;
    $scope.endorsements = [];
    
    var setNotificationCount = function() {
      // need to remove this timeout, and find out why it doesnt work without
      self.ids = UserSettings.getData('endorsementIds');
      $timeout(function () {


        endorsementsService.bindAll($scope,'endorsements');

        var notificationCount = 0;
        
        for (var i = $scope.endorsements.length - 1; i >= 0; i--) {
          if(_.indexOf(self.ids, $scope.endorsements[i].id) == -1){
            notificationCount++;
          }
        };

        $scope.notificationCount = notificationCount; 
      }, 1000);
    };
    setNotificationCount();
    $scope.$on('build.rebuild.succeeded', function() {
      setNotificationCount();
    });
    $scope.onClick = function () {

      for (var i = $scope.endorsements.length - 1; i >= 0; i--) {
        if(_.indexOf(self.ids, $scope.endorsements[i].id) == -1){
          self.ids.push($scope.endorsements[i].id);
        }
      };
      UserSettings.setData({'endorsementIds':self.ids});
      
      $scope.notificationCount = 0;

      ModalService.show('app/blades/endorsement/popup/templates/endorsementPopupTemplate.html', 'EndorsementPopupController', $scope.endorsements);
    };
  }
]);
/**
* @ngdoc directive
* @name settingsCard
* @description settings card button on the bottom left bar
* @example
* Usage:
* <settings-card></settings-card>
*/
angular.module('blades.endorsement')
.directive('endorsementMenuButton', [function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/endorsement/menuButton/templates/endorsementMenuButtonTemplate.html',
      controller: 'EndorsementMenuButtonController'
    };
  }]);
angular.module('blades.help', [])
.run([
  'AppBuilder',
  function (AppBuilder) {
  }
]);
angular.module('blades.help').factory('HelpPopupService', [
  '$rootScope',
  'PopupService',
  function ($rootScope, PopupService) {
    var HelpPopupService = {};

    HelpPopupService.showPopup = function (message, lastMessage) {
      var popupScope = $rootScope.$new();
      popupScope.helpMessage = message;
      popupScope.hint = 
      popupScope.lastMessage = lastMessage || false;

      PopupService.show({
        templateUrl: 'app/blades/help/popup/templates/helpPopupTemplate.html',
        scope: popupScope
      });
    };

    HelpPopupService.hidePopup = function () {
      PopupService.hide();
    };

    return HelpPopupService;
  }
]);

angular.module('blades.help').factory('HelpService', [
  'UserSettings',
  '$timeout',
  'HelpPopupService',
  'AuthService',
  '$rootScope',
  'ModalService',
  function (UserSettings, $timeout, HelpPopupService, AuthService, $rootScope, ModalService) {
    var HelpService = {
      deregister: null
    };
    HelpService.isActive = UserSettings.getData('firstTime');
    HelpService.observers = [];

    var helpMessage = {
      //"shell.fixtures": { title: "Welcome to the game", message: "Choose your team's next match to play", hint: 'Earn bonus points by predicting early on future fixtures'},
      "shell.challenges": { title: "New challenges await you", message: "Join the Global challenge to play against fans all over the world!", hint: 'Different challenges have different prizes to win'},
      "shell.predict": { title: "Predict the score", message: "Choose the score and scorers for each team", hint: 'Get bonus points for predicting the correct player scoring order'},
      "shell.login": { title: "Save your score", message: "Create a profile or login to save your score", hint: 'The more complete your profile, the more challenges you can join'}
    };

    HelpService.register = function(observer) {
      if (HelpService.isActive) {
        HelpService.observers.push(observer);
        observer.update(HelpService.isActive);
        return true;
      } else {
        return false;
      }
    };
    HelpService.finish = function() {
      HelpService.isActive = false;
      UserSettings.setData({'firstTime':false});
      HelpService.update();
      if (HelpService.deregister) {
        HelpService.deregister();        
      }
    };
    HelpService.update = function() {
      for (var i = HelpService.observers.length - 1; i >= 0; i--) {
        HelpService.observers[i].update(HelpService.isActive);
      };
    };
    HelpService.unregisterAll = function() {
      for (var i = HelpService.observers.length - 1; i >= 0; i--) {
        HelpService.unregister(HelpService.observers[i]);
      };
    };
    HelpService.unregister = function(observer) {
      var idx = HelpService.observers.indexOf(observer);
      if (idx != -1) {
        HelpService.observer.splice(idx, 1);
      }
    };

    if (HelpService.isActive) {
      HelpService.deregister = $rootScope.$on('$stateChangeSuccess', function(evt, toState) {
        if (HelpService.isActive == false) {
          return;
        }
        var msg = helpMessage[toState.name];
        if (AuthService.isAuthenticated()) {
          if ((toState.name == 'shell.fixtures' || toState.name == 'shell.challenges') ) {
            $timeout(function() {
              ModalService.show('app/blades/help/popup/templates/helpPopupTemplate.html', 'HelpPopupController', {
                title: "Play more",
                message: "Now join other challenges for the chance to win more",
                hint: 'New challenges get released every week so keep checking back'
              });
              HelpService.finish();
            });
          }
        } else if (msg) {
          $timeout(function() {
            ModalService.show('app/blades/help/popup/templates/helpPopupTemplate.html', 'HelpPopupController', msg);
          });
        }
      });
    }

    return HelpService;
  }
]);

'use strict';
angular.module('blades.help').controller('HelpPopupController', [
  '$scope',
  'parameters',
  function ($scope, parameters) {

    $scope.helpMessage = parameters;

    $scope.close = function () {
      $scope.closeModal();
    };
  }
]);


angular.module('blades.help')
.directive('bigLockForTutorial', [
  'HelpService',
  '$filter',
  function (HelpService, $filter) {
    var lockClass = 'big-locked-for-tutorial';
    return {
      restrict: 'AC',
      link: function (scope, element) {
        if (HelpService.isActive) {
          var registerObject = {
            update: function(isActive) {
              if (isActive) {
                element.addClass(lockClass);
                var text = $filter('translate')("OPEN AFTER TUTORIAL");
                var lockOverlay = angular.element('<div id="color-overlay" class="color-overlay"><div class="locker"></div><span class="text"> ' + text + '</span></div>');
                element.append(lockOverlay);     
              } else {
                var overlayElement = angular.element( document.querySelector( '#color-overlay' ) );
                if (overlayElement) {
                  overlayElement.remove();
                }
                element.removeClass(lockClass);
              }
            }
          };
          HelpService.register(registerObject);         
        }
      }
    };
  }
])
.directive('lockForTutorial', [
  'HelpService',
  function (HelpService) {
    var lockClass = 'locked-for-tutorial';
    return {
      restrict: 'AC',
      link: function (scope, element) {
        if (HelpService.isActive) {
          var registerObject = {
            update: function(isActive) {
              if (isActive) {
                element.addClass(lockClass);                
              } else {
                element.removeClass(lockClass);
              }
            }
          };
          HelpService.register(registerObject);         
        }
      }
    };
  }
])
.directive('hideForTutorial', [
  'HelpService',
  function (HelpService) {
    var lockClass = 'ng-hide';
    return {
      restrict: 'AC',
      link: function (scope, element) {
        if (HelpService.isActive) {
          var registerObject = {
            update: function(isActive) {
              if (isActive) {
                element.addClass(lockClass);                
              } else {
                element.removeClass(lockClass);
              }
            }
          };
          HelpService.register(registerObject);         
        }
      }
    };
  }
])
.directive('highlightForTutorial', [
  'HelpService',
  function (HelpService) {
    var lockClass = 'highlight-for-tutorial';
    return {
      restrict: 'AC',
      link: function (scope, element) {
        if (HelpService.isActive) {
          var registerObject = {
            update: function(isActive) {
              if (isActive) {
                element.addClass(lockClass);                
              } else {
                element.removeClass(lockClass);
              }
            }
          };
          HelpService.register(registerObject);         
        }
      }
    };
  }
]);
/**
* ngdoc controller
* @name PaymentMenuButtonController
* @requires $scope
* @description write the description here ;)
*/
angular.module('blades.help').controller('HelpMenuButtonController', [
  '$scope',
  'HelpPopupService',
  function ($scope, HelpPopupService) {
    $scope.openHelp = function () {
      HelpPopupService.showPopup();
    };
  }
]);
/**
* @ngdoc directive
* @name settingsMenuButton
* @description settings card button on the bottom left bar
* @example
* Usage:
* <settings-card></settings-card>
*/
angular.module('blades.help').directive('helpMenuButton', [function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/help/menuButton/templates/helpMenuButtonTemplate.html',
      controller: 'HelpMenuButtonController'
    };
  }]);
angular.module('blades.matchReport', []);
angular.module('blades.matchReport').factory('MatchReportService', [
  function () {
    var MatchReportService = {};
    return MatchReportService;
  }
]);
'use strict';
angular.module('blades.matchReport').controller('MatchReportPopupController', [
  '$scope',
  '$stateParams',
  'clubsService',
  'fixturesService',
  'MatchReportService',
  'MatchReportPopupService',
  function ($scope, $stateParams, clubsService, fixturesService, MatchReportService, MatchReportPopupService) {
    $scope.fixture = fixturesService.get($stateParams.fixtureId);
    $scope.homeClub = clubsService.get($scope.fixture.homeClubId);
    $scope.awayClub = clubsService.get($scope.fixture.awayClubId);

    $scope.close = function() {
      MatchReportPopupService.close();
    };
  }
]);
angular.module('blades.matchReport').factory('MatchReportPopupService', [
  '$rootScope',
  'ModalService',
  function ($rootScope, ModalService) {
    var MatchReportPopupService = {};  
    

    MatchReportPopupService.show = function () {
      ModalService.show('app/blades/matchReport/matchReportPopup/template/matchReportPopupTemplate.html', 'MatchReportPopupController');
    };

    return MatchReportPopupService;
  }
]);
/**
* ngdoc controller
* @name PaymentMenuButtonController
* @requires $scope
* @description write the description here ;)
*/
angular.module('blades.matchReport').controller('MatchReportButtonController', [
  '$scope',
  '$stateParams',
  'clubsService',
  'fixturesService',
  'MatchReportPopupService',
  function ($scope, stateParams, clubsService, fixturesService, MatchReportPopupService) {
    if (stateParams.fixtureId) {
      $scope.fixture = fixturesService.get(stateParams.fixtureId);
      $scope.homeClub = clubsService.get($scope.fixture.homeClubId);
      $scope.awayClub = clubsService.get($scope.fixture.awayClubId);      
    }

    $scope.openMatchReport = function () {
      MatchReportPopupService.show();
    };
  }
]);
angular.module('blades.matchReport').directive('matchReportButton', [
  function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/matchReport/matchReportButton/templates/matchReportButtonTemplate.html',
      controller: 'MatchReportButtonController'
    };
  }
]);
angular.module('blades.settings', []).run([
  'AppBuilder',
  'ModalService',
  'NavigationService',
  function (AppBuilder, ModalService, NavigationService) {
    AppBuilder.registerMenu({
      menu: 'bottom',
      side: 'left',
      template: '<settings-menu-button></settings-menu-button>'
    });
    NavigationService.registerRoute('modal.settings', function(args) {
    	ModalService.show('app/blades/settings/modal/templates/settingsModalTemplate.html', 'SettingsModalController');
    });
  }
]);
/**
* ngdoc controller
* @name PaymentMenuButtonController
* @requires $scope
* @description write the description here ;)
*/
angular.module('blades.settings').controller('SettingsMenuButtonController', [
  '$scope',
  '$rootScope',
  'ModalService',
  function ($scope, $rootScope, ModalService) {
    $scope.onClick = function () {
    	ModalService.show('app/blades/settings/modal/templates/settingsModalTemplate.html', 'SettingsModalController');
    };

    $scope.showFBModal = function showFBModal() {
      if ($rootScope.user.facebook && $rootScope.user.facebook.id) {
        ModalService.show('/app/blades/user/modal/templates/facebookFriendsLeagueTemplate.html', 'UserFacebookLeagueModalController');
      }
    };
  }
]);

/**
* @ngdoc directive
* @name settingsMenuButton
* @description settings card button on the bottom left bar
* @example
* Usage:
* <settings-card></settings-card>
*/
angular.module('blades.settings').directive('settingsMenuButton', [function () {
    return {
      restrict: 'E',
      templateUrl: 'app/blades/settings/menuButton/templates/settingsMenuButtonTemplate.html',
      controller: 'SettingsMenuButtonController'
    };
  }]);
'use strict';
angular.module('blades.settings').controller('SettingsModalController', [
  '$scope',
  '$rootScope',
  'LanguageService',
  'UserSettings',
  'usersService',
  'languages',
  'ToastService',
  'AppBuilder',
  'faq',
  'enquiriesService',
  'TOAST',
  'club',
  '$timeout',
  'Urls',
  'COUNTRY_FLAG_RESOLVER',
  function ($scope, $rootScope, LanguageService, UserSettings, usersService, languages, ToastService, AppBuilder, faq, enquiriesService, TOAST, club, $timeout, Urls, COUNTRY_FLAG_RESOLVER) {

    // feedback
    $scope.feedback = '';
    var saveFeedback = function(card, feedback) {
      card.active = !card.active;
      $scope.active = !$scope.active;
      if (!feedback || feedback.length <= 0) {
        return;
      }

      var metaData = {
        device: ionic.Platform.ua,
        clubId: club.clubId,
        userId: $rootScope.user ? $rootScope.user.id : 'guess'
      };
      enquiriesService.create({
        type:'feedback',
        description: feedback + '\n' + JSON.stringify(metaData),
        subdomain: club.subdomain
      }).then(function(){
        ToastService.hideToast();
        feedback = '';
        ToastService.showNativeToast('Success! Thanks for your feedback.', '', TOAST.types.save);        
      }, function(err) {
        console.log(err);
      });
    }
    //terms
    var openTerms = function () {
      var termsUrl = 'http://' + club.subdomain + '.playabl.com/terms.php';
      if (window.cordova && cordova.InAppBrowser) {
        cordova.InAppBrowser.open(termsUrl, '_system');
      } else {
        window.open(termsUrl, 'blank');    
      }
    };
    // Push notifications
    var savePushNotification = function (card) {
      $scope.Settings.pushnotification = !$scope.Settings.pushnotification;
      card.icon = $scope.Settings.pushnotification ? 'ion-android-notifications' : 'ion-android-notifications-off';
      UserSettings.setData({'pushnotification':$scope.Settings.pushnotification});
      $rootScope.$broadcast('pushnotification-settings-changed');
      if ($rootScope.user && $rootScope.user.isAdmin) {
        UserSettings.setData({'firstTime':$scope.Settings.pushnotification});    
      }
    };
    // Sound
    var saveSound = function (card) {
      $scope.Settings.sounds = !$scope.Settings.sounds;
      card.icon = $scope.Settings.sounds ? 'ion-android-volume-up' : 'ion-android-volume-off';
      UserSettings.setData({'soundEnable':$scope.Settings.sounds});
    };
    // Language
    var saveLanguage = function (card, language) {
      if (language) {
        ToastService.showNativeToast('loading', '', TOAST.types.loading_progress, true);
        LanguageService.setLanguage(language.name).then(function () {
          $scope.currentLanguageObject = language;
          ToastService.setProgress(40);
          AppBuilder.rebuild(true, false).then(function() {
            ToastService.setProgress(60);
            if ($rootScope.user) {
              var update = {
                _request: {
                  'language': language.name
                }
              };
              ToastService.setProgress(80);
              usersService.update($rootScope.user.id, update).then(function () {
                $rootScope.user.language = language.name;
                ToastService.hideNativeToast();
              }, function (err) {
                ToastService.hideNativeToast();
                console.log(err);
              });          
            } else {
              ToastService.hideNativeToast();
            }
          }, function(err) {
            ToastService.hideNativeToast();
          });
        }, function (err) {
          ToastService.hideNativeToast();
          console.log(err);
        });
      } else {
        card.active = !card.active;
        $scope.active = !$scope.active;
      }
    };
    $scope.settingCards = [
      { title: 'NOTIFICATIONS', icon: UserSettings.getData('pushnotification') ? 'ion-android-notifications' : 'ion-android-notifications-off', active: false, cta: savePushNotification },
      { title: 'Sound', icon: UserSettings.getData('soundEnable') ? 'ion-android-volume-up' : 'ion-android-volume-off', active: false, cta: saveSound },
      { title: "FAQ'S", icon: 'ion-help', active: false },
      { title: 'TERMS', icon: 'ion-ios-book-outline', active: false, cta: openTerms },
      { title: 'FEEDBACK', icon: 'ion-chatboxes', active: false, cta: saveFeedback, template: 'app/blades/settings/modal/templates/feedbackCardTemplate.html' },
      { title: 'LANGUAGE', icon: 'ion-earth', active: false, cta: saveLanguage }
    ];
    $scope.active = false;
    $scope.activate = function(card, openCard) {
      if (card.cta) {
        card.cta(card);
      } else {
        card.active = !card.active;
        $scope.active = !$scope.active;        
      }
      if (openCard) {
        for (var i = $scope.settingCards.length - 1; i >= 0; i--) {
          if ($scope.settingCards[i].title == openCard) {
            $scope.activate($scope.settingCards[i]);
          }
        }
      }
    };

    $scope.ready = false;
    $timeout(function() {
      $scope.ready = true;
    });



    // By default, display settings
    $scope.actualView = "settings";
    $scope.version = club.appVersion;
    $scope.close = function () {
      if ($scope.active) {
        for (var i = $scope.settingCards.length - 1; i >= 0; i--) {
          if ($scope.settingCards[i].active == true) {
            $scope.activate($scope.settingCards[i]);
            break;
          }
        };
      } else {
        $scope.closeModal();        
      }
    };
    // ng-model object
    $scope.Settings = {
      sounds: UserSettings.getData('soundEnable'),
      pushnotification: UserSettings.getData('pushnotification'),
      languages: UserSettings.getData('languages')
    };

    $scope.faq = faq;

    // Languages for the dropdown
    $scope.languages = languages;

    var countryCode;
    for (var key in $scope.languages) {
      var shortCode = $scope.languages[key].name.slice(-2).toLowerCase();
      var countryCode =
          COUNTRY_FLAG_RESOLVER[shortCode] ? COUNTRY_FLAG_RESOLVER[shortCode] : shortCode;

      $scope.languages[key].flagImageUrl = Urls.urlRemoteBuilder('countries/' + countryCode + '/flags/28x14/main.png');
    }
    // current language to select the dropdown
    $scope.currentLanguage = LanguageService.current;
    // current language object for the view
    $scope.currentLanguageObject = languages[LanguageService.current];

    // lanauge to be bound to the model
    $scope.Settings.language = $scope.currentLanguage;


  }
]);

angular.module('blades.settings')
.directive('feedbackForm', [
  function () {
    return {
      name: 'feedbackForm',
      restrict: 'AE',
      templateUrl: 'app/blades/settings/modal/templates/feedbackCardTemplate.html'
    };
  }
])
.directive('languagesForm', [
  function () {
    return {
      name: 'languagesForm',
      restrict: 'AE',
      templateUrl: 'app/blades/settings/modal/templates/languagesCardTemplate.html'
    };
  }
])
.directive('faqForm', [
  function () {
    return {
      name: 'faqForm',
      restrict: 'AE',
      templateUrl: 'app/blades/settings/modal/templates/faqCardTemplate.html'
    };
  }
]);

/**
* Truncate Filter
* @Param text
* @Param length, default is 10
* @Param end, default is "..."
* @return string
*/
angular.module('shell.filters').filter('truncate', function () {
    return function (text, length, end) {
        if (isNaN(length)) {
            length = 10;
        }
        if (end === undefined) {
            end = '...';
        }
        if (text.length <= length || text.length - end.length <= length) {
            return text;
        } else {
            return String(text).substring(0, length-end.length) + end;
        }
    };
});
